/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.dataobjects;

import java.awt.Color;
import java.awt.Font;
import java.awt.Image;
import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Random;
import java.util.Set;
import java.util.Vector;

import javax.media.j3d.Appearance;
import javax.media.j3d.Billboard;
import javax.media.j3d.BoundingBox;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Geometry;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.Group;
import javax.media.j3d.Link;
import javax.media.j3d.Material;
import javax.media.j3d.PolygonAttributes;
import javax.media.j3d.Shape3D;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.media.j3d.TriangleArray;
import javax.swing.JOptionPane;
import javax.vecmath.Color3f;
import javax.vecmath.Matrix3f;
import javax.vecmath.Point3d;
import javax.vecmath.Point3f;
import javax.vecmath.Vector2f;
import javax.vecmath.Vector3f;

import org.cellmicrocosmos.cm2.coarsegrainsupport.MoleculeProfile;
import org.cellmicrocosmos.cm2.coarsegrainsupport.Molecule_Profile_IO;
import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.functionality3d.Atom_Creator_3D;
import org.cellmicrocosmos.cm2.functionality3d.SelectionBox;
import org.cellmicrocosmos.cm2.io.PdbReader_3D;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.Molecule;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;

import com.sun.j3d.utils.behaviors.mouse.MouseRotate;
import com.sun.j3d.utils.geometry.Box;
import com.sun.j3d.utils.geometry.GeometryInfo;
import com.sun.j3d.utils.geometry.NormalGenerator;
import com.sun.j3d.utils.geometry.Text2D;

/**
 * This parent class represents the molecules used in the model. Extends a
 * shape_3d, so the objects itself are displayable
 * 
 * @author Tim Dingersen dackelnose@gmx.net, Philipp Unruh, Bjoern Sommer bjoern@CELLmicrocosmos.org
 * 
 */
public abstract class Molecule_3D {

	public Molecule teporaryMolecule = null;

	// removed SOURCE
	private String title, PDBcode, date, header, author;

	private String pdbFile;

	private Point3f position = new Point3f();

	private Color color;

	private Matrix3f rotationMatrix;

	private float yTranslation = 0;

	private float pdbtmTranslation = 0;

	private Matrix3f pdbtmRotation = Cm2_Toolkit.createAUnitMatrix();

	private Point3f[] trianglePoints;

	private float atomicMass = -1;

	private int currentModelIndex = -1;

	private Point3f[] alignedTPs = null;

	private Atom_3D[] alignedAtoms = null;

	private BoundingBox alignedBounds;

	private String[] chainIDs = {};

	private int domainID = 0;

	boolean needToRecalculateImages = true;

	private Box unrotatedBounds = null;

	public Atom_3D[] atoms = null;

	public Atom_3D[] allAtomsSorted = null;

	public float maxAtomRadius = -1f;

	private int modelcount = 0;

	public MouseRotate behavior;

	private int layerID = 0;

	private final TransformGroup transformgroup = new TransformGroup();

	Transform3D t3D = new Transform3D();

	private final BranchGroup branchGroup = new BranchGroup();

	private final Shape3D shape = new Shape3D();

	private Vector<Constraint_3D> activeConstraints = null;

	public Constraint_3D[] originalConstraints = null;

	public Vector<Particle> currentCGParticles = new Vector<Particle>();

	private int renderMode = CM2_CONSTANTS.RENDERMODE.RENDER_MODE_POLYGON;

	private boolean positionisinavalidspace = true;

	private boolean isPicked = false;

	private boolean isTransparent = false;

	private Image smallImage;

	private Image largeImage;

	public Point3f positionAfterReading = null;

	public Atom_3D[] opmAtoms = new Atom_3D[0];

	private boolean isDisplayingOPMAtoms = false;

	private boolean useAtomColors = true;

	public int terCount = 0;

	private String currentProfileName = "Default Profile";

	public final TransformGroup additionalAtomInfoTG = new TransformGroup();

	private float originalHeight;

	/**
	 * Creates a new molecule with a default matrix and all nessecary
	 * capabilities set
	 * 
	 */
	public Molecule_3D() {

		this.rotationMatrix = Cm2_Toolkit.createAUnitMatrix();

		Cm2_Toolkit.setAllCapabilitysForShape3D(this.shape);

		Cm2_Toolkit.setAllCapabilitysForTransformGroup(this.transformgroup);

		Cm2_Toolkit.setAllCapabilitysForTransformGroup(this.additionalAtomInfoTG);

		Cm2_Toolkit.setAllCapabilitysForBranchGroup(this.branchGroup);

		this.shape.setUserData(this);
		this.branchGroup.setUserData(this);
		this.transformgroup.addChild(this.shape);
		this.transformgroup.addChild(additionalAtomInfoTG);
		this.transformgroup.setTransform(this.t3D);

		this.branchGroup.addChild(this.transformgroup);

		final Appearance appear = new Appearance();

		final PolygonAttributes pattrib = new PolygonAttributes();
		pattrib.setPolygonMode(DataPool.J3DSettings.polygon_mode);
		pattrib.setCullFace(PolygonAttributes.CULL_NONE);
		appear.setPolygonAttributes(pattrib);

		Cm2_Toolkit.setRenderingAttributes(appear);

		final Material mat = new Material();
		mat.setLightingEnable(true);

		appear.setMaterial(mat);
		this.shape.setAppearance(appear);

		appear.setCapability(Appearance.ALLOW_MATERIAL_READ);
		appear.setCapability(Appearance.ALLOW_MATERIAL_WRITE);

		appear.setCapability(Appearance.ALLOW_COLORING_ATTRIBUTES_READ);
		appear.setCapability(Appearance.ALLOW_COLORING_ATTRIBUTES_WRITE);

		appear.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_READ);
		appear.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_WRITE);
		appear.setCapability(Appearance.ALLOW_POLYGON_ATTRIBUTES_READ);

		pattrib.setCapability(PolygonAttributes.ALLOW_MODE_WRITE);

		mat.setCapability(Material.ALLOW_COMPONENT_WRITE);
		mat.setCapability(Material.ALLOW_COMPONENT_READ);

		this.shape.setPickable(true);

		if (this instanceof Protein_3D)
			mat.setShininess(DataPool.J3DSettings.shininess_proteins);
		else
			mat.setShininess(DataPool.J3DSettings.shininess_lipids);

	}

	/**
	 * @return Returns the active constraints.
	 */
	public Vector<Constraint_3D> getActiveConstraints() {
		if (this.activeConstraints == null) {
			this.activeConstraints = new Vector<Constraint_3D>();
		}
		return activeConstraints;
	}

	/**
	 * @return Returns the description of the Protein.
	 */
	public String getHeader() {
		return this.header;
	}

	/**
	 * Sets the description of the Protein.
	 */
	public void setHeader(final String description) {
		this.header = description;
	}

	/**
	 * @return Returns the name of the Protein.
	 */
	public String getTitle() {
		return this.title;
	}

	/**
	 * Sets the description of the Protein.
	 */
	public void setTitle(final String title) {
		this.title = title;
	}

	/**
	 * Returns the author of this pdb file
	 * 
	 * @return
	 */
	public String getAuthor() {
		return this.author;
	}

	/**
	 * Sets the author of the Ppdb file.
	 */
	public void setAuthor(final String author) {
		this.author = author;
	}

	/**
	 * Returns the Position of this instance in the membrane All values are
	 * anstroms
	 */
	public Point3f getPosition() {
		if (this.position == null)
			return null;

		return this.position;
	}

	public float getX() {
		if (this.position != null) {
			return this.position.x;
		} else
			return -1;
	}

	public float getY() {
		if (this.position != null) {

			return this.position.y;
		} else
			return -1;
	}

	public float getZ() {
		if (this.position != null) {
			return this.position.z;
		} else
			return -1;
	}

	/**
	 * Sets the Position of the instance of the Protein in the membrane
	 */
	public void setPosition(final float x, final float z) {

		this.position = new Point3f(x, this.yTranslation, z);
		this.alignedTPs = null;
	}

	public void move() {

		this.t3D.setRotation(this.getRotationMatrix());
		final Point3f pos = new Point3f(this.getPosition());
		pos.scale(CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
		this.t3D.setTranslation(new Vector3f(pos));
		try {
			this.transformgroup.setTransform(this.t3D);
		} catch (Exception e) {
			System.out.println(e);
		}

	}

	public void setTransform(Transform3D t3d) {
		this.t3D = t3d;
		this.transformgroup.setTransform(t3D);
	}

	/**
	 * @return Returns the date
	 */
	public String getDate() {
		return this.date;
	}

	/**
	 * Sets the date
	 */
	public void setDate(final String date) {
		this.date = date;
	}

	/**
	 * @return Returns the PDBcode
	 */
	public String getPDBcode() {
		return this.PDBcode;
	}

	/**
	 * Sets the PDBcode
	 */
	public void setPDBcode(final String bcode) {
		if (bcode == null)
			this.PDBcode = null;
		else
			this.PDBcode = bcode;
		// else this.PDBcode=Cm2_Toolkit.extendRight(bcode,4);

	}

	/**
	 * @return Returns the sOURCE.
	 */
	public String getSOURCE() {
		// Clean
		return DataPool.moleculeSources.get(this.PDBcode);
	}

	/**
	 * @param source
	 *            The SOURCE to set.
	 */
	public void setSOURCE(final String source) {
		// Clean
		DataPool.moleculeSources.put(this.PDBcode, source);
	}

	/**
	 * Returns the pdb file this molecule was created from
	 * 
	 * @return
	 */
	public File getPDBFile() {
		return DataPool.moleculesPdbFiles.get(this.pdbFile);
	}

	/**
	 * Sets this molecule's pdb file
	 * 
	 * @param pdbFile
	 */
	public void setPDBFile(final File pdbFile) {
		this.pdbFile = pdbFile.getAbsolutePath().intern();
		DataPool.moleculesPdbFiles.put(pdbFile.getAbsolutePath().intern(), pdbFile);
	}

	/**
	 * Returns the currently used color for this object
	 */
	public Color getColor() {
		return this.color;
	}

	/**
	 * Sets the color for this object. The color of the shape will be updated
	 * instantly
	 * 
	 * @param color
	 */
	public void setColor(final Color color) {
		this.color = color;
		final Color3f c3f = new Color3f(color);

		this.shape.getAppearance().getMaterial().setAmbientColor(c3f);

	}

	/**
	 * returns the current rotation matrix of this molecule relative to the
	 * coordinates of the pdb file this instance uses
	 * 
	 * @return
	 */
	public Matrix3f getRotationMatrix() {
		return new Matrix3f(this.rotationMatrix);
	}

	/**
	 * Sets the rotaion matrix for this object
	 * 
	 * @param rotationMatrix
	 */
	public void setRotationMatrix(final Matrix3f rotationMatrix) {
		this.rotationMatrix = rotationMatrix;
		this.needToRecalculateImages = true;
		this.alignedTPs = null;

	}

	/**
	 * Returns the current Y-height of this molecule in angstrom
	 * 
	 * @return
	 */
	public float getYTranslation() {
		return this.yTranslation;

	}

	/**
	 * Returns the y-coordinate of the molecule's bounding center after
	 * transforming by the PPE.<br>
	 * <br>
	 * Value {@code 0} by default.
	 * 
	 */
	public float getPDBTMTranslation() {
		// important:
		// return a new object instead of a reference
		return new Float(this.pdbtmTranslation);
	}

	/**
	 * Returns the molecule's PDBTM rotation matrix.<br>
	 * <br>
	 * Returns a {@code identity matrix} by default.
	 * 
	 */
	public Matrix3f getPDBTMRotation() {
		// important:
		// return a new object instead of a reference
		return new Matrix3f(this.pdbtmRotation);
	}

	/**
	 * Sets the current Y-translation in angstrom for this object. Will also
	 * update the y-value of the molecule's position point
	 * 
	 * @param translation
	 */
	public void setYTranslation(final float translation) {
		this.yTranslation = translation;

		if (this.position != null)
			this.position.y = translation;

		this.alignedTPs = null;
	}

	/**
	 * Sets the y-coordinate of the molecules bounding center after transforming
	 * by the PPE.<br>
	 * Parameter {@code center} has to be a number.
	 * 
	 * @param center
	 */
	public void setPDBTMTranslation(final float center) {
		if (!Float.isNaN(center)) {
			this.pdbtmTranslation = center;
		}
	}

	/**
	 * Sets the PDBTM rotation matrix extracted from a PDBTM XML file.<br>
	 * Parameter {@code rotation} has to be not null.
	 * 
	 * @param rotation
	 */
	public void setPDBTMRotation(final Matrix3f rotation) {
		if (rotation != null) {
			this.pdbtmRotation = rotation;
		}
	}

	/**
	 * Creates a new Protein / Lipid and copies all of its data. Not exactly
	 * equivalent to usual cloning
	 */
	public Molecule_3D copy(final boolean cloneAlignment, final boolean resetAtomPositions) {

		Molecule_3D newone;
		if (this instanceof Protein_3D)
			newone = new Protein_3D();
		else
			newone = new Lipid_3D();

		newone.setAuthor(this.getAuthor());
		newone.setHeader(this.getHeader());
		newone.setTitle(this.getTitle());
		newone.setDate(this.getDate());
		newone.setSOURCE(this.getSOURCE());
		newone.setPDBFile(this.getPDBFile());
		newone.setPDBcode(this.getPDBcode());
		newone.setModelSet(this.getModelSet());
		newone.setDomainID(this.getDomainID());
		newone.setLayerID(this.getLayerID());
		newone.setModelcount(this.getModelcount());
		newone.chainIDs = this.chainIDs;
		newone.setAtomicMass(this.getAtomicMass());
		newone.maxAtomRadius = this.maxAtomRadius;
		newone.terCount = this.terCount;
		newone.setCurrentProfileName(this.getCurrentProfileName());

		// in combination with:
		// - set/getPDBTMTranslation()
		// - set/getPDBTMRotation()
		// =============================================================
		// TODO: PPE Update 03_2012										|
		// =============================================================
		newone.setPDBTMTranslation(this.getPDBTMTranslation());
		newone.setPDBTMRotation(this.getPDBTMRotation());

		// newone.setCenter_unscaled(this.getCenter_unscaled());
		newone.setUnrotatedBounds(new Box(this.unrotatedBounds.getXdimension(),
				this.unrotatedBounds.getYdimension(), this.unrotatedBounds.getZdimension(), null));
		if (cloneAlignment)
			newone.setRotationMatrix(Cm2_Toolkit.copyMatrix(this.getRotationMatrix()));

		if (cloneAlignment)
			newone.setYTranslation(this.getYTranslation());

		if (cloneAlignment)
			newone.setPosition(this.getX(), this.getZ());

		final Point3f[] newpoints = new Point3f[this.getOriginalTrianglePoints().length];
		for (int i = 0; i < this.getOriginalTrianglePoints().length; i++) {
			newpoints[i] = new Point3f(this.getOriginalTrianglePoints()[i]);
		}
		newone.setTrianglePoints(newpoints);

		final TriangleArray geo = new TriangleArray(newpoints.length, GeometryArray.NORMALS
				| GeometryArray.COORDINATES | GeometryArray.COLOR_3);

		final GeometryInfo gi = new GeometryInfo(GeometryInfo.TRIANGLE_ARRAY);

		gi.setCoordinates(newpoints);

		new NormalGenerator().generateNormals(gi);

		geo.setCoordinates(0, newpoints);
		geo.setNormals(0, gi.getNormals());

		geo.setCapability(Geometry.ALLOW_INTERSECT);

		newone.getShape().setGeometry(geo);

		this.shape.setAppearance((Appearance) this.shape.getAppearance().cloneNodeComponent(true));
		if (this.getColor() != null)
			newone.setColor(this.getColor());

		//	

		Vector<Atom_3D> v = new Vector<Atom_3D>();
		newone.allAtomsSorted = new Atom_3D[this.allAtomsSorted.length];

		for (int i = 0; i < this.allAtomsSorted.length; i++) {
			v.addElement(this.allAtomsSorted[i]);
			this.allAtomsSorted[i].indexForPDBWriter = i;
			newone.allAtomsSorted[i] = this.allAtomsSorted[i].copy();

		}

		newone.atoms = new Atom_3D[this.atoms.length];

		for (int i = 0; i < this.atoms.length; i++) {
			// regular atoms
			if (v.contains(atoms[i])) {
				newone.atoms[i] = newone.allAtomsSorted[atoms[i].indexForPDBWriter];
				// coarse-grained particles
			} else {
				Vector<Atom_3D> w = new Vector<Atom_3D>();
				Particle p = (Particle) atoms[i];

				for (int j = 0; j < p.includedAtoms.size(); j++) {
					Atom_3D a = p.includedAtoms.get(j);
					if (v.contains(a)) {
						w.addElement(newone.allAtomsSorted[a.indexForPDBWriter]);

					}
				}
				Particle newp = new Particle(w.toArray(new Atom_3D[0]));
				newp.color = p.color;
				newp.radiusCov = p.radiusCov;
				newp.radiusVDW = p.radiusVDW;
				newp.name = p.name;
				newp.particleIdentifier = p.particleIdentifier;
				newp.color = p.color;
				newp.comparator = p.comparator;
				// newp.userName2 = p.userName2;
				newone.atoms[i] = newp;
				newone.currentCGParticles.addElement(newp);
			}
		}

		newone.originalConstraints = new Constraint_3D[this.originalConstraints.length];
		for (int i = 0; i < this.originalConstraints.length; i++) {
			Atom_3D a = newone.allAtomsSorted[this.originalConstraints[i].atomA.indexForPDBWriter];
			Atom_3D b = newone.allAtomsSorted[this.originalConstraints[i].atomB.indexForPDBWriter];
			newone.originalConstraints[i] = new Constraint_3D(a, b);
			// newone.originalConstraints[i].originalLength=this.originalConstraints[i].originalLength;
		}
		for (int i = 0; i < this.allAtomsSorted.length; i++) {
			this.allAtomsSorted[i].indexForPDBWriter = null;
		}

		if (resetAtomPositions) {
			MoleculeProfile prof;
			try {
				String code = this.getPDBcode();
				if (code == null || code.length() < 1)
					code = Cm2_Toolkit.getSimpleName(this.getPDBFile());
				prof = Molecule_Profile_IO.readProfile(this, this.currentProfileName, code);
				// newone.applyProfile(prof);
				for (int i = 0; i < prof.positions.length; i++) {
					newone.allAtomsSorted[i].position = prof.positions[i];
				}
				for (int i = 0; i < newone.currentCGParticles.size(); i++) {
					Particle p = newone.currentCGParticles.get(i);
					MyBounds mb = new MyBounds(p.includedAtoms.toArray(new Atom_3D[0]));
					p.setPosition(mb.center);
				}
			} catch (Exception e) {

				e.printStackTrace();
			}

		}
		newone.setRenderMode(this.renderMode);
		newone.updateAllBonds();

		newone.opmAtoms = new Atom_3D[this.opmAtoms.length];

		for (int i = 0; i < this.opmAtoms.length; i++) {
			newone.opmAtoms[i] = this.opmAtoms[i].copy();
		}

		if (this instanceof Protein_3D) {

			((Protein_3D) newone).setID(((Protein_3D) this).getID());

		} else {
			((Lipid_3D) newone).setPercent(((Lipid_3D) this).getPercent());
			((Lipid_3D) newone).setMembraneSide(((Lipid_3D) this).getMembraneSide());

		}

		if (cloneAlignment)
			this.realignTPs();
		return newone;
	}

	/**
	 * Returns a string representation of this object for debugging
	 */
	public String toString() {
		String type;

		if (this instanceof Protein_3D)
			type = "PROTEIN ";
		else
			type = "LIPID ";

		String s = type + this.getPDBcode() + " ; ";

		s += this.getPosition().toString();
		return s;

	}

	/**
	 * Sets the triangle points for this object. Happens after construction is
	 * finished
	 * 
	 * @param point3fs
	 */
	public void setTrianglePoints(final Point3f[] point3fs) {
		this.trianglePoints = point3fs;

	}

	/**
	 * returns the original, unaligned trianglepoints
	 * 
	 * @return
	 */
	public Point3f[] getOriginalTrianglePoints() {
		return this.trianglePoints;
	}

	public void setOriginalHeight(float height) {
		this.originalHeight = height;
	}

	public float getOriginalHeight() {
		return originalHeight;
	}

	/**
	 * Returns the copy of the original trianglepoints of this object computed
	 * to the correct alignment regarding to the translation and rotation of
	 * this molecule. If any changes occured to the position point or the
	 * rotation matrix, these points will be recalculated when calling this
	 * method first. If this method gets called another time, and no changes
	 * were made meanwhile, it simply returns the already computed triangle
	 * points That way, this method may be called whenever needed, without
	 * bothering about costs for computation. The returned points will always be
	 * up to date.
	 * 
	 * @return
	 */
	public Point3f[] getAlignedTrianglePoints() {
		if (this.alignedTPs == null)
			this.realignTPs();
		return this.alignedTPs;
	}

	public Atom_3D[] getAlignedAtoms() {
		if (this.alignedAtoms == null)
			this.realignTPs();
		return this.alignedAtoms;
	}

	/**
	 * Returns the current bounding box of this molecule. Like in the methods
	 * above, it is ensured that the data that is returned is up to date
	 * 
	 * @return
	 */
	public BoundingBox getAlignedGeometricBounds() {
		if (this.alignedTPs == null)
			this.realignTPs();
		final BoundingBox bb = (BoundingBox) this.alignedBounds.clone();
		// if (!DataPool.J3DSettings.atomCollisionCheck)
		// return bb;

		final Point3d upper = new Point3d();
		bb.getUpper(upper);
		final Point3d lower = new Point3d();
		bb.getLower(lower);

		final float maxRad = (DataPool.J3DSettings.atomCollisionCheck && !DataPool.J3DSettings.atomDistanceAbsolute) ? this.maxAtomRadius
				* CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE
				: 0;
		//		final float maxRad = (DataPool.J3DSettings.atomDistanceAbsolute)?0:this.maxAtomRadius * CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE; 
		//		System.out.println("maxRad "+maxRad+" DataPool.J3DSettings.atomDistanceAbsolute "+DataPool.J3DSettings.atomDistanceAbsolute);

		bb.setUpper(upper.x + maxRad, upper.y + maxRad, upper.z + maxRad);
		bb.setLower(lower.x - maxRad, lower.y - maxRad, lower.z - maxRad);
		return bb;
	}

	/**
	 * Returns the upper corner of the current boundingbox. If the passed
	 * boolean is set true, then the returned value will have angstrom scale,
	 * otherwise, the internal scale will be returned
	 * 
	 * @param scaledToAngstrom
	 * @return
	 */
	public Point3f getUpper(final boolean scaledToAngstrom) {
		final Point3d p = new Point3d();
		this.getAlignedGeometricBounds().getUpper(p);
		if (scaledToAngstrom) {
			final Point3f f = new Point3f(p);
			f.scale(1 / CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
			return f;
		} else
			return new Point3f(p);
	}

	/**
	 * Returns the lower corner of the current boundingbox. If the passed
	 * boolean is set true, then the returned value will have angstrom scale,
	 * otherwise, the internal scale will be returned
	 * 
	 * @param scaledToAngstrom
	 * @return
	 */
	public Point3f getLower(final boolean scaledToAngstrom) {
		final Point3d p = new Point3d();
		this.getAlignedGeometricBounds().getLower(p);
		if (scaledToAngstrom) {
			final Point3f f = new Point3f(p);
			f.scale(1 / CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
			return f;
		} else
			return new Point3f(p);
	}

	/**
	 * realigns the triangle points and the bounding box according to the
	 * current position and rotation
	 * 
	 */
	private void realignTPs() {

		Point3f position = new Point3f(this.getPosition());

		position.scale(CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);

		final int length = this.trianglePoints.length;
		this.alignedTPs = new Point3f[length];
		for (int i = 0; i < length; i++) {
			this.alignedTPs[i] = Cm2_Toolkit.transformToNewPoint(this.getRotationMatrix(),
					new Point3f(this.trianglePoints[i]));

		}

		if (this.alignedAtoms == null) {
			this.alignedAtoms = new Atom_3D[this.atoms.length];
			for (int i = 0; i < this.atoms.length; i++) {
				this.alignedAtoms[i] = this.atoms[i].copy();
			}
		}
		final int length2 = this.alignedAtoms.length;
		for (int i = 0; i < length2; i++) {

			this.alignedAtoms[i].position = Cm2_Toolkit.transformToNewPoint(
					this.getRotationMatrix(), new Point3f(this.atoms[i].position));

		}

		for (int i = 0; i < length; i++) {
			this.alignedTPs[i] = new Point3f(this.alignedTPs[i].x + position.x,
					this.alignedTPs[i].y + position.y, this.alignedTPs[i].z + position.z);

		}

		position = this.getPosition();
		float maxx = -1000000, maxy = -1000000, maxz = -1000000, minx = 1000000, miny = 1000000, minz = 1000000;
		for (int i = 0; i < length2; i++) {
			this.alignedAtoms[i].position = new Point3f(this.alignedAtoms[i].position.x
					+ position.x, this.alignedAtoms[i].position.y + position.y,
					this.alignedAtoms[i].position.z + position.z);

			minx = Math.min(minx, this.alignedAtoms[i].position.x);
			miny = Math.min(miny, this.alignedAtoms[i].position.y);
			minz = Math.min(minz, this.alignedAtoms[i].position.z);

			maxx = Math.max(maxx, this.alignedAtoms[i].position.x);
			maxy = Math.max(maxy, this.alignedAtoms[i].position.y);
			maxz = Math.max(maxz, this.alignedAtoms[i].position.z);
		}

		minx *= CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;
		miny *= CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;
		minz *= CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;
		maxx *= CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;
		maxy *= CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;
		maxz *= CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;

		this.alignedBounds = new BoundingBox(new Point3d(minx, miny, minz),
				new Point3d(maxx, maxy, maxz));

		this.needToRecalculateImages = true;
	}

	/**
	 * Returns the atomcount of this molecule
	 * 
	 * @return
	 */
	public int getAtomCount() {
		return this.atoms.length;

	}

	/**
	 * Returns the index of the model this molecule is constructed from. Only
	 * used if the underlying pdb file has multiple models
	 * 
	 * @return
	 */
	public int getModelSet() {

		return this.currentModelIndex;
	}

	/**
	 * Sets the model index this molecule was constructed from
	 * 
	 * @param model
	 */
	public void setModelSet(final int model) {

		this.currentModelIndex = model;
	}

	/**
	 * Allows to realign the trianglePoints and the boundingbox manually.
	 * Happens when a membrane gets read to have all data ready when displaying
	 * starts
	 * 
	 */
	public void recalculatePoints() {
		this.realignTPs();

	}

	/**
	 * Returns the count of chains in the underlying pdb file
	 * 
	 * @return
	 */
	public int getChainCount() {
		return this.chainIDs.length;
	}

	/**
	 * Sets the characters of the chain ids used in the underlying pdb file.
	 * Those are used later in the pdb writer
	 * 
	 * @param name
	 */
	public void setChainIDs(final Set<String> name) {
		this.chainIDs = name.toArray(new String[0]);

	}

	/**
	 * Returns all chain ids used in the pdb file
	 * 
	 * @return
	 */
	public String[] getChainIDs() {
		return this.chainIDs;

	}

	/**
	 * Returns a copy of all atoms for this instance in the right alignment.
	 * Those atoms are retrieved from the datapool and aligned afterwards.
	 * Aligning all these atoms may be expensive, dependent on the count of
	 * atoms, so this method should only be called when really needed
	 * 
	 * @return
	 */
	public Atom_3D[] getAtoms(final boolean aligned, final boolean copy) {

		if (this.atoms == null)
			return new Atom_3D[] {};

		Atom_3D[] ret;

		if (aligned) {
			if (this.alignedTPs == null)
				this.realignTPs();
			ret = this.alignedAtoms;
		} else
			ret = this.atoms;

		if (copy) {
			final Atom_3D[] newatoms = new Atom_3D[ret.length];

			for (int i = 0; i < ret.length; i++) {

				newatoms[i] = ret[i].copy();

			}

			ret = newatoms;
		}

		return ret;
	}

	//	/**
	//	 * Same as getAtoms, only that the returned atoms are not translated (but
	//	 * rotated) Their center should be zero at all 3 axis.
	//	 * 
	//	 * @return
	//	 */
	//	public Atom_3D[] getUntranslatedRotatedAtoms() {
	//
	//		if (this.atoms == null)
	//			return new Atom_3D[] {};
	//
	//		final Atom_3D[] newatoms = new Atom_3D[this.atoms.length];
	//
	//		for (int i = 0; i < this.atoms.length; i++) {
	//
	//			newatoms[i] = this.atoms[i].copy();
	//			newatoms[i].position = Cm2_Toolkit.transformToNewPoint(this
	//					.getRotationMatrix(), newatoms[i].position);
	//
	//		}
	//
	//		return newatoms;
	//	}

	public void setDomainID(final int id) {
		this.domainID = id;
	}

	public int getDomainID() {
		return this.domainID;
	}

	public float getAtomicMass() {

		if (this.atomicMass < 0)
			this.atomicMass = DataPool.AtomValues.calculateAtomicMass(this);
		return this.atomicMass;
	}

	public void setAtomicMass(final float mass) {
		this.atomicMass = mass;
	}

	public void setMouseBehavior(final MouseRotate behavior) {
		this.behavior = behavior;
	}

	public MouseRotate getMouseBehavior() {
		return this.behavior;
	}

	public TransformGroup getTransformGroup() {
		return this.transformgroup;
	}

	public boolean equalsType(final Molecule_3D other) {
		if (!((this instanceof Protein_3D && other instanceof Molecule_3D) || (this instanceof Lipid_3D && other instanceof Lipid_3D)))
			return false;

		if (this.getPDBcode().equals(other.getPDBcode()) == false)
			return false;
		return true;
	}

	public float calculateRadius() {
		final Atom_3D[] points = this.getAlignedAtoms();

		float max = 0f;
		final Vector2f orig = new Vector2f(this.position.x, this.position.z);
		for (int i = 0; i < points.length; i++) {
			final Vector2f v = new Vector2f(points[i].position.x + points[i].radiusCov,
					points[i].position.z + points[i].radiusCov);
			v.sub(orig);

			max = Math.max(max, Math.abs(v.length()));
		}
		return max * CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;
	}

	/**
	 * Sets and applies the specified render mode (see contants fields) if the
	 * objects are currently live, else it does nothing.
	 * 
	 */
	public void setRenderMode(final Integer renderMode) {
		if (renderMode != null && renderMode == this.renderMode) {
			return;
		}

		if (renderMode != null)
			this.renderMode = renderMode.intValue();
		Group parent;

		parent = (Group) this.branchGroup.getParent();

		if (parent != null) {
			try {
				parent.removeChild(this.branchGroup);
			} catch (Exception e) {
				JOptionPane.showMessageDialog(null,
						"An error occurred: \n"+e.getMessage()+"\nPlease re-open the alignment mode.",
						"Error", JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			}
		}

		if (!DataPool.J3DSettings.enableAtomPerformanceMode) {
		// we do not need to add the branchgroup because of the use of sharedgroups
			for (int i = 0; i < this.atoms.length; i++) {
				if (this.atoms[i].branchGroup != null)
					this.transformgroup.removeChild(this.atoms[i].branchGroup);
				this.atoms[i].branchGroup = null;
			}
		}

		final Enumeration<?> en = this.transformgroup.getAllChildren();

		while (en.hasMoreElements()) {
			Object o = en.nextElement();
			if (o instanceof BranchGroup && ((BranchGroup) o).getChild(0) instanceof Shape3D) {
				this.transformgroup.removeChild((BranchGroup) o);
				// o = null;
				// Remove the link to the sharedgroup
			} else if (o instanceof Link) {
				this.transformgroup.removeChild((Link) o);
			}
		}

		if (this.renderMode == CM2_CONSTANTS.RENDERMODE.RENDER_MODE_POLYGON) {

			this.shape.getAppearance().getPolygonAttributes()
					.setPolygonMode(DataPool.J3DSettings.polygon_mode);

		} else {

			this.shape.getAppearance().getPolygonAttributes()
					.setPolygonMode(PolygonAttributes.POLYGON_POINT);
			Atom_Creator_3D.setUpMolecule(this, this.renderMode);

			if (!DataPool.J3DSettings.enableAtomPerformanceMode) {
			// we do not need to add the branchgroup because of the use of sharedgroups
				for (int i = 0; i < this.atoms.length; i++) {
					if (this.atoms[i].branchGroup != null)
						this.transformgroup.addChild(this.atoms[i].branchGroup);
				}
			}

		}

		if (parent != null)
			parent.addChild(this.branchGroup);
	}

	public void updateAtomColors() {

		Group parent;

		parent = (Group) this.branchGroup.getParent();

		if (parent != null)
			parent.removeChild(this.branchGroup);

		for (int i = 0; i < this.atoms.length; i++) {
			if (this.atoms[i].branchGroup != null)
				this.transformgroup.removeChild(this.atoms[i].branchGroup);
			this.atoms[i].branchGroup = null;

		}

		final Enumeration<?> en = this.transformgroup.getAllChildren();

		while (en.hasMoreElements()) {
			Object o = en.nextElement();
			if (o instanceof BranchGroup && ((BranchGroup) o).getChild(0) instanceof Shape3D) {
				this.transformgroup.removeChild((BranchGroup) o);
				// o = null;
			}
		}

		if (this.renderMode == CM2_CONSTANTS.RENDERMODE.RENDER_MODE_POLYGON) {

			this.shape.getAppearance().getPolygonAttributes()
					.setPolygonMode(DataPool.J3DSettings.polygon_mode);

		} else {

			this.shape.getAppearance().getPolygonAttributes()
					.setPolygonMode(PolygonAttributes.POLYGON_POINT);
			Atom_Creator_3D.setUpMolecule(this, this.renderMode);

			for (int i = 0; i < this.atoms.length; i++) {

				if (this.atoms[i].branchGroup != null)
					this.transformgroup.addChild(this.atoms[i].branchGroup);

			}

		}

		if (parent != null)
			parent.addChild(this.branchGroup);

		for (int i = 0; i < this.activeConstraints.size(); i++) {
			this.activeConstraints.get(i).update();
		}
	}

	public int getLayerID() {

		return this.layerID;
	}

	public void setLayerID(final int id) {
		this.layerID = id;
	}

	/**
	 * Updates the Geometry (needs to be called after one or more
	 * applyAtomPosition() calls)
 	 * @param boolean updatePositionFactor move the molecule to a newly generated position 
	 */
	public void updateGeometry(boolean updatePositionFactor) {
		this.needToRecalculateImages = true;
		final Matrix3f oldm = Cm2_Toolkit.copyMatrix(this.getRotationMatrix());

		MyBounds mybounds = new MyBounds(this.allAtomsSorted);

		Point3f pointToUpdatePositionFactor = new Point3f();

		pointToUpdatePositionFactor.sub(mybounds.center);
		

		// for(int
		// i=this.allAtomsSorted.length-5;i<this.allAtomsSorted.length;i++) {
		// Point3f p = this.allAtomsSorted[i].getPosition();
		// p.add(this.getPosition());
		// System.out.println(i+" "+p);
		// }
		// System.out.println();
		for (int i = 0; i < this.allAtomsSorted.length; i++) {
			final Atom_3D a = this.allAtomsSorted[i];
			a.position.sub(mybounds.center);
		}
		for (int i = 0; i < this.currentCGParticles.size(); i++) {
			// MyBounds my = new
			// MyBounds(this.currentCGParticles.get(i).includedAtoms);
			// this.currentCGParticles.get(i).position=my.center;
			this.currentCGParticles.get(i).position.sub(mybounds.center);
		}
		new PdbReader_3D().shapeIt(this);

		if (updatePositionFactor) {
			Point3f newPos = new Point3f(this.getPosition());
			pointToUpdatePositionFactor = Cm2_Toolkit.transformToNewPoint(oldm,
					pointToUpdatePositionFactor);
			newPos.sub(pointToUpdatePositionFactor);
			this.setPosition(newPos);
		}

		if (this.isPicked) {
			this.setPicked(false);
			this.setPicked(true);
		}
		if (!this.isPositionInAValidSpace()) {
			this.setPositionIsInAValidSpace(true);
			this.setPositionIsInAValidSpace(false);
		}

		this.setRotationMatrix(oldm);
		if (this.transformgroup.isLive()) {
			this.move();

		}
		this.updateVisualAtomPositionsOnly();

		// for(int
		// i=this.allAtomsSorted.length-5;i<this.allAtomsSorted.length;i++) {
		// Point3f p = this.allAtomsSorted[i].getPosition();
		// p.add(this.getPosition());
		// System.out.println(i+" "+p);
		// }
		// System.out.println();
		// if (this.renderMode == Molecule_3D.RENDER_MODE_BALLS_N_STICKS
		// || this.renderMode == Molecule_3D.RENDER_MODE_COLORED_STICKS) {
		// Group parent;
		// try {
		// parent = (Group) this.branchGroup.getParent();
		// } catch (final Exception e) {
		// e.printStackTrace();
		// return;
		// }
		// if (parent != null) {
		// parent.removeChild(this.branchGroup);
		//
		// final Enumeration<?> en = this.transformgroup.getAllChildren();
		//
		// while (en.hasMoreElements()) {
		// Object o = en.nextElement();
		// if (o instanceof BranchGroup
		// && ((BranchGroup) o).getChild(0) instanceof Shape3D) {
		// this.transformgroup.removeChild((BranchGroup) o);
		// o = null;
		// }
		// }
		//
		// parent.addChild(this.branchGroup);
		// }
		//
		// }
		// System.out.println(this.getPDBcode() + " : Atoms "
		// + this.getAtomCount() + ", Constraints "
		// + this.currentConstraints.size() + "   updateGeometry()");
	}

	/**
	 * Updates the Geometry (needs to be called after one or more
	 * applyAtomPosition() calls)
	 */
	public void updateGeometry() {
		this.updateGeometry(true);
	}

	public void updateVisualAtomPositionsOnly() {
		if (this.renderMode == CM2_CONSTANTS.RENDERMODE.RENDER_MODE_COVALENT
				|| this.renderMode == CM2_CONSTANTS.RENDERMODE.RENDER_MODE_BALLS_N_STICKS
				|| this.renderMode == CM2_CONSTANTS.RENDERMODE.RENDER_MODE_VAN_DER_WAALS) {
			for (int i = 0; i < this.atoms.length; i++) {
				if (this.atoms[i].branchGroup != null) {

					for (int j = 0; j < this.atoms[i].branchGroup.numChildren(); j++) {
						final TransformGroup tg = (TransformGroup) this.atoms[i].branchGroup
								.getChild(j);
						final Transform3D t3d = new Transform3D();
						final Vector3f v = new Vector3f(this.atoms[i].position);
						v.scale(CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
						t3d.setTranslation(v);
						tg.setTransform(t3d);
					}
				}

			}

		}

		for (int i = 0; i < this.activeConstraints.size(); i++) {
			this.activeConstraints.get(i).update();
		}
		for (int i = 0; i < this.originalConstraints.length; i++) {
			this.originalConstraints[i].update();
		}

	}

	public int getRenderMode() {
		return this.renderMode;
	}

	/**
	 * Sets this protein picked or unpicked. This effects the way the Shape of
	 * this protein gets rendered.
	 * 
	 * @param b
	 */
	public void setPicked(final boolean b) {
		if (b == this.isPicked)
			return;

		this.isPicked = b;

		final Enumeration<?> en = this.transformgroup.getAllChildren();

		while (en.hasMoreElements()) {
			Object o = en.nextElement();
			if (o instanceof BranchGroup && ((BranchGroup) o).getUserData() instanceof SelectionBox
					&& ((SelectionBox) ((BranchGroup) o).getUserData()).mode == SelectionBox.PICKED) {
				this.transformgroup.removeChild((BranchGroup) o);
				o = null;
			}
		}

		if (!b)
			return;
		final SelectionBox box = new SelectionBox(this, SelectionBox.PICKED);
		this.transformgroup.addChild(box.getBranchGroup());

	}

	/**
	 * Sets, wether this protein is in a valid space (that means not colliding
	 * with anything) or not. If set true, this protein will be rendered in its
	 * normal mode. If set false, this protein will be rendered transparent and
	 * red
	 * 
	 * @param b
	 */
	public void setPositionIsInAValidSpace(final boolean proteinRedAndTransparent) {

		if (proteinRedAndTransparent == this.positionisinavalidspace)
			return;
		this.positionisinavalidspace = proteinRedAndTransparent;

		final Enumeration<?> en = this.transformgroup.getAllChildren();

		while (en.hasMoreElements()) {
			Object o = en.nextElement();
			if (o instanceof BranchGroup
					&& ((BranchGroup) o).getUserData() instanceof SelectionBox
					&& ((SelectionBox) ((BranchGroup) o).getUserData()).mode == SelectionBox.INVALID_POSITION) {
				this.transformgroup.removeChild((BranchGroup) o);
				o = null;
			}
		}

		if (proteinRedAndTransparent)
			return;
		final SelectionBox box = new SelectionBox(this, SelectionBox.INVALID_POSITION);
		this.transformgroup.addChild(box.getBranchGroup());

	}

	/**
	 * Returns wether this protein has the valid position flag set or not Use
	 * isPositionInValidSpace() instead.
	 * 
	 * @return boolean
	 */
	@Deprecated
	public boolean isPositionisinavalidspace() {
		return this.positionisinavalidspace;
	}

	/**
	 * Returns wether this protein has the valid position flag set or not
	 * 
	 * @return boolean
	 */
	public boolean isPositionInAValidSpace() {
		return this.positionisinavalidspace;
	}

	/**
	 * Returns wether this protein is currently picked or not
	 * 
	 * @return boolean
	 */
	public boolean isPicked() {
		return this.isPicked;
	}

	/**
	 * Sets the shape to pickable.
	 * 
	 * @param pickable
	 */
	public void setPickable(boolean pickable) {
		this.shape.setPickable(pickable);
	}

	/**
	 * This method sets the molecule into transparent mode. Note that the
	 * regular transparent mode is correlated with an NOT pickable shape. If you
	 * need a special setting for pickable, use instead: setTransparent(final
	 * boolean transparent, final boolean pickable) or setPickable(boolean
	 * pickable)
	 * 
	 * @param b
	 */
	public void setTransparent(final boolean b) {
		this.setTransparent(b, !b);
	}

	/**
	 * Sets the transparency and the picking value of the molecule.
	 * 
	 * @param transparent
	 * @param pickable
	 */
	public void setTransparent(final boolean transparent, final boolean pickable) {

		this.isTransparent = transparent;
		this.setPickable(pickable);

		if (this.renderMode != CM2_CONSTANTS.RENDERMODE.RENDER_MODE_POLYGON)
			return;

		if (transparent)
			this.shape.getAppearance().getPolygonAttributes()
					.setPolygonMode(PolygonAttributes.POLYGON_POINT);
		else
			this.shape.getAppearance().getPolygonAttributes()
					.setPolygonMode(DataPool.J3DSettings.polygon_mode);
	}

	/**
	 * Assigns a new Position to the Atom at the specified index. Note that the
	 * position that is passed must be relative to the original alignment of the
	 * molecule without rotation and 0,0,0 at its center
	 * 
	 * @param atomIndex
	 * @param newPosition
	 */
	public void applyAtomPosition(final int atomIndex, Point3f newPosition) {

		final Atom_3D atom = this.atoms[atomIndex];

		final Point3f newPositionCP = new Point3f(newPosition);
		newPositionCP.sub(this.getPosition());

		final Matrix3f revMatrix = Cm2_Toolkit.copyMatrix(this.getRotationMatrix());
		revMatrix.invert();

		final Point3f newPoint = Cm2_Toolkit.transformToNewPoint(revMatrix, newPositionCP);

		atom.setPosition(newPoint);

		for (int i = 0; i < this.activeConstraints.size(); i++) {
			Constraint_3D con = this.activeConstraints.get(i);
			if (atom == con.atomA || atom == con.atomB)
				this.activeConstraints.get(i).update();
		}

		this.alignedTPs = null;
		this.needToRecalculateImages = true;

	}

	public void setPosition(final Point3f p) {
		this.setPosition(p.x, p.z);
		this.setYTranslation(p.y);
	}

	public Shape3D getShape() {
		return this.shape;
	}

	public BranchGroup getBranchGroup() {

		return this.branchGroup;
	}

	public boolean isTransparent() {
		return this.isTransparent;
	}

	/**
	 * Returns if the Molecule is displaying atom colors in Covalent or Van Der
	 * Waals Mode.
	 * 
	 * @return boolean
	 */
	public boolean isUsingAtomColors() {
		return useAtomColors;
	}

	/**
	 * Sets if the Molecule should be displayed using atom colors in Covalent or
	 * Van Der Waals Mode.
	 * 
	 * @param useAtomColors
	 */
	public void setUseAtomColors(boolean useAtomColors) {
		this.useAtomColors = useAtomColors;
	}

	public Box getUnrotatedBounds() {
		return this.unrotatedBounds;
	}

	public void setUnrotatedBounds(final Box unrotatedBounds) {
		this.unrotatedBounds = unrotatedBounds;
	}

	public int getModelcount() {
		return this.modelcount;
	}

	public void setModelcount(final int modelcount) {
		this.modelcount = modelcount;
	}

	public void changeAndApplyModel(final int newModel, boolean updatePositionFactor) {

		final PdbReader_3D reader = new PdbReader_3D();

		try {
			final String[][] in = reader.readModel(DataPool.getRawPdbData(this
					.getPDBFile().getAbsolutePath()), newModel);
			final Vector<Atom_3D> normal = new Vector<Atom_3D>();
			final Vector<Atom_3D> opm = new Vector<Atom_3D>();
			for (int i = 0; i < in[0].length; i++) {
				try {
					normal.addElement(new Atom_3D(in[0][i]));
				} catch (final Exception e) {

				}
			}
			for (int i = 0; i < in[1].length; i++) {
				try {
					opm.addElement(new Atom_3D(in[1][i]));
				} catch (final Exception e) {

				}
			}

			if (normal.size() != this.getAtomCount())
				throw new Exception("Orig= " + this.getAtomCount() + " Lines "
						+ normal.size());
			else if (opm.size() != this.opmAtoms.length)
				throw new Exception("OrigOPM= " + this.getAtomCount()
						+ " OPMLines " + opm.size());

			for (int i = 0; i < normal.size(); i++) {
				this.atoms[i].position = normal.get(i).position;
			}
			for (int i = 0; i < opm.size(); i++) {
				this.opmAtoms[i].position = opm.get(i).position;
			}
		} catch (final IOException e) {
			e.printStackTrace();
			return;
		} catch (final Exception e) {
			e.printStackTrace();
			return;
		}
		this.updateGeometry(updatePositionFactor);
		this.setModelSet(newModel);
	}
	
	public void changeAndApplyModel(final int newModel) {
		changeAndApplyModel(newModel, true);
	}

	public Image getImageSmall() {

		if (this.needToRecalculateImages || this.smallImage == null)
			this.repaintImages();
		return this.smallImage;
	}

	public Image getImageLarge() {
		if (this.needToRecalculateImages || this.largeImage == null)
			this.repaintImages();
		return this.largeImage;
	}

	private void repaintImages() {

		this.largeImage = Cm2_Toolkit.Membrane_Image2D_Generator.createImage(this, 50, 60);
		this.smallImage = this.largeImage.getScaledInstance(30, 30, Image.SCALE_FAST);
		this.needToRecalculateImages = false;
	}

	public Atom_3D[] getAlignedOPMAtoms() {

		Atom_3D[] ret = new Atom_3D[this.opmAtoms.length];
		for (int i = 0; i < ret.length; i++) {
			Atom_3D a = this.opmAtoms[i].copy();
			a.position = Cm2_Toolkit.transformToNewPoint(this.getRotationMatrix(), a.position);
			a.position.add(this.getPosition());
			ret[i] = a;
		}
		return ret;
	}

	public synchronized void displayOPMAtoms(boolean b) {
		if (this.opmAtoms.length < 1)
			return;
		if (b == this.isDisplayingOPMAtoms)
			return;
		this.isDisplayingOPMAtoms = b;

		Group parent;

		parent = (Group) this.branchGroup.getParent();

		if (parent != null)
			parent.removeChild(this.branchGroup);

		// for (int i = 0; i < this.opmAtoms.length; i++) {
		// if (this.atoms[i].branchGroup != null)
		// this.transformgroup.removeChild(this.opmAtoms[i].branchGroup);
		// this.opmAtoms[i].branchGroup = null;
		//
		// }

		// final Enumeration<?> en = this.transformgroup.getAllChildren();
		//
		// while (en.hasMoreElements()) {
		// Object o = en.nextElement();
		// if (o instanceof BranchGroup
		// && ((BranchGroup) o).getChild(0) instanceof Shape3D) {
		// this.transformgroup.removeChild((BranchGroup) o);
		// o = null;
		// }
		// }

		if (!b) {

			for (int i = 0; i < this.opmAtoms.length; i++) {

				if (this.opmAtoms[i].branchGroup != null)
					this.transformgroup.removeChild(this.opmAtoms[i].branchGroup);

				this.opmAtoms[i].branchGroup = null;
			}

		} else {

			Atom_Creator_3D.addBalls(this, this.opmAtoms, false, null, 0, true);

			for (int i = 0; i < this.opmAtoms.length; i++) {

				if (this.opmAtoms[i].branchGroup != null && this.opmAtoms[i].branchGroup.getParent() == null)
					this.transformgroup.addChild(this.opmAtoms[i].branchGroup);

			}

		}

		if (parent != null)
			parent.addChild(this.branchGroup);

	}

	// public void addNewConstraint(Atom atomA, Atom atomB) {
	//
	// Constraint constr = new Constraint(atomA, atomB);
	// this.currentConstraints.addElement(constr);
	//
	// if (this.renderMode > Molecule_3D.RENDER_MODE_VAN_DER_WAALS)
	// this.getTransformGroup().addChild(constr.getBranchGroup());
	//
	// }

	@SuppressWarnings("unchecked")
	public void coarseGrain(Particle newParticle) {

		Collections.sort(newParticle.includedAtoms);

		// newParticle.comparator=this.allAtomsSorted.length+newParticle.includedAtoms.firstElement().comparator;

		Vector<Atom_3D> newAtoms = new Vector<Atom_3D>();

		for (int i = 0; i < this.atoms.length; i++) {
			if (!newParticle.includes(atoms[i])) {
				newAtoms.addElement(atoms[i]);
			} else {
				if (!newAtoms.contains(newParticle)) {
					newAtoms.addElement(newParticle);
				}
			}
		}

		for (int i = 0; i < newAtoms.size(); i++) {
			newAtoms.get(i).comparator = i;
		}

		this.currentCGParticles.addElement(newParticle);

		//		// try to sort the molecules according the Y-position, highest position first, 
		//		// still makes problems in the resulting PDB file
		//		if (this.currentCGParticles.size()==0) {
		//			this.currentCGParticles.addElement(newParticle);
		//		} else {
		//			for (int i=0; i<this.currentCGParticles.size(); i++) {
		//				if (this.currentCGParticles.get(i).getPosition().y <= newParticle.getPosition().y) {
		//					this.currentCGParticles.add(i,newParticle);
		//					break;
		//				} else if (i==this.currentCGParticles.size()-1) {
		//					this.currentCGParticles.add(newParticle);
		//					break;
		//				}
		//			}
		//		}
		//		System.out.println("\nList of Particles:\n"+Arrays.toString(currentCGParticles.toArray()));	
		//		}

		for (int i = 0; i < newAtoms.size(); i++) {

			if (newAtoms.get(i).branchGroup == null) {

				if (this.renderMode == CM2_CONSTANTS.RENDERMODE.RENDER_MODE_COVALENT
						|| this.renderMode == CM2_CONSTANTS.RENDERMODE.RENDER_MODE_VAN_DER_WAALS)
					Atom_Creator_3D
							.addBalls(
										this, 
										new Atom_3D[] { newAtoms.get(i) }, 
										true, 
										null,
										this.renderMode != CM2_CONSTANTS.RENDERMODE.RENDER_MODE_VAN_DER_WAALS ? 0
												: 1, 
										false);
				else if (this.renderMode == CM2_CONSTANTS.RENDERMODE.RENDER_MODE_BALLS_N_STICKS)
					Atom_Creator_3D.addBalls(
										this, 
										new Atom_3D[] { newAtoms.get(i) }, 
										false, 
										null,
										0, 
										false);
				this.transformgroup.addChild(newAtoms.get(i).branchGroup);
			}

		}
		for (int i = 0; i < this.atoms.length; i++) {
			if ((newAtoms.contains(atoms[i]) == false) && atoms[i].branchGroup != null) {
				this.transformgroup.removeChild(atoms[i].branchGroup);
				atoms[i].branchGroup = null;

			}
		}

		this.atoms = newAtoms.toArray(new Atom_3D[0]);

		// this.sortAtoms();
		this.updateAllBonds();

		this.alignedTPs = null;
		this.alignedAtoms = null;

		this.needToRecalculateImages = true;

	}

	public void restoreFromCoarseGrain(Particle oldParticle) {

		Vector<Atom_3D> newAtoms = new Vector<Atom_3D>();

		for (int i = 0; i < this.atoms.length; i++) {
			if (this.atoms[i] != oldParticle) {
				newAtoms.addElement(atoms[i]);
			} else {
				for (int j = 0; j < oldParticle.includedAtoms.size(); j++) {
					newAtoms.addElement(oldParticle.includedAtoms.get(j));
				}
			}
		}

		for (int i = 0; i < newAtoms.size(); i++) {
			newAtoms.get(i).comparator = i;
		}
		this.currentCGParticles.removeElement(oldParticle);

		for (int i = 0; i < newAtoms.size(); i++) {

			if (newAtoms.get(i).branchGroup == null) {

				if (this.renderMode == CM2_CONSTANTS.RENDERMODE.RENDER_MODE_COVALENT
						|| this.renderMode == CM2_CONSTANTS.RENDERMODE.RENDER_MODE_VAN_DER_WAALS)
					Atom_Creator_3D
							.addBalls(
										this, 
										new Atom_3D[] { newAtoms.get(i) }, 
										true, 
										null,
										this.renderMode != CM2_CONSTANTS.RENDERMODE.RENDER_MODE_VAN_DER_WAALS ? 0
												: 1, 
										false);
				else if (this.renderMode == CM2_CONSTANTS.RENDERMODE.RENDER_MODE_BALLS_N_STICKS)
					Atom_Creator_3D.addBalls(
										this, 
										new Atom_3D[] { newAtoms.get(i) }, 
										false, 
										null,
										0, 
										false);
				this.transformgroup.addChild(newAtoms.get(i).branchGroup);
			}

		}
		for (int i = 0; i < this.atoms.length; i++) {
			if ((newAtoms.contains(atoms[i]) == false) && atoms[i].branchGroup != null) {
				this.transformgroup.removeChild(atoms[i].branchGroup);
				atoms[i].branchGroup = null;

			}
		}

		this.atoms = newAtoms.toArray(new Atom_3D[0]);

		this.updateAllBonds();
		for (int i = 0; i < this.activeConstraints.size(); i++) {
			this.activeConstraints.get(i).update();
		}

		this.alignedTPs = null;
		this.alignedAtoms = null;
		this.needToRecalculateImages = true;

	}

	public Particle getHostParticle(Atom_3D atom) {

		for (int i = 0; i < this.currentCGParticles.size(); i++) {
			if (this.currentCGParticles.get(i).includes(atom))
				return this.currentCGParticles.get(i);

		}

		return null;
	}

	public void updateAllBonds() {
		Vector<Object> v = Cm2_Toolkit.fromArrayToVector(this.atoms);
		Vector<Constraint_3D> newC = new Vector<Constraint_3D>();

		for (int i = 0; i < this.atoms.length; i++) {

			atoms[i].currentNeighbors.clear();
			if (atoms[i] instanceof Particle) {
				Particle p = (Particle) atoms[i];
				for (int j = 0; j < p.includedAtoms.size(); j++) {
					p.includedAtoms.get(j).currentNeighbors.clear();
				}
			}

		}

		for (int i = 0; i < this.originalConstraints.length; i++) {
			Atom_3D a = this.originalConstraints[i].atomA;
			Atom_3D b = this.originalConstraints[i].atomB;

			boolean isFreeA = v.contains(a);
			boolean isFreeB = v.contains(b);

			if (isFreeA && isFreeB) {
				newC.addElement(this.originalConstraints[i]);
				a.currentNeighbors.addElement(b);
				b.currentNeighbors.addElement(a);

			} else {
				if (!isFreeA && !isFreeB) {

					Particle p = this.getHostParticle(a);
					Particle pp = this.getHostParticle(b);
					if (p != pp) {

						newC.addElement(new Constraint_3D(p, pp));

						p.currentNeighbors.addElement(pp);
						pp.currentNeighbors.addElement(p);
					}

				} else if (isFreeA) {

					Particle p = this.getHostParticle(b);
					newC.addElement(new Constraint_3D(a, p));
					a.currentNeighbors.addElement(p);
					p.currentNeighbors.addElement(a);
				} else {

					Particle p = this.getHostParticle(a);
					newC.addElement(new Constraint_3D(b, p));
					b.currentNeighbors.addElement(p);
					p.currentNeighbors.addElement(b);
				}

			}
		}

		for (int i = 0; i < this.atoms.length; i++) {

			// if (atoms[i] instanceof Particle) {
			// Particle p = (Particle) atoms[i];
			// p.originalAngles=new Point3f[p.currentNeighbors.size()];
			// for (int j = 0; j < p.currentNeighbors.size(); j++) {
			// p.originalAngles[j]=p.currentNeighbors.get(j).getPosition();
			// }
			// p.oldPostion=new Point3f(p.getPosition());
			// }

		}

		if (this.renderMode == CM2_CONSTANTS.RENDERMODE.RENDER_MODE_BALLS_N_STICKS
				|| this.renderMode == CM2_CONSTANTS.RENDERMODE.RENDER_MODE_COLORED_STICKS) {

			for (int i = 0; i < this.activeConstraints.size(); i++) {
				Constraint_3D c = this.activeConstraints.get(i);
				if (newC.contains(c)) {
					if (c.getBranchGroup().getParent() == null) {
						this.transformgroup.addChild(c.getBranchGroup());
					}
				} else {
					if (c.getBranchGroup().getParent() != null)
						this.transformgroup.removeChild(c.getBranchGroup());
				}
			}
			for (int i = 0; i < newC.size(); i++) {
				Constraint_3D c = newC.get(i);

				if (c.getBranchGroup().getParent() == null) {
					this.transformgroup.addChild(c.getBranchGroup());
				}

			}
		}

		this.activeConstraints = newC;

		// for(int i=0;i<this.activeConstraints.size();i++) {
		// this.activeConstraints.get(i).update();
		// }
	}

	public void sortAtoms() {

		Arrays.sort(this.atoms);

	}

	public String getCurrentProfileName() {
		return currentProfileName;
	}

	public void setCurrentProfileName(String currentProfileName) {
		this.currentProfileName = currentProfileName;
	}

	public void applyProfile(MoleculeProfile profile) {

		Group parent;

		parent = (Group) this.branchGroup.getParent();

		if (parent != null)
			parent.removeChild(this.branchGroup);

		this.atoms = new Atom_3D[this.allAtomsSorted.length];
		for (int i = 0; i < this.allAtomsSorted.length; i++) {
			this.atoms[i] = this.allAtomsSorted[i];
		}

		for (int i = 0; i < this.activeConstraints.size(); i++) {
			Constraint_3D c = this.activeConstraints.get(i);
			if (c.getBranchGroup() != null)
				this.transformgroup.removeChild(c.getBranchGroup());
		}
		this.activeConstraints.clear();

		for (int i = 0; i < this.currentCGParticles.size(); i++) {
			Particle p = this.currentCGParticles.get(i);

			if (p.branchGroup != null)
				this.transformgroup.removeChild(p.branchGroup);
		}
		this.currentCGParticles.clear();

		this.setCurrentProfileName(profile.name);

		for (int i = 0; i < profile.positions.length; i++) {
			this.allAtomsSorted[i].position = new Point3f(profile.positions[i]);

		}

		this.updateAllBonds();

		for (int i = 0; i < profile.particleIDs.length; i++) {

			Integer[] indices = profile.particleMemberIndices[i];

			Atom_3D[] members = new Atom_3D[indices.length];
			for (int j = 0; j < indices.length; j++) {
				members[j] = this.allAtomsSorted[indices[j]];
			}

			Particle newP = new Particle(members);
			newP.name = profile.particleAtomNames[i];
			newP.element = profile.particleElementNames[i];
			newP.particleIdentifier = profile.particleIDs[i];
			newP.color = profile.colors[i];
			this.coarseGrain(newP);

		}

		if (this.renderMode == CM2_CONSTANTS.RENDERMODE.RENDER_MODE_COVALENT
				|| this.renderMode == CM2_CONSTANTS.RENDERMODE.RENDER_MODE_BALLS_N_STICKS
				|| this.renderMode == CM2_CONSTANTS.RENDERMODE.RENDER_MODE_VAN_DER_WAALS) {
			Vector<Atom_3D> newAtoms = new Vector<Atom_3D>();
			for (int i = 0; i < this.atoms.length; i++) {
				if ((this.atoms[i].branchGroup == null)
				// || (!this.atoms[i].branchGroup.isLive())
				) {
					newAtoms.addElement(this.atoms[i]);

				}

			}

			if (this.renderMode == CM2_CONSTANTS.RENDERMODE.RENDER_MODE_BALLS_N_STICKS) {
				Atom_Creator_3D.addBalls(
											this, 
											newAtoms.toArray(new Atom_3D[0]), 
											false, 
											null, 
											0,
											false);

			} else {
				Atom_Creator_3D.addBalls(
											this, 
											newAtoms.toArray(new Atom_3D[0]), 
											true, 
											null,
											this.getRenderMode() != CM2_CONSTANTS.RENDERMODE.RENDER_MODE_VAN_DER_WAALS ? 0
													: 1,
											false);

			}

			for (int i = 0; i < newAtoms.size(); i++) {

				this.transformgroup.addChild(newAtoms.get(i).branchGroup);
			}

		}

		this.alignedTPs = null;
		this.alignedAtoms = null;
		this.needToRecalculateImages = true;

		if (parent != null)
			parent.addChild(this.branchGroup);
	}

	public void displayAdditionalInformation(Atom_3D atCursor, float camDist) {

		this.additionalAtomInfoTG.removeAllChildren();
		if (atCursor == null)
			return;

		for (int i = 0; i < this.activeConstraints.size(); i++) {

			Constraint_3D c = this.activeConstraints.get(i);
			if (atCursor != c.atomA && atCursor != c.atomB)
				continue;
			BranchGroup newBranch = new BranchGroup();

			newBranch.setCapability(BranchGroup.ALLOW_DETACH);
			newBranch.setPickable(false);

			String cL = "" + c.currentLength;
			if (cL.contains(".")) {
				try {
					cL = cL.substring(0, cL.indexOf(".") + 3);
				} catch (Exception e) {

				}
			}
			String text = "" + cL + "A";

			Color3f fg = new Color3f(DataPool.J3DSettings.bright_appearance ? Color.black
					: Color.white);

			final Text2D textX = new Text2D(text, new Color3f(fg), "Arial", 25, Font.PLAIN);

			final TransformGroup objScale = new TransformGroup();
			final TransformGroup objTrans = new TransformGroup();
			final TransformGroup billTrans = new TransformGroup();
			billTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
			billTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);

			// final BoundingBox textB = (BoundingBox) textX.getBounds();
			newBranch.addChild(objTrans);

			final Transform3D t3d = new Transform3D();

			objScale.addChild(billTrans);
			billTrans.addChild(textX);

			final BoundingSphere bounds = new BoundingSphere(new Point3d(), 100000.0);

			final Billboard bill = new Billboard(billTrans);
			bill.setSchedulingBounds(bounds);
			bill.setAlignmentMode(Billboard.ROTATE_ABOUT_POINT);
			objTrans.addChild(bill);
			bill.setRotationPoint(c.centerPoint);

			t3d.setTranslation(new Vector3f(c.centerPoint));

			if (camDist < 0.2f) {
				t3d.setScale(0.025f);
			} else {
				t3d.setScale(0.025f + (camDist / 20f));
			}

			objScale.setTransform(t3d);

			newBranch.addChild(objScale);

			this.additionalAtomInfoTG.addChild(newBranch);

		}

	}

	// public void rotateCoarseGrainMembers() {
	// for(int a=0;a<this.currentCGParticles.size();a++) {
	//			
	// Particle p=this.currentCGParticles.get(a);
	// if(p.includedAtoms.size()<2)continue;
	//			
	// Atom_3D[]included = p.includedAtoms.toArray(new Atom_3D[0]);
	//			
	// Matrix3f matrix = Cm2_Toolkit.createAUnitMatrix();
	//			
	//			
	// Vector<Constraint_3D>vConst=new Vector<Constraint_3D>();
	// Vector<Atom_3D>rot=new Vector<Atom_3D>();
	// for(int i=0;i<this.originalConstraints.length;i++) {
	//				
	// Constraint_3D c = this.originalConstraints[i];
	// boolean b1=p.includes(c.atomA);
	// boolean b2=p.includes(c.atomB);
	// if((b1&&b2)||(!b1&&!b2))
	// continue;
	//			
	//			
	//			
	// vConst.addElement(c);
	//			
	//			
	// if(b1)rot.addElement(c.atomA);
	// else rot.addElement(c.atomB);
	//			
	// }
	//			
	// Constraint_3D[] c = vConst.toArray(new Constraint_3D[0]);
	// Atom_3D[]toRot=rot.toArray(new Atom_3D[0]);
	//			
	//			
	//			
	// for(int i=0;i<this.allAtomsSorted.length;i++) {
	// this.allAtomsSorted[i].position.sub(p.position);
	// }
	//			
	//			
	//			
	// Random r = new Random();
	//			
	//			
	//			
	// Point3f[]oldPos=new Point3f[toRot.length];
	// for(int j=0;j<toRot.length;j++) {
	// oldPos[j]=new Point3f(toRot[j].position);
	// }
	//			
	// float[]prevD=new float[c.length];
	// for(int j=0;j<c.length;j++) {
	// prevD[j]=c[j].atomA.position.distance(c[j].atomB.position);
	// }
	//			
	// int d=0;
	// while(true) {
	//				
	// Matrix3f old=Cm2_Toolkit.copyMatrix(matrix);
	//				
	//				
	// float rX=(r.nextInt(2)+r.nextFloat())*(r.nextBoolean()?-1:1);
	// Cm2_Toolkit.rotate_X_Axis_Matrix(matrix,rX);
	//			
	// float rY=(r.nextInt(2)+r.nextFloat())*(r.nextBoolean()?-1:1);
	// Cm2_Toolkit.rotate_Y_Axis_Matrix(matrix,rY);
	//			
	// float rZ=(r.nextInt(2)+r.nextFloat())*(r.nextBoolean()?-1:1);
	// Cm2_Toolkit.rotate_Z_Axis_Matrix(matrix,rZ);
	//				
	//				
	//					
	//				
	// for(int i=0;i<toRot.length;i++) {
	//					
	//					
	// toRot[i].position=
	// Cm2_Toolkit.transformToNewPoint(matrix,toRot[i].position);
	//					
	//					
	//					
	// }
	// float[] newD=new float[prevD.length];
	//				
	// boolean pass=true;
	// for(int i=0;i<c.length;i++) {
	//					
	// newD[i]=c[i].atomA.position.distance(c[i].atomB.position);
	//					
	// if(newD[i]<=prevD[i]&&(newD[i]>=(c[i].atomA.radiusCov+c[i].atomB.radiusCov)))
	// {
	//						
	//						
	//						
	// }else {
	//						
	//					
	// pass=false;
	//						
	// break;
	// }
	//					
	// }
	//				
	// if(pass) {
	//					
	//					
	// // for(int j=0;j<toRot.length;j++) {
	// // oldPos[j]=new Point3f(toRot[j].position);
	// // }
	//					
	// prevD=newD;
	// d=0;
	//					
	// }else {
	//					
	// matrix=old;
	//					
	// }
	//				
	// for(int j=0;j<toRot.length;j++) {
	// toRot[j].position=new Point3f(oldPos[j]);
	// }
	//				
	// d++;
	//			
	// if(d>200)break;
	//				
	//				
	// }
	//			
	//			
	//			
	//		
	//			
	//			
	//			
	//			
	//			
	//			
	//			
	//			
	//			
	//			
	//			
	//
	//			
	//			
	// for(int i=0;i<included.length;i++) {
	//
	//				
	// included[i].position=Cm2_Toolkit.transformToNewPoint(matrix,included[i].position);
	//				
	//
	// }
	//			
	//			
	//			
	// for(int i=0;i<this.allAtomsSorted.length;i++) {
	// this.allAtomsSorted[i].position.add(p.position);
	// }
	//			
	//			
	//			
	//			
	//		
	//		
	// }
	// }

	@Deprecated
	/**
	 * Please use getOriginalAllAtomConstraintLengths() instead.
	 */
	public float[] getOriginalAllAtomConstraintLenghts() {
		return getOriginalAllAtomConstraintLengths();
	}

	public float[] getOriginalAllAtomConstraintLengths() {

		MoleculeProfile profile = null;

		try {

			String code = this.getPDBcode();
			if (code == null || code.length() < 1)
				code = Cm2_Toolkit.getSimpleName(this.getPDBFile());

			profile = Molecule_Profile_IO.readProfile(this, this.getCurrentProfileName(), code);

			Constraint_3D[] constr = this.originalConstraints;

			Vector<Object> v = Cm2_Toolkit.fromArrayToVector(allAtomsSorted);

			float[] ret = new float[constr.length];
			for (int i = 0; i < constr.length; i++) {

				Atom_3D a = constr[i].atomA;
				Atom_3D b = constr[i].atomB;

				int indexA = v.indexOf(a);
				int indexB = v.indexOf(b);

				Point3f posA = profile.positions[indexA];

				Point3f posB = profile.positions[indexB];

				ret[i] = posA.distance(posB);

			}

			return ret;

		} catch (Exception e) {
			return null;

		}

	}

	public void rotateCoarseGrainMembers() {
		if (this.currentCGParticles.isEmpty())
			return;

		Vector<Particle> toTreat = new Vector<Particle>();
		for (int i = 0; i < this.currentCGParticles.size(); i++) {
			toTreat.addElement(this.currentCGParticles.get(i));
		}

		float[] ideals = this.getOriginalAllAtomConstraintLengths();

		float currentDiff = 0f;

		for (int i = 0; i < this.originalConstraints.length; i++) {

			float newDistance = this.originalConstraints[i].atomA.position
					.distance(this.originalConstraints[i].atomB.position);

			float diff = Math.abs(ideals[i] - newDistance);
			currentDiff += diff;

		}

		Random r = new Random();

		Point3f[] backup = new Point3f[this.allAtomsSorted.length];
		for (int i = 0; i < backup.length; i++) {
			backup[i] = this.allAtomsSorted[i].getPosition();
		}

		int deadCount = 0;
		while (true) {

			Collections.shuffle(toTreat);

			int stop = r.nextInt(toTreat.size());
			stop += 1;

			Point3f[] backUpParticles = new Point3f[stop];

			for (int a = 0; a < stop; a++) {

				Particle p = toTreat.get(a);

				backUpParticles[a] = p.getPosition();

				if (p.includedAtoms.size() < 2)
					continue;

				Atom_3D[] included = p.includedAtoms.toArray(new Atom_3D[0]);

				Matrix3f matrix = Cm2_Toolkit.createAUnitMatrix();

				for (int i = 0; i < included.length; i++) {
					included[i].position.sub(p.position);
				}

				float rX = (r.nextInt(2) + r.nextFloat()) * (r.nextBoolean() ? -1 : 1);
				Cm2_Toolkit.rotate_X_Axis_Matrix(matrix, rX);

				float rY = (r.nextInt(2) + r.nextFloat()) * (r.nextBoolean() ? -1 : 1);
				Cm2_Toolkit.rotate_Y_Axis_Matrix(matrix, rY);

				float rZ = (r.nextInt(2) + r.nextFloat()) * (r.nextBoolean() ? -1 : 1);
				Cm2_Toolkit.rotate_Z_Axis_Matrix(matrix, rZ);

				for (int i = 0; i < included.length; i++) {

					included[i].position = Cm2_Toolkit.transformToNewPoint(matrix,
							included[i].position);

				}

				for (int i = 0; i < included.length; i++) {
					included[i].position.add(p.position);
				}

				if (deadCount > 50) {

					float mX = (r.nextFloat()) * (r.nextBoolean() ? -1 : 1);
					float mY = (r.nextFloat()) * (r.nextBoolean() ? -1 : 1);
					float mZ = (r.nextFloat()) * (r.nextBoolean() ? -1 : 1);

					Point3f trans = new Point3f(mX, mY, mZ);

					for (int i = 0; i < included.length; i++) {
						included[i].position.add(trans);
					}
					p.position.add(trans);
				}
			}

			float newDiff = 0f;
			boolean accept = true;
			for (int i = 0; i < this.originalConstraints.length; i++) {

				float newDistance = this.originalConstraints[i].atomA.position
						.distance(this.originalConstraints[i].atomB.position);

				float diff = Math.abs(ideals[i] - newDistance);
				newDiff += diff;
				if (newDiff >= currentDiff) {
					accept = false;
					break;
				}

			}

			if (accept) {
				for (int i = 0; i < backup.length; i++) {
					backup[i] = this.allAtomsSorted[i].getPosition();
				}
				deadCount = 0;

				currentDiff = newDiff;
			} else {
				for (int i = 0; i < backup.length; i++) {
					this.allAtomsSorted[i].position = new Point3f(backup[i]);
				}
				deadCount++;

				for (int i = 0; i < backUpParticles.length; i++) {
					toTreat.get(i).position = new Point3f(backUpParticles[i]);
				}
			}

			if (deadCount > 500)
				break;

		}
	}

}
