/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.engine;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.util.Locale;
import java.util.Random;

import javax.swing.JPopupMenu;
import javax.swing.ToolTipManager;
import javax.swing.UIManager;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.gui.MainGui;
import org.cellmicrocosmos.cm2.gui.dialogs.HelpDialog;
import org.cellmicrocosmos.cm2.gui.dialogs.StatusDialog;
import org.cellmicrocosmos.cm2.io.Installer;
import org.cellmicrocosmos.cm2.misc.Cm2_IconDepot;
import org.cellmicrocosmos.cm2.misc.ConfigParser;


/**
 * The Entry Class of the Application. Has to be executed in order to run the
 * MembraneEditor.
 * 
 * @author Tim Dingersen dackelnose@gmx.net, Bjoern Sommer bjoern@CELLmicrocosmos.org
 * 
 */
public class StartMain {

	private static Engine engine = null;
	
	/**
	 * Entry Point. If a project filepath gets passed as the first argument,
	 * this will be opened initially
	 * 
	 * @param args
	 */
	public static void startMain(final String[] args) {
		
		// resetting LastLaunchedEngine
		engine = null;
		
		/*
		 * setting the Runtime to english language
		 */
		
		Locale.setDefault(Locale.ENGLISH);
		
		// If active, all occuring error prints are redirected to a file
		if (CM2_CONSTANTS.DEBUG.REDIRECT_ERRORS_TO_LOGFILE) {
			try {
				System.setErr(new PrintStream(new File("Errorlog.txt")));
			} catch (final FileNotFoundException e) {
				System.setErr(null);
			}
		}

		System.out.println(CM2_CONSTANTS.GENERAL.NAME_OF_THIS_PROGRAM+" "+CM2_CONSTANTS.GENERAL.VERSION+"\n\n");

		System.out.println("[Parameters]");
		System.out.println("-filepath of an existing membrane");
		System.out.println("[Options]");
		System.out.println("-help       |  starts the program's user manual");
		System.out.println("-reset      |  start with default settings     ");
		System.out.println("-system_L&F |  use the platform Look & Feel    ");
		System.out.println("                                               ");
		System.out.println("Memory Check:  max. "+Runtime.getRuntime().maxMemory()+" bytes for Java heap space reserved");
		System.out.println("                                               ");
		fixLinuxJavaLibraryBug();
		System.out.println("                                               ");
		System.out.print("please wait .");
		/*
		 * Extracting the initial project, if one is passed as an argument
		 */
		Installer.init();
		Installer.installImages();
		System.out.print(".");

		for (int i = 0; i < args.length; i++) {
			if (args[i].trim().equalsIgnoreCase("-reset")) {
				CM2_CONSTANTS.IO.CONFIG_FILE.delete();
				CM2_CONSTANTS.IO.LAYOUT_FILE.delete();
			}
		}

		// setting the Configparser to the configfile
		ConfigParser.setConfigFile(CM2_CONSTANTS.IO.CONFIG_FILE);
		try {
			ConfigParser.readDataFromDisc();
		} catch (final Exception e) {
			// can be ignored, application is set to use default values anyway

		}
		final File file2open;
		final String cm1TempName2Edit;
		File f = null;
		String tempName = "null";

		for (int i = 0; i < args.length; i++) {
		if (new File(args[i].trim()).canRead()) {
				f = new File(args[i].trim());
			} else if (args[i].trim().equalsIgnoreCase("-system_L&F")) {
				DataPool.useSystemLookandFeel = true;
			} else {
				if (!args[i].trim().equalsIgnoreCase("-help") && 
					!args[i].trim().equalsIgnoreCase("-reset"))
				tempName = args[i].trim();
			}

		}

		file2open = f;
		cm1TempName2Edit = tempName;

		DataPool.J3DSettings.restoreSettings();

		if (DataPool.useSystemLookandFeel == false) {
			if (DataPool.J3DSettings.bright_appearance) {
				MainGui.setBrightLookAndFeel();
			} else {
				MainGui.setDarkLookAndFeel();
			}
		} else {
			try {
				UIManager.setLookAndFeel(UIManager
						.getSystemLookAndFeelClassName());
			} catch (final Exception e) {

			}
		}

		for (int i = 0; i < args.length; i++) {
			if (args[i].trim().equalsIgnoreCase("-help")) {
				Cm2_IconDepot.readIcons();
				new HelpDialog().popupHelp();
				return;
			}
		}

		/*
		 * Setting up some stuff that increases hospitality towards the
		 * heavyweight j3d implementation
		 */
		JPopupMenu.setDefaultLightWeightPopupEnabled(false);
		ToolTipManager.sharedInstance().setLightWeightPopupEnabled(false);
		ToolTipManager.sharedInstance().setInitialDelay(100);
		ToolTipManager.sharedInstance().setDismissDelay(10000);

		// initializing the startup dialog
		final int random = new Random()
				.nextInt(CM2_CONSTANTS.IMAGES_AND_ICONS.STARTUP_IMAGES.length);
		final StatusDialog welcomescreen = new StatusDialog(
				CM2_CONSTANTS.IMAGES_AND_ICONS.STARTUP_IMAGES[random],
				true,
				CM2_CONSTANTS.IMAGES_AND_ICONS.STARTUP_DIALOG_TEXT_FOREGROUND[random]);

		final Thread runCm2 = new Thread() {

			public void run() {

				engine = new Engine(welcomescreen, file2open, cm1TempName2Edit);

				welcomescreen.dispose();

			}
		};

		runCm2.start();

	}
	
	/**
	 * Start the program without any arguments directly.
	 *
	 */
	public static void startMain() {
		final String[] args = {"null"};
		startMain(args);
	}
	
	/**
	 * This fix is required for new Java versions on Linux which do not find the
	 * libjawt.so library anymore.
	 * 
	 * As proposed by kappa:
	 * http://lwjgl.org/forum/index.php/topic,4085.0.html
	 * http://www.webcitation.org/6Ms1wLAW2
	 */
	public static void fixLinuxJavaLibraryBug() {
		
		try {
			
			if (System.getProperty("os.name").equals("Linux")) {
				System.out.println("Relocating 'libjawt.so' ...");
				String osArch = System.getProperty("os.arch");
		
				java.awt.Toolkit.getDefaultToolkit(); // loads libmawt.so (needed by jawt)

				// 64 bit version
				if ("amd64".equals(osArch) || "x86_64".equals(osArch)) 
					System.load(System.getProperty("java.home") + "/lib/amd64/libjawt.so");
				// 32 bit version
				else 
					System.load(System.getProperty("java.home") + "/lib/i386/libjawt.so");
				
				System.out.println("... done! Found at: "+System.getProperty("java.home") + "/lib/amd64/libjawt.so");
			}
		
		} catch (Exception ex) {
			System.out.println("... failed! Not found at: "+System.getProperty("java.home") + "/lib/amd64/libjawt.so\n"
								+ "    Only if the program fails now to run, 'libawt.so' might not be linked correctly on your system.\n"
								+ "    This is a known bug of new versions of OpenJava and Oracle Java.\n"
								+ "    1) Please locate '/lib/amd64/libjawt.so' manually in your Java installation directory.\n"
								+ "    2) Type the following line into your console:\n"
								+ "        export LD_LIBRARY_PATH=/opt/java/jre/lib/amd64\n"
								+ "       where amd64 is the directory where the libjawt.so file is found\n"
								+ "    3) Then start the JNLP file from the same console with\n"
								+ "        javaws startCm2.jnlp\n");
		}
	}

	/**
	 * Retrieves the engine of the program.
	 * This is only needed by external applications.
	 * @return
	 */
	public static Engine getLastLaunchedEngine() {
		return engine;
	}

}
