/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.functionality3d;

import java.awt.Color;
import java.awt.Font;

import javax.media.j3d.Appearance;
import javax.media.j3d.Billboard;
import javax.media.j3d.BoundingBox;
import javax.media.j3d.BoundingSphere;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.ColoringAttributes;
import javax.media.j3d.Geometry;
import javax.media.j3d.GeometryArray;
import javax.media.j3d.LineArray;
import javax.media.j3d.LineAttributes;
import javax.media.j3d.Node;
import javax.media.j3d.PolygonAttributes;
import javax.media.j3d.Shape3D;
import javax.media.j3d.Transform3D;
import javax.media.j3d.TransformGroup;
import javax.vecmath.Color3f;
import javax.vecmath.Point3d;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;


import com.sun.j3d.utils.geometry.Text2D;

/**
 * 
 * The selection box which is drawn around a 
 * selected molecule is handled here.
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class SelectionBox {

	public static final int PICKED = 0;
	public static final int INVALID_POSITION = 1;

	Shape3D shape = new Shape3D();
	BranchGroup branchGroup = new BranchGroup();
	BranchGroup branchGroup2 = new BranchGroup();
	Color color;
	public int mode;

	public SelectionBox(final Molecule_3D mol, final int mode) {
		this.mode = mode;

		if (DataPool.J3DSettings.bright_appearance)
			this.color = Color.GREEN.darker().darker();
		else
			this.color = Color.GREEN;

		if (mode == SelectionBox.INVALID_POSITION)
			this.color = Color.red;

		Cm2_Toolkit.setAllCapabilitysForBranchGroup(this.branchGroup);
		Cm2_Toolkit.setAllCapabilitysForBranchGroup(this.branchGroup2);
		Cm2_Toolkit.setAllCapabilitysForShape3D(this.shape);

		this.shape.setPickable(mode == SelectionBox.PICKED);
		this.shape.setUserData(mol);
		this.branchGroup.setUserData(this);
		
		final Appearance appearance = new Appearance();
		
		final ColoringAttributes coloringAttrs = new ColoringAttributes(
				new Color3f(this.color), ColoringAttributes.SHADE_GOURAUD);
		appearance.setColoringAttributes(coloringAttrs);
		
		final LineAttributes la = new LineAttributes();
		la.setPatternMask(LineAttributes.PATTERN_DOT);
		if (mode == SelectionBox.INVALID_POSITION)
			la.setLineWidth(4f);
		else
			la.setLineWidth(2f);
		
		final PolygonAttributes polyattrs = new PolygonAttributes();
		polyattrs.setPolygonMode(PolygonAttributes.POLYGON_LINE);
		appearance.setPolygonAttributes(polyattrs);
		appearance.setLineAttributes(la);
		
		Cm2_Toolkit.setRenderingAttributes(appearance);
		
		this.shape.setAppearance(appearance);
		
		if(mol.getUnrotatedBounds()==null)return;

		float x = mol.getUnrotatedBounds().getXdimension() / 2;
		float y = mol.getUnrotatedBounds().getYdimension() / 2;
		float z = mol.getUnrotatedBounds().getZdimension() / 2;

		if (mode == SelectionBox.INVALID_POSITION) {
			x += (1f * CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
			y += (1f * CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
			z += (1f * CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
		}

		final float over = 10f * CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;

		Point3f[][] allVertices;
		if (mode == SelectionBox.PICKED)
			allVertices = new Point3f[15][2];
		else
			allVertices = new Point3f[12][2];

		int c = 0;
		allVertices[c++] = new Point3f[] { new Point3f(-x, -y, -z),
				new Point3f(x, -y, -z) };
		allVertices[c++] = new Point3f[] { new Point3f(-x, -y, -z),
				new Point3f(-x, -y, z) };
		allVertices[c++] = new Point3f[] { new Point3f(x, -y, z),
				new Point3f(x, -y, -z) };
		allVertices[c++] = new Point3f[] { new Point3f(x, -y, z),
				new Point3f(-x, -y, z) };

		allVertices[c++] = new Point3f[] { new Point3f(-x, y, -z),
				new Point3f(x, y, -z) };
		allVertices[c++] = new Point3f[] { new Point3f(-x, y, -z),
				new Point3f(-x, y, z) };
		allVertices[c++] = new Point3f[] { new Point3f(x, y, z),
				new Point3f(x, y, -z) };
		allVertices[c++] = new Point3f[] { new Point3f(x, y, z),
				new Point3f(-x, y, z) };

		allVertices[c++] = new Point3f[] { new Point3f(-x, -y, -z),
				new Point3f(-x, y, -z) };
		allVertices[c++] = new Point3f[] { new Point3f(x, -y, -z),
				new Point3f(x, y, -z) };
		allVertices[c++] = new Point3f[] { new Point3f(-x, -y, z),
				new Point3f(-x, y, z) };
		allVertices[c++] = new Point3f[] { new Point3f(x, -y, z),
				new Point3f(x, y, z) };

		if (mode == SelectionBox.PICKED) {
			allVertices[c++] = new Point3f[] { new Point3f(0, 0, 0),
					new Point3f(x + over, 0, 0) };
			allVertices[c++] = new Point3f[] { new Point3f(0, 0, 0),
					new Point3f(0, y + over, 0) };
			allVertices[c++] = new Point3f[] { new Point3f(0, 0, 0),
					new Point3f(0, 0, z + over) };

			this.addMyTextNode("X", new Point3f(x + over + over, 0, 0),
					new Color3f(this.color), new Font("Arial", Font.BOLD, 17));
			// this.addMyTextNode("-X",new Point3f(-x-over-over,0,0),new
			// Color3f(color),new Font("Arial", Font.BOLD, 17));
			this.addMyTextNode("Y", new Point3f(0, y + over + over, 0),
					new Color3f(this.color), new Font("Arial", Font.BOLD, 17));
			// this.addMyTextNode("-Y",new Point3f(0,-y-over-over,0),new
			// Color3f(color),new Font("Arial", Font.BOLD, 17));
			this.addMyTextNode("Z", new Point3f(0, 0, z + over + over),
					new Color3f(this.color), new Font("Arial", Font.BOLD, 17));
			// this.addMyTextNode("-Z",new Point3f(0,0,-z-over-over),new
			// Color3f(color),new Font("Arial", Font.BOLD, 17));

		}

		this.branchGroup.addChild(this.branchGroup2);

		for (int i = 0; i < allVertices.length; i++) {

			final LineArray pla = new LineArray(2, GeometryArray.COORDINATES);
			pla.setCoordinates(0, allVertices[i]);
			this.shape.addGeometry(pla);
			pla.setCapability(Geometry.ALLOW_INTERSECT);

		}

		this.branchGroup.addChild(this.shape);

	}

	private void addMyTextNode(final String text, final Point3f position,
			final Color3f color, final Font font) {
		// this.branchGroup.removeChild(this.branchGroup2);
		// creating the text
		final Text2D textX = new Text2D(text, color, font.getFontName(), font
				.getSize(), font.getStyle());

		// those are necessary for the billboard to work
		final TransformGroup objScale = new TransformGroup();
		final TransformGroup objTrans = new TransformGroup();
		final TransformGroup billTrans = new TransformGroup();
		billTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		billTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);

		this.branchGroup2.addChild(objTrans);

		final Transform3D t3d = new Transform3D();

		objScale.addChild(billTrans);
		billTrans.addChild(textX);

		// determining the center of the text used as the rotation point
		final BoundingBox textB = (BoundingBox) textX.getBounds();
		final Point3d upper = new Point3d();
		textB.getUpper(upper);
		final Point3f rotationPt = new Point3f((float) (0 + upper.x / 2),
				(float) (0 + upper.y / 2), (float) (0 + upper.z / 2));

		final BoundingSphere bounds = new BoundingSphere(
				new Point3d(rotationPt), 100000.0);

		// Setting up the billboard node
		final Billboard bill = new Billboard(billTrans);
		bill.setSchedulingBounds(bounds);
		bill.setAlignmentMode(Billboard.ROTATE_ABOUT_POINT);
		objTrans.addChild(bill);
		bill.setRotationPoint(rotationPt);

		// positioning of the entire Scene
		t3d.setTranslation(new Vector3f((float) (position.x - upper.x / 2),
				(float) (position.y - upper.y / 2),
				(float) (position.z - upper.z / 2)));

		objScale.setTransform(t3d);

		this.branchGroup2.addChild(objScale);

	}

	public Node getBranchGroup() {

		return this.branchGroup;
	}

}
