/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.gui.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;
import java.awt.event.WindowListener;
import java.lang.reflect.InvocationTargetException;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;

/**
 * A simple Dialog designed to show a Progress to the User. Will become visible
 * when created and remains visible until it is disposed.
 * 
 * Can be updated by setProgress(String,int) while the passed String shows the
 * current Operation and the int sets the Progress in the bar in percent.
 * 
 * Has 3 different Modes which is determined by the Constructer that is used:
 * 
 * -display an image that is passes in the constructor. the OP String is drawn
 * into the progress bar. Size will be set automatically
 * 
 * -display the OP String in a Label and show percentages in the bar
 * 
 * -if a JButton is passed, this one will be added. Most likely a button from
 * the host class that cancels the operation
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
@SuppressWarnings("serial")
public class StatusDialog extends JDialog {

	private final JProgressBar bar = new JProgressBar();

	private final int mode;

	private boolean showBar = true;

	private boolean paintStringOnImage = false;

	private boolean stringwithbackground = false;

	private String string = null;

	private Color startupcolor = Color.black;

	private JPanel ground;
	
	private boolean closePressed = false;

	private final JLabel label = new JLabel() {

		@Override
		public void paintComponent(final Graphics g) {
			super.paintComponent(g);
			if (StatusDialog.this.string == null)
				return;

			if (StatusDialog.this.paintStringOnImage) {

				if (StatusDialog.this.stringwithbackground) {
					final JLabel l = new JLabel();
					l.setFont(new Font("Arial", Font.BOLD, 12));
					l.setText(StatusDialog.this.string);
					final Rectangle r = new Rectangle(3, (this.getHeight() - 6)
							- l.getPreferredSize().height,
							l.getPreferredSize().width + 4, l
									.getPreferredSize().height - 1);

					g.setColor(this.getBackground());
					g.fillRect(r.x + 2, r.y + 2, r.width, r.height);
					g.setColor(this.getBackground().brighter().brighter());
					g.fillRect(r.x, r.y, r.width, r.height);

				}

				g.setColor(StatusDialog.this.startupcolor);
				g.setFont(new Font("Arial", Font.BOLD, 12));
				g
						.drawString(StatusDialog.this.string, 5, this
								.getHeight() - 10);
			}
		}

		@Override
		public Dimension getPreferredSize() {
			final Dimension dim = super.getPreferredSize();
			if (StatusDialog.this.mode != 1)
				return new Dimension(dim.width + 20, dim.height + 10);
			else
				return dim;
		}
	};

	/**
	 * show only the label and the progressbar
	 * 
	 */
	public StatusDialog(final boolean showBar) {
		this.showBar = showBar;
		this.mode = 0;

		this.build(null, null);

	}

	/**
	 * show the dialog also with the passed button inside
	 * 
	 * @param btn
	 *            - a button instance
	 */
	public StatusDialog(final JButton btn, final boolean showBar) {
		this.showBar = showBar;
		this.mode = 0;
		this.build(null, btn);

	}

	/**
	 * Show the dialog with an image and a small statusbar at the bottom
	 * 
	 * @param imagepath
	 */
	public StatusDialog(final String imagepath, final boolean showBar,
			final Color textforeground) {

		if (textforeground == null) {
			this.stringwithbackground = true;

		} else
			this.startupcolor = textforeground;

		this.showBar = showBar;
		this.mode = 1;
		this.build(imagepath, null);

	}

	/**
	 * Builds up the dialog
	 * 
	 * @param imagepath
	 * @param btn
	 */
	private void build(final String imagepath, final JButton btn) {

		if (this.mode == 1) {
			this.setUndecorated(true);
			// bar.setForeground(new Color(140, 180, 210));
		} else
			// bar.setForeground(new Color(100, 200, 100));

			this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);

		this.bar.setFont(new Font("Arial", Font.PLAIN, 12));
		
		this.addWindowListener(new WindowListener() {
			
			public void windowOpened(WindowEvent e) {
			}
			
			public void windowIconified(WindowEvent e) {
			}
			
			public void windowDeiconified(WindowEvent e) {
			}
			
			public void windowDeactivated(WindowEvent e) {
			}
			
			public void windowClosing(WindowEvent e) {
				StatusDialog.this.closePressed = true;
			}
			
			public void windowClosed(WindowEvent e) {
			}
			
			public void windowActivated(WindowEvent e) {
			}

		});

		this.addWindowFocusListener(new WindowFocusListener() {

			public void windowGainedFocus(final WindowEvent arg0) {

			}

			public void windowLostFocus(final WindowEvent arg0) {
				StatusDialog.this.requestFocus();

			}

		});

		if (imagepath != null) {

			final Image img = this.getToolkit().getImage(imagepath);

			this.label.setIcon(new ImageIcon(img));

			this.getContentPane().add(this.label, BorderLayout.NORTH);
			this.getContentPane().add(this.bar, BorderLayout.SOUTH);

			this.pack();

			if (this.getWidth() >= 200 && this.getHeight() >= 50) {
				this.paintStringOnImage = true;
			}

		} else {

			this.ground = new JPanel() {
				@Override
				public void doLayout() {
					if (this.getPreferredSize().width < 200)
						this.setPreferredSize(new Dimension(200, this
								.getPreferredSize().height));

					if (this.getPreferredSize().height < 70)
						this.setPreferredSize(new Dimension(this
								.getPreferredSize().width, 70));

					if (StatusDialog.this.label.getPreferredSize().width + 20 > this
							.getPreferredSize().width)
						this
								.setPreferredSize(new Dimension(
										StatusDialog.this.label
												.getPreferredSize().width + 20,
										this.getPreferredSize().height));

					super.doLayout();
				}
			};

			this.ground.setLayout(new BoxLayout(this.ground, BoxLayout.Y_AXIS));
			this.ground.add(Box.createVerticalGlue());
			final JPanel p1 = new JPanel();

			p1.setLayout(new BoxLayout(p1, BoxLayout.X_AXIS));
			p1.add(Box.createHorizontalStrut(10));
			p1.add(this.label);
			p1.add(Box.createHorizontalGlue());
			p1.add(Box.createHorizontalStrut(10));
			this.ground.add(p1);
			this.ground.add(Box.createVerticalGlue());
			if (this.showBar) {
				final JPanel p2 = new JPanel();

				p2.setLayout(new BoxLayout(p2, BoxLayout.X_AXIS));
				p2.add(Box.createHorizontalGlue());
				p2.add(this.bar);
				p2.add(Box.createHorizontalGlue());
				this.ground.add(p2);
				this.ground.add(Box.createVerticalGlue());
			}

			if (btn != null) {
				final JPanel p3 = new JPanel();

				p3.setLayout(new BoxLayout(p3, BoxLayout.X_AXIS));
				this.ground.add(p3);
				p3.add(Box.createHorizontalGlue());
				p3.add(btn);
				p3.add(Box.createHorizontalGlue());
				btn.setFont(this.label.getFont());
				this.ground.add(Box.createVerticalGlue());
			}

			final JPanel border = new JPanel();
			border.add(this.ground);

			this.getContentPane().add(border);

			this.ground.setBorder(BorderFactory
					.createLineBorder(Color.LIGHT_GRAY));

			if (this.showBar)
				this.bar.setBorder(BorderFactory.createLoweredBevelBorder());

			if (this.showBar) {
				this.bar.setMinimumSize(new Dimension(this.getToolkit()
						.getScreenSize().width / 4, 23));
				this.bar.setMaximumSize(new Dimension(this.getToolkit()
						.getScreenSize().width / 4, 23));
				this.bar.setPreferredSize(new Dimension(this.getToolkit()
						.getScreenSize().width / 4, 23));
				this.ground
						.setPreferredSize(new Dimension(
								this.bar.getPreferredSize().width + 20,
								this.bar.getPreferredSize().height
										+ 70
										+ (btn != null ? btn.getPreferredSize().height + 10
												: 0)));
			}
		}

		this.setResizable(false);

	}

	/**
	 * Update the current Progress that should be shown
	 * 
	 * @param string
	 *            may contain information about the step
	 * @param progress
	 *            the progress the progressbar will display
	 */
	public void setProgress(final String string, final int progress,
			final boolean ownThread) {

		if (progress==-1) {
			bar.setIndeterminate(true);
		} else {
			bar.setIndeterminate(false);
		}
		
		if (ownThread) {
			SwingUtilities.invokeLater(new Thread() {
				@Override
				public void run() {
					StatusDialog.this.updateProgress(string, progress);
				}
			});
		} else {
			this.updateProgress(string, progress);
		}
	}

	private void updateProgress(final String string, final int progress) {

		if (this.mode == 1) {
			this.string = string;
			if (this.showBar && this.paintStringOnImage == false)
				this.bar.setString(string);

			if (this.showBar)
				this.bar.setValue(progress);

			if (this.showBar && this.paintStringOnImage == true) {

				this.label.repaint();
			}
			this.pack();

			this.setLocation(this.getToolkit().getScreenSize().width / 2
					- this.getWidth() / 2,
					this.getToolkit().getScreenSize().height / 2
							- this.getHeight() / 2);
			if (this.isVisible() == false)
				this.setVisible(true);
			
		} else {
			this.label.setText(string);

			this.ground.doLayout();
			this.pack();

			this.setLocation(this.getToolkit().getScreenSize().width / 2
					- this.getWidth() / 2,
					this.getToolkit().getScreenSize().height / 2
							- this.getHeight() / 2);
			if (this.isVisible() == false)
				this.setVisible(true);
			if (this.showBar)
				this.bar.setValue(progress);

		}
		
		this.revalidate();
		this.pack();
		
	}
	
	/**
	 * Check if the user pressed the close button.
	 * @return boolean was closed button pressed
	 */
	public boolean isClosedPressed() {
		return this.closePressed;
	}
	
	/**
	 * Use method to make a secure disposal of the StatusDialog.
	 */
	public void disposeLater() {
		SwingUtilities.invokeLater(new Thread() {
			public void run() {
				StatusDialog.this.dispose();
			}
		});
	}
}
