/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.io;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import javax.swing.JOptionPane;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.engine.Engine;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;
import org.cellmicrocosmos.cm2.misc.ConfigParser;


/**
 *  The Installer installs the images, the external Jmol library, the help files,
 *  the settings and the workspace during the first start. Aditionally it checks,
 *  it there are updates if you are starting an updated version of the MembraneEditor.
 * 
 * @author Tim Dingersen dackelnose@gmx.net, Bjoern Sommer bjoern@CELLmicrocosmos.org
 * 
 */
public class Installer {

	public static void init() {

		FileIOHandler.home = System.getProperty("user.home") != null ? System
				.getProperty("user.home")
				+ File.separator + "Cm2" + File.separator : "";
		new File(FileIOHandler.home).mkdir();

		new File("extract.jar").deleteOnExit();

	}

	public static void installImages() {

		if (!Installer.isUpToDate(FileIOHandler.home + "cm2images",
				Engine.VERSION_IMAGES)) {
			try {
				
				Cm2_Toolkit
						.deleteDir(new File(FileIOHandler.home + "cm2images"),true);
				Cm2_Toolkit.extractFromJar("cm2images.jar", FileIOHandler.home
						+ "cm2images");
				
				Installer.createVersionFile(FileIOHandler.home + "cm2images",
						Engine.VERSION_IMAGES);
			} catch (final Exception e) {
				e.printStackTrace();
				JOptionPane.showMessageDialog(null,
						"Files not found!\n\nReason:\n" + e.toString(),
						"Error", JOptionPane.ERROR_MESSAGE);
				System.exit(0);
			}
		}

	}
	
	public static void installJmol() {
		if (!Installer.isUpToDate(FileIOHandler.home + "jmol",
				Engine.VERSION_JMOL)) {
			try {

				Cm2_Toolkit.extractFromJar("jmol.jar",
						FileIOHandler.home + "jmol");
				Installer.createVersionFile(FileIOHandler.home + "jmol",
						Engine.VERSION_JMOL);
			} catch (final Exception e) {
				e.printStackTrace();
				JOptionPane.showMessageDialog(null,
						"Files not found!\n\nReason:\n" + e.toString(),
						"Error", JOptionPane.ERROR_MESSAGE);
				System.exit(0);
			}
		}

	}

	public static void installHelpfiles() {
		if (!Installer.isUpToDate(FileIOHandler.home + "cm2help",
				Engine.VERSION_HELP)) {
			try {
				Cm2_Toolkit.deleteDir(new File(FileIOHandler.home + "cm2help"),true);
				Cm2_Toolkit.extractFromJar("cm2help.jar", FileIOHandler.home
						+ "cm2help");
				Installer.createVersionFile(FileIOHandler.home + "cm2help",
						Engine.VERSION_HELP);
			} catch (final Exception e) {
				e.printStackTrace();
				JOptionPane.showMessageDialog(null,
						"Files not found!\n\nReason:\n" + e.toString(),
						"Error", JOptionPane.ERROR_MESSAGE);
				System.exit(0);
			}
		}

	}

	public static void installSettings() {
		if (!Installer.isUpToDate(FileIOHandler.home + "cm2settings",
				Engine.VERSION_SETTINGS)) {
			try {

				Cm2_Toolkit.extractFromJar("cm2settings.jar",
						FileIOHandler.home + "cm2settings");
				Installer.createVersionFile(FileIOHandler.home + "cm2settings",
						Engine.VERSION_SETTINGS);
			} catch (final Exception e) {
				e.printStackTrace();
				JOptionPane.showMessageDialog(null,
						"Files not found!\n\nReason:\n" + e.toString(),
						"Error", JOptionPane.ERROR_MESSAGE);
				System.exit(0);
			}
		}

	}

	public static void initWorkspace() {
		
		new File(File.separator + "extract.jar").delete();
		File workspace;
		try {
			workspace = new File(ConfigParser.getValue("workspace")[0]);
			if (!workspace.exists())
				throw new Exception();
		} catch (final Exception e) {
			workspace = FileIOHandler.askForWorkspace();

			if (workspace == null)
				System.exit(0);

		}

		DataPool.workspace = workspace;
		DataPool.IO.initFilePaths();

		ConfigParser.deleteAllAt("workspace");
		ConfigParser.addValue("workspace", workspace.getPath());

		// this seems not tow work anymore, maybe Java version dependent?
		//		if (DataPool.IO.DEFAULTPDBDIRECTORY.isDirectory())
			try {
				Cm2_Toolkit.extractFromJar("defaultpdbfiles.jar",
						DataPool.IO.DEFAULTPDBDIRECTORY.getAbsolutePath());
			} catch (final Exception e) {
				JOptionPane.showMessageDialog(null,
						"The defined PDB directory does not exist or is no directory.\n\n",
						"Write access denied", JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			}

		final File[] files = new File[] { DataPool.IO.DEFAULTMEMBRANEDIRECTORY,
				DataPool.IO.DEFAULTPDBDIRECTORY,
				DataPool.IO.DEFAULTDBDIRECTORY,
				CM2_CONSTANTS.IO.TEMP_DIRECTORY,
				DataPool.IO.DEFAULTINCOMINGDIRECTORY };

		for (int i = 0; i < files.length; i++) {
			if (files[i].isDirectory() == false)
				if (!files[i].mkdirs()) {
					JOptionPane.showMessageDialog(null,
							"Needing permission to write\n\n"
									+ files[i].getPath(),
							"Write access denied", JOptionPane.ERROR_MESSAGE);
					System.exit(0);
				}

		}

	}

	private static boolean isUpToDate(final String directory,
			final long currentVersion) {

		try {
			
			final BufferedReader br = new BufferedReader(new FileReader(
					new File(directory + File.separator + "version.txt")));
			while (true) {
				final String s = br.readLine();
				if (s == null) {
					br.close();
					System.out.println("Updating directory "+directory+" with version "+currentVersion);
					return false;
				}
				try {
					final long version = Long.parseLong(s.trim());
					br.close();
					if (version == currentVersion) {
						return true;
					} else {
						System.out.println("Updating directory "+directory+" with version "+currentVersion);
						return false;
					}
				} catch (final Exception e) {

					continue;
				}

			}

		} catch (final Exception e) {

			return false;
		}

	}

	private static void createVersionFile(final String directory,
			final long version) {
		try {
			final BufferedWriter bw = new BufferedWriter(new FileWriter(
					new File(directory + File.separator + "version.txt")));

			bw.write("This file holds the current version of the directory.");
			bw.newLine();
			bw.newLine();
			bw.write("" + version);
			bw.newLine();
			bw.close();

		} catch (final IOException e) {

		}
	}

}
