/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.io;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.geom.Ellipse2D;
import java.io.File;
import java.io.IOException;
import java.util.Hashtable;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.vecmath.Matrix3f;
import javax.vecmath.Point3f;

import org.cellmicrocosmos.cm2.coarsegrainsupport.MoleculeProfile;
import org.cellmicrocosmos.cm2.coarsegrainsupport.Molecule_Profile_IO;
import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.Layer;
import org.cellmicrocosmos.cm2.dataobjects.Lipid_3D;
import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;
import org.cellmicrocosmos.cm2.dataobjects.MicroDomain_3D;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
import org.cellmicrocosmos.cm2.dataobjects.MyBounds;
import org.cellmicrocosmos.cm2.dataobjects.Protein_3D;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.gui.dialogs.StatusDialog;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;


/**
 * The Reader for our Membranes; Reads in a File, creates a new Membrane Model
 * which gets set up with the Data.
 * 
 * Doesnt send every Entry through the PDBReader; This happens only when a new
 * Type that isnt already parsed appears. Otherwise, the existing Types gets
 * cloned and some of their attributes replaced
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class MembraneXmlReader {

	public final static String ROOTELEMENTNAME_MEMBRANE = "CM2_MEMBRANE_DESCRIPTION";

	public final static String[] TAGNAMES_MEMBRANE = { "PROTEINS",
			"LIPIDS_EXTRA", "LIPIDS_INTRA", "EXAMPLE_EX", "EXAMPLE_IN",
			"MEMBRANE_ATTRIBUTES", "INSTANCE", "ATTR", "ENTRY_COUNTS",
			"MICRODOMAINS", "LAYERS" };

	// The MembraneModel that will be created and filled during process.
	// Can be fetched from this reader when reading is through.
	private MembraneModel model = null;

	// the dialog to show status
	private final StatusDialog dialog;

	

	
	private final PdbReader_3D reader = new PdbReader_3D();
	/**
	 * Plain Constructor
	 * 
	 */
	public MembraneXmlReader() {
		this.dialog = null;
	}

	/**
	 * Contructor with the StatusDialog that will be updated each step
	 * 
	 * @param dialog
	 */
	public MembraneXmlReader(final StatusDialog dialog) {
		this.dialog = dialog;
	}

	/**
	 * Reads in the passed file and fills the MembraneModel with Data
	 * 
	 * @param file
	 *            the file to read
	 * @throws IOException
	 * @throws JDOMException
	 * @throws InvalidEntriesException
	 *             thrown if at least one Entry got ignored
	 */
	public void readMembrane(final File file) throws IOException,
			JDOMException, InvalidEntriesException {

		int finished = 0;
		double step = 0;

		final Document doc = new SAXBuilder().build(file);

		final InvalidEntriesException IEE = new InvalidEntriesException();

		this.model = new MembraneModel();
		

		if (doc.getRootElement() == null)
			throw new JDOMException("No Root Element found!");
		final Element rootelement = doc.getRootElement();

		if (rootelement.getName().equals(
				MembraneXmlReader.ROOTELEMENTNAME_MEMBRANE) == false)
			throw new JDOMException("No Root Element found!");

		/*
		 * resolving the number of Entries to read; Doesnt effect anything,
		 * except the progress show in the statusbar
		 */
		try {
			final int proteins = rootelement.getChild(
					MembraneXmlReader.TAGNAMES_MEMBRANE[0]).getChildren()
					.size();
			final int lipids_extra = rootelement.getChild(
					MembraneXmlReader.TAGNAMES_MEMBRANE[1]).getChildren()
					.size();
			final int lipids_intra = rootelement.getChild(
					MembraneXmlReader.TAGNAMES_MEMBRANE[2]).getChildren()
					.size();
			final int examples_extra = rootelement.getChild(
					MembraneXmlReader.TAGNAMES_MEMBRANE[3]).getChildren()
					.size();
			final int examples_intra = rootelement.getChild(
					MembraneXmlReader.TAGNAMES_MEMBRANE[4]).getChildren()
					.size();
			final int todo = proteins + lipids_extra + lipids_intra
					+ examples_extra + examples_intra;
			step = 100 / new Integer(todo).doubleValue();
		} catch (final Exception e) {
			// step remains 0
		}

		final List rawdata = rootelement.getChildren("RAWDATA");

		for (int i = 0; i < rawdata.size(); i++) {
			final Element ele = (Element) rawdata.get(i);

			final String s = ele.getContent(0).getValue();

			final String[] all = s.split(CM2_CONSTANTS.IO.RAWDATA_SEPERATOR);

			final String[] data = new String[all.length - 1];
			for (int j = 0; j < data.length; j++) {
				data[j] = all[j + 1].trim();
				if(data[j].startsWith(")"))
					data[j]=data[j].substring(1);
				if(data[j].endsWith("("))
					data[j]=data[j].substring(0,data[j].length()-1);
				
			}
			
			 
			 if(all[0].endsWith("("))
					all[0]=all[0].substring(0,all[0].length()-1);
			DataPool.storeRawPdbData(all[0].trim(), data);
			
//			 System.out.println(all[0]);
//			 System.out.println(data[0]);

		}

		final List membraneattrs = rootelement
				.getChildren(MembraneXmlReader.TAGNAMES_MEMBRANE[5]);

		// reading the Attributes of the Membrane first
		for (int i = 0; i < membraneattrs.size(); i++) {

			final Element ele = (Element) membraneattrs.get(i);

			if (ele.getAttributeValue("Header") != null)
				this.model.setHeader(ele.getAttributeValue("Header"));
			else if (ele.getAttributeValue("PdbCode") != null)
				this.model.setCode(ele.getAttributeValue("PdbCode"));
			else if (ele.getAttributeValue("Title") != null)
				this.model.setTitle(ele.getAttributeValue("Title"));
			else if (ele.getAttributeValue("Species") != null)
				this.model.setSOURCE(ele.getAttributeValue("Species"));
			else if (ele.getAttributeValue("Author") != null)
				this.model.setAuthor(ele.getAttributeValue("Author"));
			else if (ele.getAttributeValue("Date") != null)
				this.model.setDate(ele.getAttributeValue("Date"));
			else if (ele.getAttributeValue("Comment") != null)
				this.model.setRemark(ele.getAttributeValue("Comment"));
			else if (ele.getAttributeValue("Keywds") != null)
				this.model.setKeywds(ele.getAttributeValue("Keywds"));
			else if (ele.getAttributeValue("Expdta") != null)
				this.model.setExpdta(ele.getAttributeValue("Expdta"));
			else if (ele.getAttributeValue("Jrnl") != null)
				this.model.setJrnl(ele.getAttributeValue("Jrnl"));
			else if (ele.getAttributeValue("AlgorithmName") != null)
				this.model.setAlgorithmname(ele
						.getAttributeValue("AlgorithmName"));
			else if (ele.getAttributeValue("AlgorithmDescription") != null)
				this.model.setAlgorithmdescription(ele
						.getAttributeValue("AlgorithmDescription"));
			else if (ele.getAttributeValue("AlgorithmStringRepresentation") != null)
				this.model.setAlgorithmStringRepresentation(ele
						.getAttributeValue("AlgorithmStringRepresentation"));

			else if (ele.getAttributeValue("Width") != null
					&& ele.getAttributeValue("Height") != null) {

				try {
					final int x = Integer.parseInt(ele
							.getAttributeValue("Width"));
					final int y = Integer.parseInt(ele
							.getAttributeValue("Height"));
					this.model.setSize(new Dimension(x, y));
				} catch (final Exception e) {
					throw new IOException(
							"A proper Membrane Size cannot be retrieved");
				}
			} else if (ele.getAttributeValue("DecimalPlaces") != null) {

				try {
					final int d = Integer.parseInt(ele
							.getAttributeValue("DecimalPlaces"));
					this.model.setDecimalPlaces(d);
				} catch (final Exception e) {
					IEE
							.addEntry("Number of Decimal Places cannot be retrieved.\nUsing the default entry.",e);
					this.model
							.setDecimalPlaces(CM2_CONSTANTS.MISC.LIPID_PERCENTAGE_DIVISOR);
				}

			} else if (ele.getAttributeValue("ComputeAbsoluteValues") != null) {

				try {
					final boolean d = Boolean.parseBoolean(ele
							.getAttributeValue("ComputeAbsoluteValues"));
					this.model.setComputeAbsoluteValues(d);
				} catch (final Exception e) {
					IEE
							.addEntry("If absolute values should be computed cannot be retrieved.\nUsing the default entry.",e);
					this.model
							.setComputeAbsoluteValues(CM2_CONSTANTS.MISC.COMPUTE_ABSOLUTE_VALUES);
				}

			} 
				
			else if (ele.getAttributeValue("LastGeneratedMembraneFile") != null) {
				final File lastone = new File(ele
						.getAttributeValue("LastGeneratedMembraneFile"));
				if (lastone.exists())
					this.model.setLastGeneratedMembraneFile(lastone);
			}

		}

		if (this.model.getSize() == null)
			throw new IOException("A proper Membrane Size cannot be retrieved");
		if (this.model.getSize().width < CM2_CONSTANTS.MEMBRANE_SIZES.MIN_SIZE_OF_MEMBRANE
				|| this.model.getSize().height < CM2_CONSTANTS.MEMBRANE_SIZES.MIN_SIZE_OF_MEMBRANE
				|| this.model.getSize().width > CM2_CONSTANTS.MEMBRANE_SIZES.MAX_SIZE_OF_MEMBRANE
				|| this.model.getSize().height > CM2_CONSTANTS.MEMBRANE_SIZES.MAX_SIZE_OF_MEMBRANE)
			throw new IOException("Membrane Size is outside valid range!");

		final Element element = rootelement
				.getChild(MembraneXmlReader.TAGNAMES_MEMBRANE[9]);
		if (element != null) {
			final List domains = element.getChildren();
			if (domains != null) {

				for (int i = 0; i < domains.size(); i++) {

					final Element domain = (Element) domains.get(i);

					final MicroDomain_3D micro = new MicroDomain_3D();

					try {

						final int id = new Integer(domain
								.getAttributeValue("Id")).intValue();
						micro.id = id;
						try {
							final int layerid = new Integer(domain
									.getAttributeValue("LayerId")).intValue();
							micro.layerId = layerid;
						} catch (final Exception e) {

						}
						final String geo = domain.getAttributeValue("Geom");

						if (geo.equalsIgnoreCase("Rectangular")) {

							final int x = new Integer(domain
									.getAttributeValue("x")).intValue();
							final int y = new Integer(domain
									.getAttributeValue("y")).intValue();
							final int w = new Integer(domain
									.getAttributeValue("width")).intValue();
							final int h = new Integer(domain
									.getAttributeValue("height")).intValue();

							micro.setGeometryType(MicroDomain_3D.RECTANGULAR);
							micro.geoObject = new Rectangle(x, y, w, h);
							micro.updateShape3D();
						} else if (geo.equalsIgnoreCase("Ellipse")) {

							final int x = new Integer(domain
									.getAttributeValue("x")).intValue();
							final int y = new Integer(domain
									.getAttributeValue("y")).intValue();
							final int w = new Integer(domain
									.getAttributeValue("width")).intValue();
							final int h = new Integer(domain
									.getAttributeValue("height")).intValue();

							micro.setGeometryType(MicroDomain_3D.ELLIPSE);
							micro.geoObject = new Ellipse2D.Double(x, y, w, h);
							micro.updateShape3D();
						} else if (geo.equalsIgnoreCase("Polygon")) {

							final String string = domain
									.getAttributeValue("Coords");

							final String[] coords = string.split(" ");

							final Polygon newpoly = new Polygon();

							for (int j = 0; j < coords.length; j++) {
								newpoly.addPoint(new Integer(coords[j++])
										.intValue(), new Integer(coords[j])
										.intValue());
							}

							micro.setGeometryType(MicroDomain_3D.POLYGON);

							micro.geoObject = newpoly;
							micro.updateShape3D();

						} else
							continue;

						this.model.addMicroDomain(micro);
					} catch (final Exception e) {

						continue;
					}

				}
			}
		}

		final Element element2 = rootelement
				.getChild(MembraneXmlReader.TAGNAMES_MEMBRANE[10]);
		if (element2 != null) {
			final List layers = element2.getChildren();
			if (layers != null) {

				for (int i = 0; i < layers.size(); i++) {

					final Element layer = (Element) layers.get(i);

					try {

						final int id = new Integer(layer
								.getAttributeValue("Id")).intValue();

						final float height = new Float(layer
								.getAttributeValue("Y_Val")).floatValue();

						this.model.addLayer(new Layer(height, id));
					} catch (final Exception e) {

						continue;
					}

				}
			}
		}

		Hashtable<String, String> parsedOnes = new Hashtable<String, String>();

		// first set the Proteins, then Lipids
		for (int group = 0; group < 5; group++) {

			// Hashtable<String, String> alreadyinforthisrunofloop = new
			// Hashtable<String, String>();
			// all types that have been read through the PDBReader are
			// registered
			// here
			if (group == 1)
				parsedOnes = new Hashtable<String, String>();

			final Element moleculecontainer = rootelement
					.getChild(MembraneXmlReader.TAGNAMES_MEMBRANE[group]);

			List instances;
			if (moleculecontainer == null)
				continue;
			else
				instances = moleculecontainer
						.getChildren(MembraneXmlReader.TAGNAMES_MEMBRANE[6]);

			// regarding a single Entry from here on
			final int instancecount = instances.size();

			for (int i = 0; i < instancecount; i++) {

				final int progress = (int) (finished * step);
				if (this.dialog != null)
					this.dialog.setProgress("Loading " + file.getName(),
							progress, false);

				final List attrs = ((Element) instances.get(i))
						.getChildren(MembraneXmlReader.TAGNAMES_MEMBRANE[7]);

				String Id = "";
				String PdbFile = "";
				String Profile = "Default Profile";
				String Pos_Z = "";
				String Pos_X = "";
				Color color;
				boolean useAtomColors = true;
				int modelnumber = -1;
				int domainID = 0;
				int LayerID = 0;
				int percent = 0;
				final Vector<String> alteredAtoms = new Vector<String>();

				if (group == 0)
					color = CM2_CONSTANTS.MOLECULE_DEFAULTS.DEFAULT_PROTEIN_COLOR;
				else
					color = CM2_CONSTANTS.MOLECULE_DEFAULTS.DEFAULT_LIPID_COLOR;
				Matrix3f matrix3f = null;
				float translation = 0;

				for (int j = 0; j < attrs.size(); j++) {
					final Element ele = (Element) attrs.get(j);

					if (ele.getAttributeValue("Id") != null)
						Id = ele.getAttributeValue("Id");
					else if (ele.getAttributeValue("PdbFile") != null)
						PdbFile = ele.getAttributeValue("PdbFile");
					else if (ele.getAttributeValue("Profile") != null)
						Profile = ele.getAttributeValue("Profile");

					else if (ele.getAttributeValue("Translation") != null) {
						try {
							translation = Float.parseFloat(ele
									.getAttributeValue("Translation"));
						} catch (final Exception e) {
						}
					} else if (ele.getAttributeValue("Matrix3f") != null) {
						final String matrixString = ele
								.getAttributeValue("Matrix3f");
						// TODO this is a workaround: after reloading/refreshing
						// a membrane,
						// the matrixString is parsed as a matrix string, not as
						// a string of numbers
						// and the resulting matrix leads to errors,
						// seems to be be a problem with the JDOM parser, maybe
						// the value Matrix3f is
						// not cleared correctly somewhere?
						// matrixString = matrixString.replace("[", "");
						// matrixString = matrixString.replace("]", "");
						// matrixString = matrixString.replace("\n", "");

						final StringTokenizer token = new StringTokenizer(
								matrixString);
						final int tokenLength = token.countTokens();
						if (tokenLength != 9) {

							IEE
									.addEntry(MembraneXmlReader.TAGNAMES_MEMBRANE[group]
											+ "_ENTRY "
											+ i
											+ " - Invalid Matrix!",null);
							continue;

						}

						final float[] floats = new float[9];

						for (int k = 0; k < tokenLength; k++) {

							try {

								floats[k] = Float.parseFloat(token.nextToken());

							} catch (final Exception e) {

								IEE
										.addEntry(MembraneXmlReader.TAGNAMES_MEMBRANE[group]
												+ "_ENTRY "
												+ i
												+ " - Invalid Matrix!",null);

								continue;
							}
						}

						matrix3f = new Matrix3f(floats);

					}

					else if (ele.getAttributeValue("Color") != null) {
						try {
							final int rgb = Integer.parseInt(ele
									.getAttributeValue("Color"));
							color = new Color(rgb);
						} catch (final Exception e) {

						}
					} else if (ele.getAttributeValue("UseAtomColors") != null) {
						try {
							useAtomColors = Boolean.parseBoolean(ele
									.getAttributeValue("UseAtomColors"));
						} catch (final Exception e) {

						}
					} else if (ele.getAttributeValue("Model") != null) {
						try {
							modelnumber = Integer.parseInt(ele
									.getAttributeValue("Model"));
						} catch (final Exception e) {
						}
					} else if (ele.getAttributeValue("DomainID") != null) {
						try {
							domainID = Integer.parseInt(ele
									.getAttributeValue("DomainID"));
						} catch (final Exception e) {
						}
					} else if (ele.getAttributeValue("LayerID") != null) {
						try {
							LayerID = Integer.parseInt(ele
									.getAttributeValue("LayerID"));
						} catch (final Exception e) {
						}
					} else if (ele.getAttributeValue("Altered_Atom") != null) {
						try {
							alteredAtoms.addElement(ele
									.getAttributeValue("Altered_Atom"));
						} catch (final Exception e) {
						}
					}

					else if ((group > 0)
							&& ele.getAttributeValue("Quantity") != null) {
						try {
							percent = Integer.parseInt(ele
									.getAttributeValue("Quantity"));
						} catch (final Exception e) {
						}
					}

					else if (ele.getAttributeValue("Pos_X") != null)
						Pos_X = ele.getAttributeValue("Pos_X");
					if (ele.getAttributeValue("Pos_Z") != null)
						Pos_Z = ele.getAttributeValue("Pos_Z");

				}

				if ((group < 1) && Id.length() < 1) {
					IEE.addEntry(MembraneXmlReader.TAGNAMES_MEMBRANE[group]
							+ "_ENTRY " + i + " - No Id found",null);
					continue;
				}

				// if (new File(PdbFile).canRead() == false) {
				//
				// IEE.addEntry(MembraneXmlReader.TAGNAMES_MEMBRANE[group]
				// + "_ENTRY " + i + " - Cannot read the File\n   "
				// + PdbFile);
				// continue;
				// }

				float x = 0;
				float z = 0;
				try {

					x = Float.parseFloat(Pos_X);
					z = Float.parseFloat(Pos_Z);

				} catch (final Exception e) {
					if (group < 3) {

						IEE.addEntry(MembraneXmlReader.TAGNAMES_MEMBRANE[group]
								+ "_ENTRY " + i + " - Invalid Position",e);
						continue;
					}
				}

				Molecule_3D molecule;

				// if Type has already been parsed, it won't be again...
				if ((parsedOnes
						.containsKey(new File(PdbFile).getAbsolutePath()) == false)
						) {

					

					try {
						
						if (group == 0) {
							molecule = reader.readProtein(new File(PdbFile),
									modelnumber, alteredAtoms);

						} else if (group > 0)
							molecule = reader.readLipid(new File(PdbFile),
									modelnumber, alteredAtoms);
						else
							break;

					} catch (final Exception e) {
e.printStackTrace();
						IEE.addEntry(MembraneXmlReader.TAGNAMES_MEMBRANE[group]
								+ "_ENTRY " + i
								+ " - An Error occured while reading\n   "
								+ PdbFile,e);
						continue;
					}
				}
				// ...instead an existing type will be cloned, what is way
				// faster
				else {
					 Molecule_3D toclone = (Molecule_3D) this.model
							.getFirstMoleculeThatHasFile(new File(PdbFile),
									group, modelnumber);

					
//					if(toclone!=null) {
//					final Atom[] altered = DataPool
//					.getAtomsWithAlteredPositions(toclone);
//			if (altered != null && altered.length > 0)
//				toclone = null;
//					}
					
					if (toclone != null) {
						molecule = toclone.copy(false,false);
						
						
						
						
						if (alteredAtoms != null) {
							final Point3f[] alteredPoints = new Point3f[alteredAtoms.size()];
							for (int j = 0; j < alteredAtoms.size(); j++) {
								try {
									final String[] s = alteredAtoms.get(j).split(" ");
									final int index = Integer.parseInt(s[0].trim());
									final Point3f p = new Point3f(
											Float.parseFloat(s[1].trim()), Float
													.parseFloat(s[2].trim()), Float
													.parseFloat(s[3].trim()));
									alteredPoints[index] = p;
								} catch (final Exception e) {

								}
							}
							for(int j=0;j<alteredAtoms.size();j++) {
								molecule.allAtomsSorted[j].position=alteredPoints[j];
							}
							
							for (int j = 0; j < molecule.currentCGParticles.size(); j++) {
								MyBounds my = new MyBounds(molecule.currentCGParticles.get(j).includedAtoms);
								molecule.currentCGParticles.get(j).position=my.center;
								
							}
						}
						
						
						
						
						
						
						
						
						
					}
					else {
						

						try {

							if (group == 0) {
								molecule = reader.readProtein(
										new File(PdbFile), modelnumber,
										alteredAtoms);

							} else if (group > 0)
								molecule = reader.readLipid(new File(PdbFile),
										modelnumber, alteredAtoms);
							else
								break;

						} catch (final Exception e) {
							e.printStackTrace();
							IEE
									.addEntry(MembraneXmlReader.TAGNAMES_MEMBRANE[group]
											+ "_ENTRY "
											+ i
											+ " - An Error occured while reading\n   "
											+ PdbFile,e);
							continue;
						}
					}

				}

				/*
				 * if the thread reaches this place, what means that the Entry
				 * was valid, the molecule gets its parameters and gets added to
				 * the model
				 */

				molecule.setPDBFile(new File(PdbFile));

				if (matrix3f != null) {
					molecule.setRotationMatrix(matrix3f);

				}
				molecule.setPosition(x, z);
				molecule.setYTranslation(translation);
				
				
				

				
				
				
				
				molecule.positionAfterReading=new Point3f(molecule.getPosition());

				molecule.setColor(color);
				molecule.setUseAtomColors(useAtomColors);
				molecule.setDomainID(domainID);
				molecule.setLayerID(LayerID);
				molecule.setModelSet(modelnumber);
				
				
				

				if (group > 0) {

					((Lipid_3D) molecule).setPercent(percent);

				} else {
					((Protein_3D) molecule).setID(Id);

					molecule.recalculatePoints();
				}

				if(!molecule.getCurrentProfileName().equals(Profile)) {
					
					String code=molecule.getPDBcode();
					if(code==null||code.length()<1)code=Cm2_Toolkit.getSimpleName(molecule.getPDBFile());
					
					try {
						
						MoleculeProfile prof = Molecule_Profile_IO.readProfile(molecule,Profile,code);
						prof.positions=new Point3f[0];
						molecule.applyProfile(prof);
						
						
						
						
						
//						MyBounds bb = new MyBounds(molecule.atoms);
//						
//										Point3f newPos=new Point3f(molecule.getPosition());
//										newPos.add(bb.center);
//						molecule.setPosition(newPos);
						
						
						
						
						molecule.updateGeometry();
						
						
					
					} catch (Exception e) {
						e.printStackTrace();
						
						IEE
							.addEntry(MembraneXmlReader.TAGNAMES_MEMBRANE[group]
									+ "_ENTRY "
									+ i
									+ " - Profile could not be read: "
									+ Profile
									+ "\n   Please place the file '"
									+ Profile
									+ ".cm2mp' " 
									+ "into the directory:"
									+ "\n   "+DataPool.workspace.getAbsolutePath()
									+ File.separator
									+ CM2_CONSTANTS.IO.MOLECULE_PROFILES_DIR.getPath()
									+ File.separator 
									+ code
									+ "\n   Meanwhile, the DEFAULT Profile is used.",e);
					}
				}
				
				
				

				
				
				
				if (group == 0)
					this.model.addProtein((Protein_3D) molecule, molecule
							.getLayerID());
				else if (group == 1)
					this.model.addLipidExtra((Lipid_3D) molecule, molecule
							.getDomainID(), molecule.getLayerID());
				else if (group == 2)
					this.model.addLipidIntra((Lipid_3D) molecule, molecule
							.getDomainID(), molecule.getLayerID());
				else if (group == 3)
					this.model.addExampleLipidExtra((Lipid_3D) molecule,
							molecule.getDomainID(), molecule.getLayerID());
				else
					this.model.addExampleLipidIntra((Lipid_3D) molecule,
							molecule.getDomainID(), molecule.getLayerID());

				// if (group > 0
				// && alreadyinforthisrunofloop.containsKey(PdbFile) == false) {
				// if (group == 1) {
				// model.addExampleLipidExtra((Lipid) molecule.clone());
				//
				// } else if (group == 2) {
				// model.addExampleLipidIntra((Lipid) molecule.clone());
				//
				// }
				//					
				// alreadyinforthisrunofloop.put(PdbFile, "");
				// }

				if ((parsedOnes.containsKey(PdbFile) == false)
						) {

					// register new Type

					parsedOnes.put(PdbFile, "");

				}

				finished++;

			}

		}
		// if at least one Entry was ignored, exception is thrown.
		if (IEE.getEntryCount() > 0)
			throw IEE;

	}

	/**
	 * Returns the MembraneModel that should have been filled before
	 * 
	 * @return
	 */
	public MembraneModel getResultingMembraneModel() {

		return this.model;
	}

	/**
	 * Reads in the passed file and returns Info
	 * 
	 * @param file
	 */
	public static MembraneMetaInfo readPreview(final File file)
			throws IOException, JDOMException {

		final Document doc = new SAXBuilder().build(file);

		final MembraneMetaInfo info = new MembraneMetaInfo();

		if (doc.getRootElement() == null)
			throw new JDOMException("No Root Element found!");
		final Element rootelement = doc.getRootElement();

		if (rootelement.getName().equals(
				MembraneXmlReader.ROOTELEMENTNAME_MEMBRANE) == false)
			throw new JDOMException("No Root Element found!");

		try {
			info.ProtCount = rootelement.getChild(
					MembraneXmlReader.TAGNAMES_MEMBRANE[0]).getChildren()
					.size();

		} catch (final Exception e) {

		}
		try {
			info.LipCountExtra = rootelement.getChild(
					MembraneXmlReader.TAGNAMES_MEMBRANE[1]).getChildren()
					.size();

		} catch (final Exception e) {

		}
		try {
			info.LipCountIntra = rootelement.getChild(
					MembraneXmlReader.TAGNAMES_MEMBRANE[2]).getChildren()
					.size();

		} catch (final Exception e) {

		}

		final List membraneattrs = rootelement
				.getChildren(MembraneXmlReader.TAGNAMES_MEMBRANE[5]);

		// reading the Attributes of the Membrane
		for (int i = 0; i < membraneattrs.size(); i++) {

			final Element ele = (Element) membraneattrs.get(i);

			if (ele.getAttributeValue("Header") != null)
				info.header = ele.getAttributeValue("Header");

			if (ele.getAttributeValue("PdbCode") != null)
				info.code = ele.getAttributeValue("PdbCode");

			if (ele.getAttributeValue("Author") != null)
				info.author = ele.getAttributeValue("Author");
			if (ele.getAttributeValue("Date") != null)
				info.date = ele.getAttributeValue("Date");

			if (ele.getAttributeValue("Width") != null
					&& ele.getAttributeValue("Height") != null) {

				try {
					final int x = Integer.parseInt(ele
							.getAttributeValue("Width"));
					final int y = Integer.parseInt(ele
							.getAttributeValue("Height"));
					info.size_x = x;
					info.size_y = y;

				} catch (final Exception e) {

				}
			}

		}

		return info;
	}

}
