/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithm;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import org.cellmicrocosmos.cm2.gui.ColumnSorter;
import org.jdom.Attribute;
import org.jdom.Comment;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;


/**
 * This class is for keeping various parameters as strings. They are stored in a
 * hashmap and they are put there by a key the author defines. Even more than
 * one Value can be added under the same key. They can be accessed eg. by
 * calling .getValue("key"); or .addValue("key","Value"); Adding a value to an
 * existing key will add the value, not replace it. To replace, call
 * .deleteAllAt("key"); first. The Value(s) of a key will be returned as
 * String[], even if only one exists. All values can be saved to a config file
 * by calling .writeDataToDisc(); All values will be restored by calling
 * .loadDataFromDisc();
 * 
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class ConfigIO {

	// stores all the Values
	private Hashtable<String, Vector<String>> hash = new Hashtable<String, Vector<String>>();

	// the config file.
	private File file;

	private boolean hasbeencorrectlyinitialized = false;

	// the name for the rootElement
	private final String rootElementName = "CM2_CONFIG";

	// the format for the writer
	final Format format = Format.getPrettyFormat();

	final String comment = "This is the config file used to keep some user defined Values.\n\n\n";

	/**
	 * Here, a user can add his Value(s) under his specified key. Calling this
	 * method more than once with the same key will store all Values by this
	 * key. When saved to the specified File, they will be concatenated with a
	 * seperator under the same tag
	 * 
	 * @param key
	 *            - the key for your value; will also be the name of the tag in
	 *            the cfg_file
	 * @param value
	 *            - the value as a string
	 */
	public void addValue(final String key, final String value) {

		Vector<String> vec = new Vector<String>();
		if (hash.get(key) != null)
			vec = hash.get(key);
		vec.addElement(value);
		hash.put(key, vec);

	}

	/**
	 * returns the value(s) under the given key/tagname as a String-Array. If
	 * theres only one Value stored by a key, an Array with a single element is
	 * returned
	 * 
	 * @param key
	 *            the key for your Value(s)
	 * @return The String(s)if available.
	 * @throws Exception
	 *             if no data is available
	 */
	public String[] getValue(final String key) throws Exception {

		Vector<String> vec;

		vec = hash.get(key);

		final String[] str = new String[vec.size()];

		for (int i = 0; i < vec.size(); i++)
			str[i] = vec.get(i);

		if (str.length < 1)
			throw new Exception();
		else
			return str;

	}

	/**
	 * deletes the Value(s) stored by the given key so that new ones can be
	 * added without keeping the old ones
	 * 
	 * @param key
	 *            - the key
	 */
	public void deleteAllAt(final String key) {

		if (hash.get(key) != null)
			hash.remove(key);

	}

	/**
	 * writes all values to the config-file
	 * 
	 * @throws IOException
	 */
	@SuppressWarnings("unchecked")
	public void writeDataToDisc() throws Exception {

		// create new Directory if not available yet
		if (!file.exists()) {
			final File tmp = new File(file.getParent());
			if (!tmp.isDirectory()) {
				if (!tmp.mkdirs()) {
					throw new IOException();
				}
			}
		}

		// preparing a Vector of keys that is applicable for the ColumnSorter
		final Vector<Vector<String>> keyList = new Vector<Vector<String>>();

		for (final Enumeration en = hash.keys(); en.hasMoreElements();) {
			final String string = (String) en.nextElement();

			final Vector<String> v = new Vector<String>();
			v.add(string);
			keyList.add(v);

		}

		// sort so that the configfile will look nicer
		Collections.sort(keyList, new ColumnSorter(0, true));

		final Document doc = new Document();

		final Element rootelement = new Element(rootElementName);

		for (int i = 0; i < keyList.size(); i++) {

			final Element mainelement = new Element(keyList.get(i).get(0));
			final Vector<String> values = hash.get(keyList.get(i).get(0));

			if (values.size() == 1) {
				final Attribute att = new Attribute("val", values.get(0));
				mainelement.setAttribute(att);

			} else {

				for (int j = 0; j < values.size(); j++) {

					final Element subelement = new Element("val_" + j);

					subelement.addContent(values.get(j));
					mainelement.addContent(subelement);

				}
			}
			rootelement.addContent(mainelement);

		}

		doc.setRootElement(rootelement);

		final Comment cmt = new Comment(comment);
		doc.getContent().add(0, cmt);

		final XMLOutputter outputter = new XMLOutputter();
		outputter.setFormat(format);
		final FileOutputStream output = new FileOutputStream(file);
		outputter.output(doc, output);
		output.close();

	}

	/**
	 * restores the Hasttable from the cfg_file. to protect agains inconsistence
	 * during runtime, the file can only be read once per runtime if data is
	 * already properly loaded(and only if!) calling this method will have no
	 * effect
	 * 
	 * @throws IOException
	 * 
	 * @throws Exception
	 */
	public void readDataFromDisc() throws Exception {

		if (!hasbeencorrectlyinitialized) {
			final Hashtable<String, Vector<String>> newHash = new Hashtable<String, Vector<String>>();

			final Document doc = new SAXBuilder().build(file);
			final Element rootelement = doc.getRootElement();

			if (rootelement.getName().equals(rootElementName) == false)
				throw new IOException();

			final List<?> children = rootelement.getChildren();

			for (int i = 0; i < children.size(); i++) {

				if (children.get(i) instanceof Element) {

					final Vector<String> vec = new Vector<String>();

					final Element mainelement = (Element) children.get(i);

					final List<?> subchildren = mainelement.getChildren();

					if (subchildren.size() == 0
							&& mainelement.getAttributeValue("val") != null) {
						vec.add(mainelement.getAttributeValue("val"));

					} else {

						for (int j = 0; j < subchildren.size(); j++) {

							final Element subelement = (Element) subchildren
									.get(j);

							vec.add(subelement.getValue());

						}

					}

					newHash.put(mainelement.getName(), vec);

				}

			}

			if (newHash.isEmpty())
				throw new IOException();

			hash = newHash;
			hasbeencorrectlyinitialized = true;
		}
	}

	/**
	 * Checks if the data has already been loaded properly from the file. if
	 * not, any requests to the data will definetly fail
	 * 
	 * @return the hasbeencorrectlyinitialized
	 */
	public boolean isCorrectlyInitialized() {
		return hasbeencorrectlyinitialized;
	}

	/**
	 * Sets the path of the Config File;
	 * 
	 * @param confile
	 */
	public void setConfigFile(final File confile) {
		file = confile;

	}

	/**
	 * Returns the config file
	 * 
	 * @return the Config file
	 */
	public File getConfigFile() {
		return file;
	}
}
