/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.misc;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Toolkit;
import java.awt.Transparency;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.channels.FileChannel;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.Random;
import java.util.Vector;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.jar.JarOutputStream;

import javax.media.j3d.Appearance;
import javax.media.j3d.BranchGroup;
import javax.media.j3d.Group;
import javax.media.j3d.Node;
import javax.media.j3d.RenderingAttributes;
import javax.media.j3d.Shape3D;
import javax.media.j3d.TransformGroup;
import javax.swing.BoxLayout;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.plaf.basic.BasicComboBoxEditor;
import javax.vecmath.Matrix3f;
import javax.vecmath.Point3f;
import javax.vecmath.Vector3f;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.Atom_3D;
import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.engine.Engine;
import org.cellmicrocosmos.cm2.io.FileIOHandler;
import org.cellmicrocosmos.cm2.io.PdbReader_3D;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.MembranePackingAlgorithm;

import net.infonode.docking.RootWindow;
import net.infonode.docking.util.DeveloperUtil;

import com.sun.tools.javac.Main;

/**
 * A Toolkit that offers various useful Methods
 * 
 * @author Tim Dingersen dackelnose@gmx.net, Sebastian Rubert, Bjoern Sommer bjoern@CELLmicrocosmos.org
 * 
 */
public class Cm2_Toolkit {

	/**
	 * Creates a copy of the passed matrix. Used as a workaround for the clone()
	 * method
	 * 
	 * @param m
	 * @return
	 */

	public static final char[] CHARS = { 'A', 'B', 'C', 'D', 'E', 'F', 'G',
			'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T',
			'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g',
			'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't',
			'u', 'v', 'w', 'x', 'y', 'z' };

	public static final String[] MONTHS = { "JAN", "FEB", "MAR", "APR", "MAY",
			"JUN", "JUL", "AUG", "SEP", "OCT", "NOV", "DEC" };

	public static Matrix3f copyMatrix(final Matrix3f m) {
		final Matrix3f matr = new Matrix3f();
		matr.m00 = m.m00;
		matr.m01 = m.m01;
		matr.m02 = m.m02;
		matr.m10 = m.m10;
		matr.m11 = m.m11;
		matr.m12 = m.m12;
		matr.m20 = m.m20;
		matr.m21 = m.m21;
		matr.m22 = m.m22;
		return matr;
	}

	/**
	 * Returns the simple name without the file extension
	 * 
	 * @return
	 */
	public static String getSimpleName(final File file) {

		String name = file.getName();
		// TODO this is a workaround needed for file names coming from Windows
		//      and read on a Linux system
		if (name.contains("\\"))
			name = name.substring(name.lastIndexOf("\\")+1);
		if (name.contains("/"))
			name = name.substring(name.lastIndexOf("/")+1);
		//      end of the workaround
		return name.substring(0, name.lastIndexOf("."));

	}

	/**
	 * This is for development purposes only.
	 * Shows the layout in a seperate Frame
	 * 
	 */
	public static void popUpLayoutDescription(final RootWindow root) {
		DeveloperUtil.createWindowLayoutFrame("Layout", root).setVisible(true);
	}


	/**
	 * Calculates the passed array of numbers
	 * 
	 * @param newval
	 * @param integers
	 * @return
	 */
	public static int[] calcPercentages(final int reference, final int[] olds) {
		final int divisor = (int) Math.pow(10, DataPool.getMembraneModel()
				.getDecimalPlaces());

		int[] newones = Cm2_Toolkit.calculatePercentages(reference, olds);

		while (true) {

			int total = 0;

			for (int i = 0; i < newones.length; i++) {
				total += newones[i];
			}

			if (total == 100 * divisor)
				return newones;
			else {
				newones = Cm2_Toolkit.calculatePercentages(reference, newones);
			}
		}

	}

	private static int[] calculatePercentages(final int ref, final int[] ints) {

		final MembraneModel model = DataPool.getMembraneModel();
		final int divisor = (int) Math.pow(10, model.getDecimalPlaces());
		if (ints.length < 1)
			return ints;

		if (ints.length == 1) {
			ints[0] = 100 * divisor;

			return ints;
		}

		int total = 0;

		for (int i = 0; i < ints.length; i++) {
			total += ints[i];
		}

		if (total == 100 * divisor)
			return ints;

		int diff = 100 * divisor - total;

		if (diff > 0) {

			int index = 0;
			int extreme = 100 * divisor;

			for (int i = 0; i < ints.length; i++) {
				int min;
				min = 100 * divisor;
				if (i != ref)
					min = Math.min(ints[i], extreme);
				if (min < extreme) {
					extreme = min;
					index = i;
				}
			}

			for (int i = 0; i < diff; i++) {
				try {
					if (index == ref) {
						index++;

					}
					ints[index] += 1;

					if (index >= ints.length)
						index = 0;
					else
						index++;
				} catch (final Exception e) {
					i--;
					index = 0;
				}

			}
		}

		if (diff < 0) {

			diff *= -1;

			int index = 0;
			int extreme = 0;

			for (int i = 0; i < ints.length; i++) {
				int max = 0;
				if (i != ref)
					max = Math.max(ints[i], extreme);
				if (max > extreme) {
					extreme = max;
					index = i;
				}
			}

			for (int i = 0; i < diff; i++) {
				try {
					if (index == ref) {
						index++;

					}

					ints[index] -= 1;

					if (index >= ints.length)
						index = 0;
					else
						index++;
				} catch (final Exception e) {
					i--;
					index = 0;
				}

			}

		}

		for (int i = 0; i < ints.length; i++) {
			if (ints[i] < 1) {
				int toadd = 0 - ints[i];

				ints[i] += toadd;
				int index = 0;
				while (toadd > 0) {
					if (index != ref) {
						ints[index]++;
						toadd--;
					}
					index++;
					if (index >= ints.length)
						index = 0;
				}

			}
		}

//		int t = 0;
//		for (int i = 0; i < ints.length; i++) {
//			t += ints[i];
//		}

		for (int i = 0; i < ints.length; i++) {
			if (ints[i] < 1) {

				ints[i]++;
				if (i + 1 < ints.length)
					ints[i + 1]--;
				else if (i - 1 > -1)
					ints[i - 1]--;
				else
					ints[i]--;
			}

		}
		return ints;
	}

	/**
	 * Makes the passed Filesize to a string
	 * 
	 * @param d
	 * @return
	 */
	public static String convertFileSizeToString(final long d) {
		String ext;
		if (d < 1000000)
			ext = " KB";
		else
			ext = " MB";

		if (d == 0)
			return 0 + ext;

		String ret = ((float) d) / ((float) 1000000) + "";

		if (d >= 1000000) {
			try {
				final int index = ret.indexOf(".");

				ret = ret.substring(0, index + 2);
			} catch (final Exception e) {
			}
		} else {
			ret = ret.substring(2);
			if (ret.length() > 3)
				ret = ret.substring(0, 3);
			if (ret.startsWith("0"))
				ret = ret.replaceFirst("0", "");
			if (ret.startsWith("0"))
				ret = ret.replaceFirst("0", "");

		}

		return ret + ext;
	}

	/**
	 * Get a randomly created Color for Proteins
	 * 
	 * @return
	 */
	public static Color getRandomProteinColor() {

		return Cm2_Toolkit.createRandomColor(0);
	}

	/**
	 * Get a randomly created Color for Lipids
	 * 
	 * @return
	 */
	public static Color getRandomLipidColor() {

		return Cm2_Toolkit.createRandomColor(1);
	}

	/**
	 * Generates a random color for either Proteins or Lipids
	 * 
	 * @param mode
	 *            0=Pro, 1=Lip
	 * @return
	 */
	private static Color createRandomColor(final int mode) {

		Color sample;

		int rc = new Random().nextInt(150);
		int minus;
		if (mode == 0)
			minus = new Random().nextInt(100);
		else
			minus = new Random().nextInt(50);
		rc += 20;

		int minus2 = new Random().nextInt(60);
		minus2 -= 15;
		final int rc2 = rc + minus2;

		if (mode == 0) {
			if (new Random().nextInt(3) < 2)
				sample = new Color(rc, 255 - minus, rc2);
			else
				sample = new Color(rc2 + 30, rc + 50, 255 - minus);

		} else if (new Random().nextBoolean())
			sample = new Color(255 - minus, rc, rc2 / 2);
		else
			sample = new Color(250 - minus / 4, 210 - minus, 0);
		return sample;
	}

	public static Color createRandomCholesterolColor() {

		final int c = 80 + new Random().nextInt(80);

		return new Color(c, c, c);
	}

	/**
	 * Rotates a point according to the passed matrix. The passed point, which
	 * represents the starting location wont be changed; a new point will be
	 * returned instead. So its up to the caller to replace the old point with
	 * the returned value or to use it for something else.
	 * 
	 * Besides, this method proves to work better than the transform of Matrix3f
	 * 
	 * @param matrix
	 * @param point3f
	 * @return
	 */
	public static Point3f transformToNewPoint(final Matrix3f matrix,
			final Point3f point3f) {
		float x;
		float y;
		float z;

		x = matrix.m00 * point3f.x + matrix.m01 * point3f.y + matrix.m02
				* point3f.z;
		y = matrix.m10 * point3f.x + matrix.m11 * point3f.y + matrix.m12
				* point3f.z;
		z = matrix.m20 * point3f.x + matrix.m21 * point3f.y + matrix.m22
				* point3f.z;

		return new Point3f(x, y, z);

	}

	public static Matrix3f createAUnitMatrix() {
		final Matrix3f matrix = new Matrix3f();
		matrix.m00 = 1;
		matrix.m11 = 1;
		matrix.m22 = 1;
		return matrix;
	}

	public static void rotate_X_Axis_Of_CoordinateSystem(
			final Matrix3f current, final float angle) {

		final float rad = (float) Math.toRadians((angle));

		float backZ = 0;
		float backX = 0;
		float backY = 0;

		final Point3f rz = new Point3f(0, 0, 1);
		final Point3f rx = new Point3f(1, 0, 0);

		while (true) {
			
			final Vector3f Vz = new Vector3f(Cm2_Toolkit.transformToNewPoint(
					current, rz));
			

			if (Math.abs(Vz.y) < 0.001f) {
				backZ += Math.abs(Vz.y);
				break;
			}
			final float angZ = new Vector3f(Vz.x, 0, Vz.z).angle(Vz);
			Cm2_Toolkit.rot_X_Axis(current, angZ);
			backZ += angZ;
		}

		while (true) {

			final Vector3f Vx = new Vector3f(Cm2_Toolkit.transformToNewPoint(
					current, rx));

			if (Math.abs(Vx.y) < 0.001f) {
				backX += Math.abs(Vx.y);
				break;
			}
			final float angX = new Vector3f(Vx.x, 0, Vx.z).angle(Vx);
			Cm2_Toolkit.rot_Z_Axis(current, angX);
			backX += angX;
		}

		while (true) {

			final Vector3f Vx = new Vector3f(Cm2_Toolkit.transformToNewPoint(
					current, rx));

			if (Math.abs(Vx.z) < 0.001f) {
				backY += Math.abs(Vx.z);
				break;
			}
			final float angX = new Vector3f(rx).angle(Vx);
			Cm2_Toolkit.rot_Y_Axis(current, angX);
			backY += angX;
		}

		Cm2_Toolkit.rot_X_Axis(current, rad);

		Cm2_Toolkit.rot_Y_Axis(current, -backY);
		Cm2_Toolkit.rot_Z_Axis(current, -backX);
		Cm2_Toolkit.rot_X_Axis(current, -backZ);

	}

	public static void rotate_Y_Axis_Of_CoordinateSystem(
			final Matrix3f current, final float angle) {

		final float rad = (float) Math.toRadians((angle));

		float backZ = 0;
		float backX = 0;

		final Point3f rz = new Point3f(0, 0, 1);
		final Point3f rx = new Point3f(1, 0, 0);

		while (true) {

			final Vector3f Vz = new Vector3f(Cm2_Toolkit.transformToNewPoint(
					current, rz));

			if (Math.abs(Vz.y) < 0.001f) {
				backZ += Math.abs(Vz.y);
				break;
			}
			final float angZ = new Vector3f(Vz.x, 0, Vz.z).angle(Vz);
			Cm2_Toolkit.rot_X_Axis(current, angZ);
			backZ += angZ;
		}

		while (true) {

			final Vector3f Vx = new Vector3f(Cm2_Toolkit.transformToNewPoint(
					current, rx));

			if (Math.abs(Vx.y) < 0.001f) {
				backX += Math.abs(Vx.y);
				break;
			}
			final float angX = new Vector3f(Vx.x, 0, Vx.z).angle(Vx);
			Cm2_Toolkit.rot_Z_Axis(current, angX);
			backX += angX;
		}

		Cm2_Toolkit.rot_Y_Axis(current, rad);

		Cm2_Toolkit.rot_Z_Axis(current, -backX);

		Cm2_Toolkit.rot_X_Axis(current, -backZ);

	}

	public static void rotate_Z_Axis_Of_CoordinateSystem(
			final Matrix3f current, final float angle) {

		final float rad = (float) Math.toRadians((angle));

		float backZ = 0;
		float backX = 0;
		float backY = 0;

		final Point3f rz = new Point3f(0, 0, 1);
		final Point3f rx = new Point3f(1, 0, 0);

		while (true) {

			final Vector3f Vz = new Vector3f(Cm2_Toolkit.transformToNewPoint(
					current, rz));

			if (Math.abs(Vz.y) < 0.001f) {
				backZ += Math.abs(Vz.y);
				break;
			}
			final float angZ = new Vector3f(Vz.x, 0, Vz.z).angle(Vz);
			Cm2_Toolkit.rot_X_Axis(current, angZ);
			backZ += angZ;
		}

		while (true) {

			final Vector3f Vx = new Vector3f(Cm2_Toolkit.transformToNewPoint(
					current, rx));

			if (Math.abs(Vx.y) < 0.001f) {
				backX += Math.abs(Vx.y);
				break;
			}
			final float angX = new Vector3f(Vx.x, 0, Vx.z).angle(Vx);
			Cm2_Toolkit.rot_Z_Axis(current, angX);
			backX += angX;
		}

		while (true) {

			final Vector3f Vz = new Vector3f(Cm2_Toolkit.transformToNewPoint(
					current, rz));

			if (Math.abs(Vz.x) < 0.001f) {
				backY += Math.abs(Vz.x);
				break;
			}
			final float angX = new Vector3f(rz).angle(Vz);
			Cm2_Toolkit.rot_Y_Axis(current, angX);
			backY += angX;
		}

		Cm2_Toolkit.rot_Z_Axis(current, rad);

		Cm2_Toolkit.rot_Y_Axis(current, -backY);

		Cm2_Toolkit.rot_Z_Axis(current, -backX);
		Cm2_Toolkit.rot_X_Axis(current, -backZ);

	}

//	// this was used for some experiments, may be deleted again
//	public static void rotate_Z_Axis_Of_CoordinateSystem(
//			final Matrix3f current, final float angle, final float height) {
//
//		final float rad = (float) Math.toRadians((angle));
//
//		float backZ = 0;
//		float backX = 0;
//		float backY = 0;
//
//		final Point3f rz = new Point3f(0, 0, 1);
//		final Point3f rx = new Point3f(1, 0, 0);
//
//		while (true) {
//
//			final Vector3f Vz = new Vector3f(Cm2_Toolkit.transformToNewPoint(
//					current, rz));
//
//			if (Math.abs(Vz.y) < 0.001f) {
//				backZ += Math.abs(Vz.y);
//				break;
//			}
//			final float angZ = new Vector3f(Vz.x, 0, Vz.z).angle(Vz);
//			Cm2_Toolkit.rot_X_Axis(current, angZ);
//			backZ += angZ;
//		}
//
//		while (true) {
//
//			final Vector3f Vx = new Vector3f(Cm2_Toolkit.transformToNewPoint(
//					current, rx));
//
//			if (Math.abs(Vx.y) < 0.001f) {
//				backX += Math.abs(Vx.y);
//				break;
//			}
//			final float angX = new Vector3f(Vx.x, 0, Vx.z).angle(Vx);
//			Cm2_Toolkit.rot_Z_Axis(current, angX);
//			backX += angX;
//		}
//
//		while (true) {
//
//			final Vector3f Vz = new Vector3f(Cm2_Toolkit.transformToNewPoint(
//					current, rz));
//
//			if (Math.abs(Vz.x) < 0.001f) {
//				backY += Math.abs(Vz.x);
//				break;
//			}
//			final float angX = new Vector3f(rz).angle(Vz);
//			Cm2_Toolkit.rot_Y_Axis(current, angX);
//			backY += angX;
//		}
//
//		Cm2_Toolkit.rot_Z_Axis(current, rad);
//
//		Cm2_Toolkit.rot_Y_Axis(current, -backY);
//
//		Cm2_Toolkit.rot_Z_Axis(current, -backX);
//		Cm2_Toolkit.rot_X_Axis(current, -backZ);
//
//	}

	public static void rotate_X_Axis_Matrix(final Matrix3f current,
			final float angle) {
		final float rad = (float) Math.toRadians((angle));
		Cm2_Toolkit.rot_X_Axis(current, rad);
	}

	public static void rotate_Y_Axis_Matrix(final Matrix3f current,
			final float angle) {
		final float rad = (float) Math.toRadians((angle));
		Cm2_Toolkit.rot_Y_Axis(current, rad);
	}

	public static void rotate_Z_Axis_Matrix(final Matrix3f current,
			final float angle) {
		final float rad = (float) Math.toRadians((angle));
		Cm2_Toolkit.rot_Z_Axis(current, rad);
	}

	private static void rot_X_Axis(final Matrix3f current, final float angle) {

		final Matrix3f newone = new Matrix3f();
		newone.m00 = 1f;
		newone.m11 = (float) Math.cos(angle);
		newone.m12 = -(float) Math.sin(angle);
		newone.m21 = (float) Math.sin(angle);
		newone.m22 = (float) Math.cos(angle);

		current.mul(newone);

	}

	private static void rot_Y_Axis(final Matrix3f current, final float angle) {

		final Matrix3f newone = new Matrix3f();
		newone.m00 = (float) Math.cos(angle);
		newone.m02 = (float) Math.sin(angle);
		newone.m11 = 1f;
		newone.m20 = -(float) Math.sin(angle);
		newone.m22 = (float) Math.cos(angle);

		current.mul(newone);

	}

	private static void rot_Z_Axis(final Matrix3f current, final float angle) {

		final Matrix3f newone = new Matrix3f();
		newone.m00 = (float) Math.cos(angle);
		newone.m01 = -(float) Math.sin(angle);
		newone.m10 = (float) Math.sin(angle);
		newone.m11 = (float) Math.cos(angle);
		newone.m22 = 1f;

		current.mul(newone);

	}

	public static int getCharacterPosOf(final char c) {
		for (int i = 0; i < Cm2_Toolkit.CHARS.length; i++)
			if (Cm2_Toolkit.CHARS[i] == c)
				return i;

		return -1;
	}

	public static String createCurrentDate() {
		String day = "" + Calendar.getInstance().get(Calendar.DAY_OF_MONTH);
		if (day.length() < 2)
			day = "0" + day;
		final String string = day
				+ "-"
				+ Cm2_Toolkit.MONTHS[Calendar.getInstance().get(Calendar.MONTH)]
				+ "-"
				+ ("" + Calendar.getInstance().get(Calendar.YEAR)).substring(2);
		return string;
	}

	public static String extendLeft(final String sample, final int cols) {
		if (sample == null)
			return null;
		if (cols < sample.length()) {
			return sample.substring(sample.length() - cols);
		}

		String newone = "";
		final int n = cols - sample.length();
		for (int i = 0; i < n; i++) {
			newone += " ";
		}
		return newone + sample;
	}

	public static String extendRight(final String sample, final int cols) {
		if (sample == null)
			return null;
		if (cols < sample.length()) {
			return sample.substring(0, cols);
		}

		String newone = "";

		final int n = cols - sample.length();
		for (int i = 0; i < n; i++) {
			newone += " ";
		}
		return sample + newone;
	}

	/**
	 * Tries to display the passed url in the Default OS Webbrowser
	 * 
	 */
	public static void openWebPageOnSystemBrowser(final String url) {

		final String OSname = System.getProperty("os.name");

		String command;
		try {
			if (OSname != null && OSname.startsWith("Windows")) {

				command = "rundll32 url.dll,FileProtocolHandler " + url;
				Runtime.getRuntime().exec(command);
			} else {

				command = "netscape -remote openURL(" + url + ")";
				Process p = Runtime.getRuntime().exec(command);
				try {
					// waiting for the proper exit code. If it returns 0, it
					// worked,
					// if not, we need to start the browser up

					if (p.waitFor() != 0) {

						command = "netscape " + url;
						p = Runtime.getRuntime().exec(command);
					}
				} catch (final Exception e) {
					Cm2_Toolkit.complainBrowser();
				}
			}
		} catch (final Exception e) {
			Cm2_Toolkit.complainBrowser();
		}

	}

	private static void complainBrowser() {
		complain("Unable to activate the System Browser!\n"
						+ "You should copy/paste the link into a\n"
						+ "browser manually", "No Browser found");
	}
	
	public static void complainInternet() {
		complain("Unable to connect to the Internet.\nPlease check the connection on your computer\n"
						+ "and Settings -> Connection in CmME.\nProbably the Proxy settings are wrong.\n"
						, "No Internet Connection");
	}
	
	public static void complainXML(String fileName) {
		complain("Unable to open the file "+fileName+".\nPlease check the file.\n"
						, "Problems Loading File");
	}
	
	public static void complain(String text, String title) {
		JOptionPane.showMessageDialog(null,
				text, title, JOptionPane.ERROR_MESSAGE);
	}

	public static class Membrane_Image2D_Generator {

		public static final int MODE_POLYGON = 0;

		public static final int MODE_ATOMS = 1;

		public static final int MODE_HYBRID = 2;

		public static BufferedImage create2DImageOfCurrentMembrane(
				final MembraneModel model, final int mode,
				final boolean drawLipidsToo,
				final boolean useRealLipidAtomColors, final boolean drawLabels,int layerID) {

			final Vector<Point> ps = new Vector<Point>();
			final Vector<Color> cs = new Vector<Color>();
			final Vector<Integer> rs = new Vector<Integer>();
			final float as = 1 / CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;
			final float scale = Membrane_Image2D_Generator
					.createScaleFor2DImage();

			final Dimension size = new Dimension(
					(int) (model.getSize().width * scale), (int) (model
							.getSize().height * scale));

			if (drawLipidsToo
					&& mode != Membrane_Image2D_Generator.MODE_POLYGON) {
				for (int i = 0; i < model.getLipidCountIntra(); i++) {
					final Molecule_3D mol = model.getLipidAtIntra(i);
					if(mol.getLayerID()!=layerID&&mol.getLayerID()!=-1)continue;
					final Atom_3D[] atoms = mol.getAtoms(true, false);

					for (int a = 0; a < 3; a++) {
						for (int j = 0; j < atoms.length; j++) {

							int add = (int) ((atoms[j].position.y * scale) / 10);
							if (a == 0 && add > -1)
								continue;
							if (a == 1 && add < -1 && add > 1)
								continue;
							if (a == 2 && add < 1)
								continue;
							if (add < -1)
								add = -1;
							else if (add > 1)
								add = 1;
							else
								add = 0;
							rs.addElement(new Integer(add));
							ps.addElement(new Point(
									(int) (atoms[j].position.x * scale),
									(int) (atoms[j].position.z * scale)));

							Color color = mol.getColor();
							if (useRealLipidAtomColors) {
								char c;
								try {
									c = atoms[j].name.trim().toUpperCase()
											.charAt(0);
								} catch (final Exception e) {
									c = 0;
								}
								if (c == 'C')
									color = Color.gray;
								else if (c == 'O')
									color = Color.red;
								else if (c == 'P')
									color = Color.orange;
								else if (c == 'N')
									color = Color.blue;
								else if (c == 'S')
									color = Color.yellow;
								else
									color = Color.LIGHT_GRAY;
							}

							cs.addElement(color);

						}
					}
				}

			}
			if (mode != Membrane_Image2D_Generator.MODE_POLYGON) {
				for (int i = 0; i < model.getProteinCount(); i++) {
					final Molecule_3D mol = model.getProteinAt(i);
					if(mol.getLayerID()!=layerID&&mol.getLayerID()!=-1)continue;
					final Atom_3D[] atoms = mol.getAtoms(true, false);

					for (int a = 0; a < 3; a++) {
						for (int j = 0; j < atoms.length; j++) {

							int add = (int) ((atoms[j].position.y * scale) / 10);
							if (a == 0 && add > -1)
								continue;
							if (a == 1 && add < -1 && add > 1)
								continue;
							if (a == 2 && add < 1)
								continue;
							if (add < -1)
								add = -1;
							else if (add > 1)
								add = 1;
							else
								add = 0;
							rs.addElement(new Integer(add));
							ps.addElement(new Point(
									(int) (atoms[j].position.x * scale),
									(int) (atoms[j].position.z * scale)));

							final Color color = mol.getColor();

							cs.addElement(color);

						}
					}
				}
			}

			if (drawLipidsToo
					&& mode != Membrane_Image2D_Generator.MODE_POLYGON) {
				for (int i = 0; i < model.getLipidCountExtra(); i++) {
					final Molecule_3D mol = model.getLipidAtExtra(i);
					if(mol.getLayerID()!=layerID&&mol.getLayerID()!=-1)continue;
					final Atom_3D[] atoms = mol.getAtoms(true, false);

					for (int a = 0; a < 3; a++) {
						for (int j = 0; j < atoms.length; j++) {

							int add = (int) ((atoms[j].position.y * scale) / 10);
							if (a == 0 && add > -1)
								continue;
							if (a == 1 && add < -1 && add > 1)
								continue;
							if (a == 2 && add < 1)
								continue;
							if (add < -1)
								add = -1;
							else if (add > 1)
								add = 1;
							else
								add = 0;
							rs.addElement(new Integer(add));
							ps.addElement(new Point(
									(int) (atoms[j].position.x * scale),
									(int) (atoms[j].position.z * scale)));

							Color color = mol.getColor();
							if (useRealLipidAtomColors) {
								char c;
								try {
									c = atoms[j].name.trim().toUpperCase()
											.charAt(0);
								} catch (final Exception e) {
									c = 0;
								}
								if (c == 'C')
									color = Color.gray;
								else if (c == 'O')
									color = Color.red;
								else if (c == 'P')
									color = Color.orange;
								else if (c == 'N')
									color = Color.blue;
								else if (c == 'S')
									color = Color.yellow;
								else
									color = Color.LIGHT_GRAY;
							}

							cs.addElement(color);

						}
					}
				}

			}

			final Point[] points = ps.toArray(new Point[0]);
			final Color[] colors = cs.toArray(new Color[0]);
			final Integer[] adds = rs.toArray(new Integer[0]);

			final BufferedImage bi = new BufferedImage(size.width, size.height,
					Transparency.TRANSLUCENT);
			final Graphics2D g = bi.createGraphics();

			// g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER,
			// 0.1f));
			// g.setColor(DataPool.J3DSettings.bright_appearance ? Color.white
			// : Color.black);
			// g.fillRect(0, 0, size.width, size.height);

			for (int i = 0; i < points.length; i++) {

				g.setColor(colors[i]);
				g.fillOval((points[i].x - 3) + adds[i], (points[i].y - 3)
						+ adds[i], (3 + adds[i]) * 2, (3 + adds[i]) * 2);

				g.setColor(Color.DARK_GRAY);
				g.drawOval((points[i].x - 3) + adds[i], (points[i].y - 3)
						+ adds[i], (3 + adds[i]) * 2, (3 + adds[i]) * 2);

			}

			if (mode == Membrane_Image2D_Generator.MODE_POLYGON) {

				if (drawLipidsToo) {

					for (int i = 0; i < model.getLipidCountIntra(); i++) {
						final Molecule_3D mol = model.getLipidAtIntra(i);
						if(mol.getLayerID()!=layerID&&mol.getLayerID()!=-1)continue;
						final Atom_3D[] atoms = mol.getAtoms(true, false);
						final Point3f[] apoints = new Point3f[atoms.length];
						for (int j = 0; j < atoms.length; j++) {
							apoints[j] = atoms[j].position;
						}
						final Point3f[] polypoints = new PdbReader_3D()
								.calculateLevelExtremes(apoints);
						final Polygon p = new Polygon();
						for (int j = 0; j < polypoints.length; j++) {

							final Point3f a = new Point3f(polypoints[j]);
							a.scale(as);
							a.scale(scale);

							p.addPoint((int) a.x, (int) a.z);

						}

						g.setColor(mol.getColor());
						g.fillPolygon(p);
					}
				}

				for (int i = 0; i < model.getProteinCount(); i++) {
					final Molecule_3D mol = model.getProteinAt(i);
					if(mol.getLayerID()!=layerID&&mol.getLayerID()!=-1)continue;
					final Atom_3D[] atoms = mol.getAtoms(true, false);
					final Point3f[] apoints = new Point3f[atoms.length];
					for (int j = 0; j < atoms.length; j++) {
						apoints[j] = atoms[j].position;
					}
					final Point3f[] polypoints = new PdbReader_3D()
							.calculateLevelExtremes(apoints);
					final Polygon p = new Polygon();
					for (int j = 0; j < polypoints.length; j++) {

						final Point3f a = new Point3f(polypoints[j]);

						a.scale(scale);

						p.addPoint((int) a.x, (int) a.z);

					}

					g.setColor(mol.getColor());
					g.fillPolygon(p);
				}

				if (drawLipidsToo) {

					for (int i = 0; i < model.getLipidCountExtra(); i++) {
						final Molecule_3D mol = model.getLipidAtExtra(i);
						if(mol.getLayerID()!=layerID&&mol.getLayerID()!=-1)continue;
						final Atom_3D[] atoms = mol.getAtoms(true, false);
						final Point3f[] apoints = new Point3f[atoms.length];
						for (int j = 0; j < atoms.length; j++) {
							apoints[j] = atoms[j].position;
						}
						final Point3f[] polypoints = new PdbReader_3D()
								.calculateLevelExtremes(apoints);
						final Polygon p = new Polygon();
						for (int j = 0; j < polypoints.length; j++) {

							final Point3f a = new Point3f(polypoints[j]);

							a.scale(scale);

							p.addPoint((int) a.x, (int) a.z);

						}

						g.setColor(mol.getColor());
						g.fillPolygon(p);
					}
				}

			}

			if (drawLabels) {
				final Font font = new Font("Serif", Font.BOLD, 16);
				g.setFont(font);
				for (int i = 0; i < model.getProteinCount(); i++) {
					final Molecule_3D mol = model.getProteinAt(i);
					if(mol.getLayerID()!=layerID&&mol.getLayerID()!=-1)continue;

					final Point pos = new Point((int) (scale * (mol
							.getPosition().x)), (int) (scale * (mol
							.getPosition().z)));

					g
							.setColor(DataPool.J3DSettings.bright_appearance ? Color.black
									: Color.white);
					final JLabel l = new JLabel(mol.getPDBcode());
					l.setFont(font);

					g.drawString(mol.getPDBcode(), pos.x
							- (l.getPreferredSize().width / 2) - 2, pos.y
							+ (l.getPreferredSize().height / 2) - 2);
					g.drawString(mol.getPDBcode(), pos.x
							- (l.getPreferredSize().width / 2) - 2, pos.y
							+ (l.getPreferredSize().height / 2) + 2);
					g.drawString(mol.getPDBcode(), pos.x
							- (l.getPreferredSize().width / 2) + 2, pos.y
							+ (l.getPreferredSize().height / 2) + 2);
					g.drawString(mol.getPDBcode(), pos.x
							- (l.getPreferredSize().width / 2) + 2, pos.y
							+ (l.getPreferredSize().height / 2) - 2);

					g
							.setColor(DataPool.J3DSettings.bright_appearance ? Color.white
									: Color.black);

					g.drawString(mol.getPDBcode(), pos.x
							- (l.getPreferredSize().width / 2), pos.y
							+ (l.getPreferredSize().height / 2));
				}

			}

			if (mode == Membrane_Image2D_Generator.MODE_HYBRID) {
				for (int i = 0; i < model.getProteinCount(); i++) {
					final Molecule_3D mol = model.getProteinAt(i);
					if(mol.getLayerID()!=layerID&&mol.getLayerID()!=-1)continue;
					final Point3f[] polypoints = new PdbReader_3D()
							.calculateLevelExtremes(mol
									.getAlignedTrianglePoints());

					for (int j = 0; j < polypoints.length; j++) {
						Point3f p1;
						Point3f p2;
						if (j < polypoints.length - 1) {
							p1 = polypoints[j];
							p2 = polypoints[j + 1];
						} else {
							p1 = polypoints[polypoints.length - 1];
							p2 = polypoints[0];
						}

						final Point3f a = new Point3f(p1);
						a.scale(as);
						a.scale(scale);

						final Point3f b = new Point3f(p2);
						b.scale(as);
						b.scale(scale);

						g
								.setColor(DataPool.J3DSettings.bright_appearance ? Color.black
										: Color.white);

						g.drawLine((int) a.x - 1, (int) a.z - 1, (int) b.x - 1,
								(int) b.z - 1);
						g.drawLine((int) a.x + 1, (int) a.z + 1, (int) b.x + 1,
								(int) b.z + 1);

						g.setColor(Color.orange);
						g.drawLine((int) a.x, (int) a.z, (int) b.x, (int) b.z);

					}
				}

			}

			return bi;
		}

		public static float createScaleFor2DImage() {
			final MembraneModel model = DataPool.getMembraneModel();
			final Dimension scr = Toolkit.getDefaultToolkit().getScreenSize();
			final int max = (scr.height / 3) * 2;
			final Dimension mdlw = model.getSize();
			if (mdlw.width >= mdlw.height) {
				return (float) max / (float) mdlw.width;
			} else {
				return (float) max / (float) mdlw.height;
			}

		}

		public static Image createImage(final Molecule_3D mol, final int prefX,
				final int prefY) {

			final Atom_3D[] allatoms = mol.getAtoms(true, true);
			final Vector<Atom_3D> atoms = new Vector<Atom_3D>();

			final Point3f lower = mol.getLower(true);
			final Point3f upper = mol.getUpper(true);

			final float minx = lower.x;
			final float miny = lower.y;
			final float maxx = upper.x;
			final float maxy = upper.y;

			final float w = maxx - minx;
			final float h = maxy - miny;

			float borderEW = 5;
			float borderNS = 5;

			if (w > h) {

				borderNS += (w - h) / 2;

			} else if (h > w) {
				borderEW += (h - w) / 2;
			}

			int inc = 2;
			inc += ((allatoms.length / 100) / 3);

			final float rw = (int) (w + 2 * borderEW);
			final float rh = (int) (h + 2 * borderNS);

			final float scaleX = prefX / rw;
			final float scaleY = prefY / rh;

			final BufferedImage bi = new BufferedImage((int) (rw * scaleX),
					(int) (rh * scaleY), BufferedImage.TYPE_INT_RGB);

			for (int i = 0; i < allatoms.length; i += inc) {

				allatoms[i].position.x -= minx;
				allatoms[i].position.x += borderEW;
				allatoms[i].position.x *= scaleX;
				allatoms[i].position.y -= miny;
				allatoms[i].position.y += borderNS;
				allatoms[i].position.y *= scaleY;

				allatoms[i].position.y *= -1;
				allatoms[i].position.y += bi.getHeight();

				atoms.add(allatoms[i]);
			}

			final Graphics g = bi.getGraphics();

			g.setColor(Color.white);
			g.fillRect(0, 0, bi.getWidth(), bi.getHeight());

			final int length = atoms.size();
			for (int i = 0; i < length; i++) {

				final Atom_3D atom = atoms.get(i);

				g.setColor(atom.color);

				final Point p = new Point((int) atom.position.x,
						(int) atom.position.y);

				g.fillOval(p.x - 3, p.y - 3, 6, 6);

				g.setColor(Color.DARK_GRAY);
				g.drawOval(p.x - 3, p.y - 3, 6, 6);

			}

			return bi.getScaledInstance(prefX, prefY, Image.SCALE_FAST);

		}
	}

	public static void extractFromJar(String source, String dest)
			throws Exception {
		
//		File extract = new File(source+"extract.jar"); // original
		// TODO check if this works also on Linux and Mac
		File extract = new File(dest+"_extract.jar");
		if (extract.exists()) {
			extract.delete();
			extract.createNewFile();
		}
		
		 InputStream input = Thread.currentThread()
				.getContextClassLoader().getResourceAsStream(source);

		// TODO Does this change make problems?
		extract.deleteOnExit();
		
		if(input==null) {
			input=new File(source).toURI().toURL().openStream();
		}
		final FileOutputStream os = new FileOutputStream(
				extract);
		
		byte[] bytes = new byte[1024];
		while (true) {
			final int bytesread = input.read(bytes);
			if (bytesread < 0)
				break;
			os.write(bytes, 0, bytesread);

		}
		
		
		// while (input.available() > 0) {
		// os.write(input.read());
		// }
		os.close();
		input.close();
		
		final JarFile jar = new JarFile(extract);

		final Enumeration<JarEntry> en = jar.entries();
		new File(dest).mkdirs();
		
		while (en.hasMoreElements()) {
			final JarEntry file = (JarEntry) en.nextElement();

			if (file.getName().toUpperCase().contains("META-INF"))
				continue;
			if (file.isDirectory()) {

				new File(dest + File.separator
						+ file.getName().replaceFirst(source+"extract", "")).mkdirs();
				continue;

			}


			// TODO Does this change make problems?
//			final File f = new File(dest + File.separator
//					+ file.getName().replaceFirst(source+"extract", ""));
			final File f = new File(dest + File.separator + file.getName());

			if(f.exists())continue;
			
			final InputStream is = jar.getInputStream(file);
			final FileOutputStream fos = new FileOutputStream(f);

			bytes = new byte[1024];
			while (true) {
				final int bytesread = is.read(bytes);
				if (bytesread < 0)
					break;
				fos.write(bytes, 0, bytesread);

			}
			// while (is.available() > 0) {
			//				
			// fos.write(is.read());
			// }
			fos.close();
			is.close();
		}
		
		
			extract.deleteOnExit();
		
		
//		new File("extract.jar").delete();

	}

	public static void compressToJar(final File javafile, final File dest)
			throws Exception {

		final File tempdir = new File(FileIOHandler.home + "temporaryAlgorithm");
		if (tempdir.exists())
			Cm2_Toolkit.deleteDir(tempdir,true);
		tempdir.mkdir();

		// String classpath="D:\\cm2_membranepackingalgorithm.jar";

		// int ret = Main.compile(new String[]
		// {javafile.getAbsolutePath(),"-d",tempdir.getAbsolutePath(),"-classpath",classpath});

		// get info according deprecated methods
		// int ret = Main.compile(new String[]
		// {javafile.getAbsolutePath(),"-d",tempdir.getAbsolutePath(),"-verbose","-g","-Xlint:deprecation","-Xlint:unchecked"});
		final int ret = Main.compile(new String[] { javafile.getAbsolutePath(),
				"-d", tempdir.getAbsolutePath() });

		if (ret != 0) {
			Cm2_Toolkit.deleteDir(tempdir,false);
			throw new Exception("Compilation Error");
		}

		try {
			
			final FileOutputStream os = new FileOutputStream(dest);
			
			final BufferedOutputStream bos = new BufferedOutputStream(os);
			final JarOutputStream jos = new JarOutputStream(os);
			
			final File[] tempfiles = tempdir.listFiles();

			for (int i = 0; i < tempfiles.length; i++) {
				final BufferedInputStream bis = new BufferedInputStream(
						new FileInputStream(tempfiles[i]));

				final JarEntry je = new JarEntry(tempfiles[i].getName());
				jos.putNextEntry(je);

				final byte[] bytes = new byte[1024];
				while (true) {
					final int bytesread = bis.read(bytes);
					if (bytesread < 0)
						break;
					jos.write(bytes, 0, bytesread);
					jos.flush();
				}
				
				bis.close();
			}
			
			try {
				
				bos.flush();
				os.flush();
				os.getFD().sync();

			} finally {

				jos.close();
				bos.close();
				os.close();
				
			}

		} catch (final Exception e) {

			Cm2_Toolkit.deleteDir(tempdir,false);
			throw e;
		}

		Cm2_Toolkit.deleteDir(tempdir,false);
		
	}

	public static void deleteDir(final File dir,boolean instantly) {
		if (dir.isDirectory()) {
			final File[] children = dir.listFiles();
			for (int i = 0; i < children.length; i++) {
				Cm2_Toolkit.deleteDir(children[i],instantly);

			}
		}

		if(instantly)dir.delete();
		if(dir.exists())dir.deleteOnExit();
	}

	/**
	 * This is a dirty approach, i know
	 * 
	 * @return
	 */
	public static boolean amIRunningOnWebstart() {
		try {
			ClassLoader.getSystemClassLoader().loadClass(
					"javax.jnlp.DownloadService");
			return true;
		} catch (final ClassNotFoundException e) {
			return false;
		}
	}

	public static BufferedImage rotateImage90(final BufferedImage img) {
		final int w = img.getWidth();
		final int h = img.getHeight();

		final BufferedImage rotated_img = new BufferedImage(h, w, img.getType());
		final Graphics2D g = rotated_img.createGraphics();
		g.rotate(Math.toRadians(90), w / 2, h / 2);
		g.drawImage(img, null, 0, 0);

		return rotated_img;
	}

	/**
	 * sets up capabilitys for BranchGroup
	 * 
	 * @param branch
	 */
	public static void setAllCapabilitysForBranchGroup(final BranchGroup branch) {
		branch.setCapability(Group.ALLOW_CHILDREN_EXTEND);
		branch.setCapability(Group.ALLOW_CHILDREN_WRITE);
		branch.setCapability(Group.ALLOW_CHILDREN_READ);
		branch.setCapability(BranchGroup.ALLOW_DETACH);
		branch.setCapability(Node.ALLOW_PICKABLE_WRITE);

	}

	/**
	 * sets up capabilitys for TransformGroup
	 * 
	 * @param trans
	 */
	public static void setAllCapabilitysForTransformGroup(
			final TransformGroup trans) {
		trans.setCapability(TransformGroup.ALLOW_TRANSFORM_READ);
		trans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		trans.setCapability(Group.ALLOW_CHILDREN_EXTEND);
		trans.setCapability(Group.ALLOW_CHILDREN_READ);
		trans.setCapability(Group.ALLOW_CHILDREN_WRITE);
	
	}

	/**
	 * Sets up the required capabilities for a molecule
	 * 
	 * @param m
	 */
	public static void setAllCapabilitysForShape3D(final Shape3D shape) {
		shape.setCapability(Shape3D.ALLOW_APPEARANCE_OVERRIDE_WRITE);
		shape.setCapability(Shape3D.ALLOW_APPEARANCE_OVERRIDE_READ);
		shape.setCapability(Shape3D.ALLOW_APPEARANCE_READ);
		shape.setCapability(Shape3D.ALLOW_GEOMETRY_READ);
		shape.setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
		shape.setCapability(Node.ALLOW_PICKABLE_READ);
		shape.setCapability(Node.ALLOW_PICKABLE_WRITE);
		shape.setCapability(Shape3D.ALLOW_COLLISION_BOUNDS_READ);
		shape.setCapability(Shape3D.ALLOW_COLLISION_BOUNDS_WRITE);
		shape.setCapability(Node.ALLOW_BOUNDS_READ);
		shape.setCapability(Node.ALLOW_BOUNDS_WRITE);
		shape.setCapability(Node.ALLOW_COLLIDABLE_READ);
		shape.setCapability(Node.ALLOW_COLLIDABLE_WRITE);
		shape.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
	}

	/**
	 * The given Appearance is assigned to the RenderingAttributes.
	 * This method is important for the Stereo3D Rendering. Otherwise
	 * one eye is rendered using the wrong Z-Order. 
	 * 
	 * Note: This method is not used at the moment if there is no
	 *       compatible Stereo3D device found
	 * 
	 * @param appear the Appearance which should get the RenderingAttributes
	 */
	public static void setRenderingAttributes(Appearance appear) {
//		if (Engine.getStereoAvailable()) {
		if (CM2_CONSTANTS.J3D.STEREOSCOPIC_3D_QUAD_MODE_PREFERRED) {
			RenderingAttributes render = new RenderingAttributes();
			// if this is not included one eye in stereo mode creates a wrong
			// Z-Order, probably a result of the HUD rendering process
			render.setDepthBufferEnable(true);
			render.setCapability(RenderingAttributes.ALLOW_VISIBLE_WRITE);
			appear.setRenderingAttributes(render);
		}
	}
	
	/**
	 * Puts the contents of first v1 and then v2 into a new Vector.
	 * 
	 * @param v1
	 * @param v2
	 * @return
	 */
	public static Vector<Object> combineVectors(final Vector<Object> v1,
			final Vector<Object> v2) {
		final Vector<Object> ret = new Vector<Object>();
		int size = v1.size();
		for (int i = 0; i < size; i++) {
			ret.add(v1.get(i));
		}
		size = v2.size();
		for (int i = 0; i < size; i++) {
			ret.addElement(v2.get(i));
		}
		return ret;

	}

	/**
	 * Copies the array into a new Vector
	 * 
	 * @param obj
	 * @return
	 */
	public static Vector<Object> fromArrayToVector(final Object[] obj) {
		final Vector<Object> ret = new Vector<Object>();
		final int size = obj.length;
		for (int i = 0; i < size; i++) {
			ret.add(obj[i]);
		}

		return ret;

	}
	
	/**
	 *  Returns the normal vector of two given points
	 *  which are lying in one plane. So it is the normal
	 *  vector to a plane. Uses the cross product.
	 */
	public static Point3f normalVectorInPlane(Point3f pointX, Point3f pointX0) {
		return new Point3f(	pointX.y*pointX0.z-pointX.z*pointX0.y,
							pointX.z*pointX0.x-pointX.x*pointX0.z,
							pointX.x*pointX0.y-pointX.y-pointX0.x);
	}

	/**
	 * This class is only used for direct access to the different methods
	 * of the MembranePackingAlgorithm interface. For example to use intersection 
	 * methods for aligning proteins and lipids. 
	 * Used e.g. for the PPE (OPM Alignment).
	 * 
	 */
	public static class ToolkitMembranePackingAlgorithm extends MembranePackingAlgorithm {

		public boolean buildAndShowGui() {
			return false;
		}

		public void fillWithLipids() throws Exception {
		}

		public String getFullInformation() {
			return null;
		}

		public String getName() {
			return null;
		}

		public Component getRuntimeControlPanel() {
			return null;
		}

		public String getShortInformation() {
			return null;
		}

		public String getStringRepresentationOfSetupParameters() {
			return null;
		}

		public boolean overrideExistingLipids() {
			return false;
		}

		public void stopAlgorithm() {
		}
		
		public boolean supportsAbsoluteValues() {
			return false;
		}

		public boolean supportsMicrodomains() {
			return false;
		}
		
	}

	public static float calculateLengthBetween(Point3f a, Point3f b) {

		return a.distance(b);
	}

	public static String showInputDialog(Component component, String message,
			String title, final String[] choices, final String[] exclusions,
			String initial) {


		
		
		
		final JDialog dia = new JDialog();
		final JButton okBtn = new JButton("Apply");
		final JButton cancelBtn = new JButton("Cancel");
		final JComboBox combo=new JComboBox();
		final DefaultComboBoxModel model = new DefaultComboBoxModel();
		JLabel label = new JLabel(message);
		final JLabel statuslabel = new JLabel("");
		
		final JTextField tf=	(JTextField) combo.getEditor().getEditorComponent();
		
		JPanel p = new JPanel();
		p.add(okBtn);
		p.add(cancelBtn);
		dia.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
	
	
	
		
		
		
	combo.setModel(model);
	
	combo.setEditable(true);

	combo.setFont(combo.getFont().deriveFont(combo.getFont().getSize2D()+2));
	combo.setPreferredSize(new Dimension(200,25));
	combo.setMaximumSize(new Dimension(200,25));
	combo.setMinimumSize(new Dimension(200,25));

	

	statuslabel.setPreferredSize(label.getPreferredSize());
	BasicComboBoxEditor bcbe = new BasicComboBoxEditor() {
		
		public Component getEditorComponent() {
			return tf;
		}
	};

	
	combo.setEditor(bcbe);
	
		if(choices!=null) {
		for(int i=0;i<choices.length;i++) {
			combo.addItem(choices[i]);
		}
		}
		
		JPanel p1=new JPanel();
		JPanel p2=new JPanel();
		JPanel p3=new JPanel();
		
		p1.add(label);
		p2.add(combo);
		p3.add(statuslabel);
	
	dia.getContentPane().setLayout(new BoxLayout(dia.getContentPane(),BoxLayout.Y_AXIS));
	dia.getContentPane().add(p1);
	dia.getContentPane().add(p2);
	dia.getContentPane().add(p3);
	dia.getContentPane().add(p);
		
		
		
		
		ActionListener ac = new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				if (arg0.getSource() == cancelBtn) {
			
				
				tf.setText(null);
			
				
					dia.dispose();
				} else if (arg0.getSource() == okBtn) {
					dia.dispose();
				}else if(arg0.getSource()==combo) {
					tf.setText((String)combo.getSelectedItem());
				}

			}

		};
		
		
		CaretListener cl = new CaretListener() {
			public void caretUpdate(CaretEvent e) {
				
				boolean b=false;
				statuslabel.setText("");
				statuslabel.setForeground(Color.red);
				
				if((tf.getText()==null||tf.getText().trim().length()<1)) {
					b=false;
				}else if(tf.getText().contains(".")||tf.getText().contains("/")||tf.getText().contains("\\")){
					statuslabel.setText("Invalid Syntax!");
					b=false;
				}else {
					String text = tf.getText().trim();
					
					b=true;
					if(exclusions!=null) {
					for(int i=0;i<exclusions.length;i++) {
						if(exclusions[i].equals(text)) {
							b=false;
							statuslabel.setText("Not allowed!");
							break;
						}
					}
					}
					
					if(b==true) {
						
						if(choices!=null) {
						for(int i=0;i<choices.length;i++) {
							if(choices[i].equals(text)) {
								statuslabel.setForeground(Color.orange);
								statuslabel.setText("Name exists! Overwrite?");
								break;
							}
						}
						}
						
					}
					
				}
				
				
				okBtn.setEnabled(b);
			}
		};

		okBtn.addActionListener(ac);
		cancelBtn.addActionListener(ac);
		combo.addActionListener(ac);

		tf.addCaretListener(cl);
		
		
		okBtn.setEnabled(false);
		
		if(choices!=null) {
		for(int i=0;i<choices.length;i++) {
			if(choices[i].equals(initial)) {
				combo.setSelectedIndex(i);
				break;
			}
		}
		}
		
		

		dia.pack();
		dia.setLocationRelativeTo(component);
		// dia.setSize(new Dimension(400,400));

		dia.setModal(true);
		dia.setAlwaysOnTop(true);
		dia.setTitle(title);
		dia.setVisible(true);
		
		
		
		

		return tf.getText().trim();
		
		
		
		
		
		
		
		
		
		
	
	}

	public static String convertAtomNameToElementName(String key) {
		String properkey = "" + key;
		for (int i = 0; i < 10; i++) {
			properkey = properkey.replaceAll("" + i, "");
		}
		if (properkey.length() > 0)
			properkey = "" + properkey.charAt(0);
		else properkey=null;
		return properkey;
	}
	
	/**
	 * This method makes a copy of the given sourceFile to the defined destinationFile.
	 * Note: This method should not be used for large files.
	 * @param sourceFile
	 * @param destinationFile
	 * @throws IOException
	 */
	public static void copyFile(File sourceFile, File destinationFile) throws IOException {
		if (!destinationFile.exists())
			destinationFile.createNewFile();
		
		FileChannel sourceChannel = null;
		FileChannel destinationChannel = null;
		
		try {
			sourceChannel = new FileInputStream(sourceFile).getChannel();
			destinationChannel = new FileOutputStream(destinationFile).getChannel();
			destinationChannel.transferFrom(sourceChannel, 0, sourceChannel.size());
		} catch (Exception e) {
			System.err.println("There was an error copying the file "+
					sourceFile.getAbsolutePath()+" to "+destinationFile.getAbsolutePath()+".\n"+
					"Normally this will not affect the program, because this method should be used for backup issues only.");
			e.printStackTrace();
		} finally {
			if (sourceChannel != null)
				sourceChannel.close();
			if (destinationChannel != null)
				destinationChannel.close();
		}
	}
	
//	public static Atom[] findAllDirectlyBondedNeighbors(Molecule_3D mol, Atom[] includedAtoms) {
//
//		Vector<Atom>v=new Vector<Atom>();
//		Vector<Object>allOfMolecule = Cm2_Toolkit.fromArrayToVector(mol.atoms);
//
//		for(int i=0;i<includedAtoms.length;i++) {
//			
//			for(int j=0;j<includedAtoms[i].currentNeighbors.size();j++) {
//			
//			Atom neighbor = includedAtoms[i].currentNeighbors.get(j);
//			if(!v.contains(neighbor)&&allOfMolecule.contains(neighbor))v.add(neighbor);
//			}
//			
//		}
//		
//		for(int i=0;i<includedAtoms.length;i++) {
//			if(v.contains(includedAtoms[i])) {
//				v.removeElement(includedAtoms[i]);
//			}
//		}
//		
//		
//		
//		
//		
//		
//		return v.toArray(new Atom[0]);
//	}

}
