/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithm;

import java.awt.geom.Rectangle2D;
import java.util.Vector;

import javax.media.j3d.BoundingBox;
import javax.vecmath.Matrix3f;
import javax.vecmath.Point3d;
import javax.vecmath.Point3f;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.Atom_3D;
import org.cellmicrocosmos.cm2.dataobjects.Constraint_3D;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;


/**
 * This class is a wrapper class for the Molecule_3D class. The intention is to
 * let the writer of membrane calculating algorithms only see what he needs to
 * see and to hide all the internal data transfer handling methods. This wrapped
 * object should let the user get the informations he needs and also lets
 * him/her manipulate data like position and rotation of the molecule.
 * 
 * 
 * @author Tim Dingersen, Sebastian Schneider
 * 
 */
public class Molecule {

	public static final int RENDER_MODE_POLYGON = 0;
	public static final int RENDER_MODE_COVALENT = 1;
	public static final int RENDER_MODE_VAN_DER_WAALS = 2;
	public static final int RENDER_MODE_COLORED_STICKS = 3;
	public static final int RENDER_MODE_BALLS_N_STICKS = 4;

	private final Molecule_3D originalMolecule;

	private Atom[] atoms;

	private Constraint[] constraints;

	private MembranePackingAlgorithm algo;
	protected boolean isUpdated = true;

	/**
	 * 
	 * Creates a new wrapper object of the given Molecule_3D
	 * 
	 * ! NOT MEANT TO BE USED BY PLUGIN DEVELOPERS !
	 * 
	 * 
	 */
	public Molecule(final Molecule_3D original, MembranePackingAlgorithm algo) {

		this.originalMolecule = original;
		this.algo = algo;
		Atom_3D[] origs = this.originalMolecule.atoms;
		Atom_3D[] origsAligned = this.originalMolecule.getAlignedAtoms();
		atoms = new Atom[origs.length];
		for (int i = 0; i < origs.length; i++) {
			atoms[i] = new Atom(origs[i], this, i, algo);
			atoms[i].position = origsAligned[i].getPosition();

		}

		Vector<Constraint_3D> constraints = this.originalMolecule.getActiveConstraints();
		float[] origLengts = this.algo.getOriginalConstraintLengths(this);

		this.constraints = new Constraint[constraints.size()];
		for (int i = 0; i < constraints.size(); i++) {
			Constraint_3D con = constraints.get(i);

			Atom A = null;
			Atom B = null;
			for (int j = 0; j < this.atoms.length; j++) {
				if (con.atomA == origs[j]) {
					A = atoms[j];
				} else if (con.atomB == origs[j]) {
					B = atoms[j];
				}
			}
			Constraint c;
			if (origLengts != null) {
				c = new Constraint(A, B, origLengts[i]);

			} else {

				c = new Constraint(A, B, A.position.distance(B.position));
			}
			this.constraints[i] = c;

		}

		for (int i = 0; i < atoms.length; i++) {

			Vector<Bond> bonds = new Vector<Bond>();
			for (int j = 0; j < this.constraints.length; j++) {

				if (this.constraints[j].atomA == atoms[i]) {
					bonds.addElement(new Bond(atoms[i],
							this.constraints[j].atomB,
							this.constraints[j].originalLength));
				}
				if (this.constraints[j].atomB == atoms[i]) {
					bonds.addElement(new Bond(atoms[i],
							this.constraints[j].atomA,
							this.constraints[j].originalLength));
				}

			}
			atoms[i].bonds = bonds.toArray(new Bond[0]);

		}

	}

	/**
	 * returns the PDB code of the molecule
	 * 
	 * @return the PDB Code of the underlying PDB File or null if none could be
	 *         determined
	 */
	public String getType() {
		return this.originalMolecule.getPDBcode();
	}

	/**
	 * returns the center point of this molecule in the membrane.
	 * 
	 * @deprecated the equivalent method getPosition() should be used instead
	 * @return the current center point of this molecule in the coordinate
	 *         system
	 * 
	 */
	@Deprecated
	public Point3f getCenterPoint() {
		return (new Point3f(this.originalMolecule.getPosition()));
	}

	/**
	 * returns the position of the molecule in the membrane
	 * 
	 * @return the current center point of this molecule in the coordinate
	 *         system
	 */
	public Point3f getPosition() {
		return (new Point3f(this.originalMolecule.getPosition()));
	}

	/**
	 * returns the current rotation matrix of this molecule.
	 * 
	 * @return a 3x3 rotation matrix representing the current rotation relative
	 *         to the alignment inside the underlying PDB-file
	 */
	public Matrix3f getRotationMatrix() {
		return Cm2_Toolkit
				.copyMatrix(this.originalMolecule.getRotationMatrix());
	}

	/**
	 * returns a copy of all atoms of the molecule. The positions of the atoms
	 * are set to their actual location inside the coordinate system of the
	 * membrane. NOTE: Manipulation of these atoms do not take any effect. This
	 * is read only!
	 * 
	 * 
	 * @return a copy of all atoms of this molecule.
	 */
	public Atom[] getAlignedAtomCopies() {

		Atom_3D[] origs = this.originalMolecule.getAtoms(true, true);
		for (int i = 0; i < origs.length; i++) {
			this.atoms[i].position = origs[i].getPosition();
		}
		return this.atoms;
		
//		Atom_3D[] origs = this.originalMolecule.getAtoms(true, true);// TODO true true!
//		
////		Atom[] tempAtoms = new Atom[origs.length];
////		for (int i = 0; i < origs.length; i++) {
////			tempAtoms[i] = new Atom(origs[i], this, i, algo);
////			tempAtoms[i].position = origs[i].getPosition();
////			tempAtoms[i].position.y = this.getYTranslation();
////		}
//		
////		Atom_3D[] origs = this.originalMolecule.getAtoms(true, true);
////		Atom[] tempAtoms = new Atom[atoms.length];
////		for (int i = 0; i < atoms.length; i++) {
////			tempAtoms[i] = new Atom(origs[i], this, i, algo);
////			tempAtoms[i].position = atoms[i].getPosition();
////		}
//		
//		for (int i = 0; i < origs.length; i++) {
//			this.atoms[i].position = origs[i].getPosition();
//		}
//		return this.atoms;
////		return tempAtoms;

	}

	/**
	 * Compares the widht (x) and the height (z) of the molecule's current
	 * boundingbox and returns the largest of these two values
	 * 
	 * @return the width of the bounding box if width>=height, else the height
	 */
	public float getMaxRadius_XZ() {
		final Point3f lower = this.originalMolecule.getLower(true);

		final Point3f upper = this.originalMolecule.getUpper(true);

		return ((Math.max(Math.abs(upper.x - lower.x), Math.abs(upper.z
				- lower.z))) / 2);
	}

	/**
	 * returns the radius the Molecule on the x/z plane.
	 * 
	 * @return the distance from the center to the extremest point of the
	 *         molecule on the x/z plane
	 */
	public float calculateRadius() {

		return this.originalMolecule.calculateRadius()
				/ CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE;
	}

	/**
	 * returns the BoundingBox of the molecule. If atomradiuses are taken into
	 * account by the program, then these will be respected here
	 * 
	 * @return a BoundingBox Object that represents the bounds of the molecule
	 */
	public BoundingBox getBounds() {
		final Point3d lower = new Point3d(this.originalMolecule.getLower(true));

		final Point3d upper = new Point3d(this.originalMolecule.getUpper(true));
		return new BoundingBox(lower, upper);

	}

	/**
	 * Determines the BoundingBox of the molecule and returns a rectangle that
	 * represents the bounds on the x/z plane
	 * 
	 * @return a rectangle that represents the current x/z bounds of the
	 *         molecule
	 */
	public Rectangle2D.Double getXZBounds() {
		final Point3d lower = new Point3d(this.originalMolecule.getLower(true));

		final Point3d upper = new Point3d(this.originalMolecule.getUpper(true));

		return new Rectangle2D.Double(lower.x, lower.z, Math.abs(upper.x
				- lower.x), Math.abs(upper.z - lower.z));
	}

	/**
	 * returns the scaled points of the triangles that build up the shape of the
	 * molecule. These triangles are essential for the 3D display of the
	 * molecule.
	 * 
	 * Always three points of the Point3f[] define one triangle. For instance:
	 * point[0] is the x value of the first triangle point[1] is the y value of
	 * the first triangle point[2] is the z value of the first triangle point[3]
	 * is the x value of the second triangle ...
	 * 
	 * 
	 * point[3] to point[5] defines the next triangle and so on.
	 * 
	 * @return a list of properly aligned triangle points
	 */
	public Point3f[] getTrianglePoints() {
		final Point3f[] points = this.originalMolecule
				.getAlignedTrianglePoints();
		final Point3f[] ret = new Point3f[points.length];
		for (int i = 0; i < points.length; i++) {
			ret[i] = new Point3f(points[i]);
			ret[i].scale(1 / CM2_CONSTANTS.J3D.J3D_ANGSTROM_SCALE);
		}
		return ret;
	}

	/**
	 * sets the rotation matrix.
	 * 
	 * NOTE: If this molecule is already added to the membrane model this will
	 * immediately rotate the molecule.
	 * 
	 * @param newmatrix the
	 *            new rotation matrix that represents the rotation of the
	 *            molecule relative to the PDB file coordinates
	 */
	public void setRotationMatrix(final Matrix3f newmatrix) {
		algo.dock();

		this.originalMolecule.setRotationMatrix(newmatrix);
		Atom[] newOnes2 = this.getAlignedAtomCopies();
		for (int j = 0; j < this.atoms.length; j++) {
			this.atoms[j].setPosition(newOnes2[j].getPosition());
		}
		if (this.originalMolecule.getBranchGroup().isLive())
			this.originalMolecule.move();
	}

	/**
	 * sets the position of the molecule. NOTE: If this molecule is already
	 * added to the membrane model this will immediately move the molecule.
	 * 
	 * @param newpoint the
	 *            new position of the molecule in the membrane space
	 */
	public void setPosition(final Point3f newpoint) {
		algo.dock();
		this.originalMolecule.setPosition(newpoint.x, newpoint.z);
		this.originalMolecule.setYTranslation(newpoint.y);

		Atom[] newOnes2 = this.getAlignedAtomCopies();
		for (int j = 0; j < this.atoms.length; j++) {
			this.atoms[j].setPosition(newOnes2[j].getPosition());
		}

		if (this.originalMolecule.getBranchGroup().isLive())
			this.originalMolecule.move();
	}

	/**
	 * rotates the molecule around the x-axis
	 * 
	 * @param angle the
	 *            angle in degrees
	 */
	public void rotX(final float angle) {
		algo.dock();
		if (angle == 0 || angle == 360)
			return;
		final Matrix3f current = Cm2_Toolkit.copyMatrix(this.originalMolecule
				.getRotationMatrix());
		Cm2_Toolkit.rotate_X_Axis_Of_CoordinateSystem(current, angle);
		this.originalMolecule.setRotationMatrix(current);
		Atom[] newOnes2 = this.getAlignedAtomCopies();
		for (int j = 0; j < this.atoms.length; j++) {
			this.atoms[j].position = newOnes2[j].getPosition();
		}
		if (this.originalMolecule.getBranchGroup().isLive())
			this.originalMolecule.move();

	}

	/**
	 * rotates the molecule around the y-axis
	 * 
	 * @param angle the
	 *            angle in degrees
	 */
	public void rotY(final float angle) {
		algo.dock();
		if (angle == 0 || angle == 360)
			return;
		final Matrix3f current = Cm2_Toolkit.copyMatrix(this.originalMolecule
				.getRotationMatrix());
		Cm2_Toolkit.rotate_Y_Axis_Of_CoordinateSystem(current, angle);
		this.originalMolecule.setRotationMatrix(current);
		Atom[] newOnes2 = this.getAlignedAtomCopies();
		for (int j = 0; j < this.atoms.length; j++) {
			this.atoms[j].position = newOnes2[j].getPosition();
		}
		if (this.originalMolecule.getBranchGroup().isLive())
			this.originalMolecule.move();

	}

	/**
	 * rotates the molecule around the z-axis
	 * 
	 * @param angle the
	 *            angle in degrees
	 */
	public void rotZ(final float angle) {
		algo.dock();
		if (angle == 0 || angle == 360)
			return;
		final Matrix3f current = Cm2_Toolkit.copyMatrix(this.originalMolecule
				.getRotationMatrix());
		Cm2_Toolkit.rotate_Z_Axis_Of_CoordinateSystem(current, angle);
		this.originalMolecule.setRotationMatrix(current);
		Atom[] newOnes2 = this.getAlignedAtomCopies();
		for (int j = 0; j < this.atoms.length; j++) {
			this.atoms[j].position = newOnes2[j].getPosition();
		}
		if (this.originalMolecule.getBranchGroup().isLive())
			this.originalMolecule.move();

	}
	
	/**
	 * returns the original Molecule_3D of this wrapper molecule.
	 * 
	 * ! NOT MEANT TO BE USED FOR PLUGIN DEVELOPERS !
	 * 
	 */
	public Molecule_3D getOriginalMolecule() {
		return this.originalMolecule;
	}

	/**
	 * returns the translation on the y-axis.
	 * 
	 * 
	 * @return the y-translation (can be regarded as the "height" of the
	 *         molecule)
	 */
	public float getYTranslation() {
		return this.originalMolecule.getYTranslation();
	}

	/**
	 * sets the y translation NOTE: If this molecule is already added to the
	 * membrane model this will immediately move the molecule.
	 * 
	 * @param translation the
	 *            desired y-value of the molecule's position
	 */
	public void setYTranslation(final float translation) {
		algo.dock();
		this.originalMolecule.setYTranslation(translation);
		if (this.originalMolecule.getBranchGroup().isLive())
			this.originalMolecule.move();
	}

	/**
	 * Returns the ID-number of the microdomain that this molecule is assigned
	 * to
	 * 
	 * @return the Domain_ID
	 */
	public int getDomainIdentifier() {
		return this.originalMolecule.getDomainID();
	}

	/**
	 * Returns the ID-number of the Layer that this molecule is assigned to
	 * 
	 * @return the Layer_ID
	 */
	public int getLayerID() {

		return this.originalMolecule.getLayerID();
	}

	/**
	 * Updates the geometry of this Molecule. Needs to be called after one or
	 * more setPosition() calls on Atoms in order to calculate a new Geometry.
	 * !You should only call this before the next Shape-based intersection test
	 * or if you want to update the visuals, because this method is quite
	 * expensive.
	 * 
	 */
	public void updateGeometry() {
		algo.dock();

		if (!isUpdated) {
			this.originalMolecule.updateGeometry();
			isUpdated = true;
		}
	}

	/**
	 * Returns the list of all actual Atom instances. Other than the Atoms
	 * returned by getAlignedAtomCopies(), manipulating these Atoms will take
	 * effect.
	 * 
	 * @return The list of the original atom instances of this molecule
	 */
	public Atom[] getOriginalAtoms() {
		return this.atoms;
	}

	private class Constraint {

		public Atom atomA;
		public Atom atomB;
		public float originalLength;

		public Constraint(Atom a, Atom b, float originalLength) {
			this.atomA = a;
			this.atomB = b;

			this.originalLength = originalLength;
		}

	}
}
