/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.membranepackingalgorithm;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JViewport;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.table.DefaultTableColumnModel;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

/**
 * This panel will display a JTable that also has a scrollable row header. Can
 * be used in order to present e.g. a parameter matrix to the user.
 * 
 * @author Tim Dingersen dackelnose@gmx.net
 * 
 */
public class ParameterMatrixPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	public JTable table = new JTable();
	public DefaultTableModel model;
	public TableColumnModel columnModel;
	public String headers[];

	/**
	 * Constructs a new ParamterMatrixPanel
	 * 
	 * @param headers
	 *            the names for each row / column (diagonally mirrored).
	 * @param cellRenderer
	 *            the cellRenderer for the table
	 * @param cellEditor
	 *            the cellEditor for the table
	 */
	public ParameterMatrixPanel(String[] headers,
			TableCellRenderer cellRenderer, TableCellEditor cellEditor) {
		super();
		this.headers = headers;

		this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		columnModel = new DefaultTableColumnModel() {

			private static final long serialVersionUID = 1L;

			boolean first = true;

			int index = 0;

			public void addColumn(TableColumn tc) {

				if (first) {
					first = false;
					return;
				}
				tc.setModelIndex(index);
				super.addColumn(tc);
				index++;

			}

		};

		model = new DefaultTableModel() {

			private static final long serialVersionUID = 1L;

			public int getColumnCount() {
				return ParameterMatrixPanel.this.headers.length;

			}

			public int getRowCount() {

				return ParameterMatrixPanel.this.headers.length;
			}

			public boolean isCellEditable(int row, int col) {

				return !((row - 1) >= col);

			}

			public void setValueAt(Object val, int row, int col) {
				super.setValueAt(val, row, col);
				super.setValueAt(val, col, row);
			}

		};

		table.setModel(model);
		table.setColumnModel(columnModel);

		JTable headerColumn = new JTable();

		DefaultTableColumnModel rowHeaderColumnModel = new DefaultTableColumnModel();
		DefaultTableModel rowHeaderTableModel = new DefaultTableModel() {

			private static final long serialVersionUID = 1L;

			public Object getValueAt(int row, int col) {

				return ParameterMatrixPanel.this.headers[row];
			}
		};

		headerColumn.setColumnModel(rowHeaderColumnModel);
		headerColumn.setModel(rowHeaderTableModel);

		TableColumn co = new TableColumn();
		co.setHeaderValue("");
		table.addColumn(co);
		co.setMaxWidth(100);

		TableColumn col = new TableColumn();
		col.setHeaderValue("");
		headerColumn.addColumn(col);
		col.setMaxWidth(100);
		col.setCellRenderer(new MyHeaderCellRenderer());

		// rowHeaderTableModel.addRow(new String[] {""});

		for (int i = 0; i < headers.length; i++) {

			TableColumn tc = new TableColumn();
			tc.setHeaderValue(headers[i]);
			table.addColumn(tc);
			rowHeaderTableModel.addRow(new String[] { headers[i] });
			tc.setMaxWidth(100);
			tc.setCellRenderer(cellRenderer);
			tc.setCellEditor(cellEditor);

		}

		table.setRowSelectionAllowed(false);
		table.setColumnSelectionAllowed(false);
		table.getTableHeader().setReorderingAllowed(false);
		// table.getTableHeader().setResizingAllowed(false);

		// headerColumn.setBackground(Color.lightGray);
		headerColumn.setColumnSelectionAllowed(false);
		headerColumn.setCellSelectionEnabled(false);
		headerColumn.setRowSelectionAllowed(false);
		headerColumn.setFocusable(false);

		table.getTableHeader().setDefaultRenderer(new MyHeaderCellRenderer());
		headerColumn.getTableHeader().setDefaultRenderer(
				new MyHeaderCellRenderer());

		JViewport jv = new JViewport();
		jv.setView(headerColumn);
		jv.setPreferredSize(headerColumn.getMaximumSize());

		table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

		JScrollPane jsp = new JScrollPane(table);
		jsp.setRowHeader(jv);
		jsp.setCorner(ScrollPaneConstants.UPPER_LEFT_CORNER, headerColumn
				.getTableHeader());
		this.add(jsp, BorderLayout.CENTER);

		this.setPreferredSize(new Dimension(
				table.getPreferredSize().width + 100,
				table.getPreferredSize().height + 30));

	}

	@SuppressWarnings("serial")
	private class MyHeaderCellRenderer extends JLabel implements
			TableCellRenderer {

		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {

			if (value != null)
				setText(value.toString());
			else
				setText("");
			setFont(table.getFont().deriveFont(Font.BOLD));

			setBorder(UIManager.getBorder("TableHeader.cellBorder"));

			setHorizontalAlignment(SwingConstants.CENTER);

			setOpaque(true);

			return this;
		}
	}

	/**
	 * Returns the table displayed in this panel.
	 * 
	 * @return the table
	 */
	public JTable getTable() {
		return table;
	}

	/**
	 * Returns the model of the table in this panel
	 * 
	 * @return the TableModel
	 */
	public DefaultTableModel getModel() {
		return model;
	}

	/**
	 * Return this names for each row / column that was passed inside the
	 * constructor
	 * 
	 * @return the headers for both, rows and columns
	 */
	public String[] getTypes() {
		return headers;
	}
}
