/*******************************************************************************
 * CELLmicrocosmos 2.2 MembraneEditor (version 2.2.2_2)
 *
 * Copyright (c) 2005, 2017 The CELLmicrocosmos 2 Development Team
 * (Bjorn Sommer, Tim Dingersen et al.
 *  University of Konstanz, Monash University, Bielefeld University)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *  
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *  
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package org.cellmicrocosmos.cm2.io;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import javax.vecmath.Matrix3f;
import javax.vecmath.Point3f;

import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
import org.cellmicrocosmos.cm2.dataobjects.Atom_3D;
import org.cellmicrocosmos.cm2.dataobjects.Lipid_3D;
import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;
import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
import org.cellmicrocosmos.cm2.dataobjects.Particle;
import org.cellmicrocosmos.cm2.dataobjects.Protein_3D;
import org.cellmicrocosmos.cm2.engine.DataPool;
import org.cellmicrocosmos.cm2.gui.dialogs.StatusDialog;
import org.cellmicrocosmos.cm2.io.PdbReader_3D.PdbAttributes;
import org.cellmicrocosmos.cm2.membranepackingalgorithm.CoordinateTransformationHints;
import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;




/**
 * This class sets up all relevant data from the membrane model, computing it
 * into a proper pdb syntax and writing it into a file.
 * 
 * @author Tim Dingersen dackelnose@gmx.net, Christian Gamroth c.gamroth@hotmail.de,
 * 		   Sebastian Rubert, Bjoern Sommer bjoern@CELLmicrocosmos.org
 *
 */
public class PDBWriter {
	
//	*****************************************************************************
//	* CONSTANTS                                                					*
//	*****************************************************************************
	private final int BOTH = 0;
	private final int PROTSONLY = 1;
	private final int NONE = 2;
	private final int MAXVALUE = -1000000;
	private final int MINVALUE = 1000000;
//	*****************************************************************************
//	* BOOLEAN                                                  					*
//	*****************************************************************************
	private int useSerials = this.BOTH;
	private int useChains = this.BOTH;
	private boolean adoptTitleSections = false;
	private boolean enableReverseParsing = false;
	private final boolean listPositions = true;
//	*****************************************************************************
//	* GLOBAL OBJECTS                                           					*
//	*****************************************************************************
	private CoordinateTransformationHints tHints = null;
	private MembraneModel model;
	private final File outputFile;
	private final StatusDialog dialog;
	private int subX, subZ, atomIndex, terCount, atomHetAtmCount, remCount, endOfSerialNumbers;
	private float cx, cy, cz;

	/**
	 * Constructor of the class PDBWriter.
	 * 
	 * @param file - the file writing the output to
	 * @param dialog - the StatusDialog for displaying processes
	 */
	public PDBWriter(final File file, final StatusDialog dialog) {
		// parameters getting global
		this.outputFile = file;
		this.dialog = dialog;
		this.useSerials = DataPool.LimitationSettings.atomserials;
		this.useChains = DataPool.LimitationSettings.chains;
		this.adoptTitleSections = DataPool.LimitationSettings.adoptTitleSections;
		this.enableReverseParsing = DataPool.LimitationSettings.reverseParsing;
	}
	
	/**
	 * 
	 * This method writes the whole membrane in PDB format. 
	 * It returns true if the writing process succeeded and false if it was canceled.
	 * 
	 * @param domainID
	 * @param layerID
	 * @param tHints
	 * @return writing success
	 */
	public boolean writeMembrane(final int domainID, final int layerID, CoordinateTransformationHints tHints) {
		
		// setting up the progress bar
		float steps = 24f;
		float currentStep = 0f; 	// goes up to steps
		/*IO*/ this.dialog.setProgress("Starting ...", (int) (currentStep++ / steps * 100), true);
		// parameters getting global
		this.tHints = tHints;	
		this.model = this.getAlteredModel(domainID, layerID);

		if (this.coordinatesWouldExceedValidNumberRangeCurrently(this.model)) {
			Cm2_Toolkit.complain("Some atom coordinate numbers would be outside the valid number range!\n" +
					"(that is -999.999 to 1000.000)\n\n"+
					"Please remove all current border intersections and, if neccessary,\n" +
					"choose different coordinate transformation settings!", "Unable to proceed");
			return false;
		}
		
		boolean saveKeyAtomIDsForAPLVoro = (tHints.exportAPLVoroFile || DataPool.LimitationSettings.exportAPLVoro);
		
//		this.model = DataPool.getMembraneModel();
		// setting up new variables
		BufferedWriter bWriter = null;
		String[] proteinTypes = null;
		String[] lipidTypes = null;
		String[] lipidTypesExtra = null;
		String[] lipidTypesIntra = null;
		Vector<String[]> topologyVec = new Vector<String[]>();
		Vector<String[]> aplVec = new Vector<String[]>();
		Content content = new Content();
		try {
			bWriter = new BufferedWriter(new FileWriter(this.outputFile));
		} catch (IOException e) {
			// handle exception here
			Cm2_Toolkit.complain(e.getMessage(), "PDBWriter Error: Could not write to PDB file");
			return false;
		}
		// getting all proteintypes
		/*IO*/ this.dialog.setProgress("Getting all types of proteins ...", (int) (currentStep++ / steps * 100), true);
		proteinTypes = this.model.getAllProteinTypes();
		// check for sortByType
		final boolean sortByType = (this.tHints != null) ?
				this.tHints.sortByType : DataPool.CoordinateTransformationSettings.sortByType;
		// check for swapXY
		final boolean swapYZ = (this.tHints != null) ?
				tHints.swapYZ : DataPool.CoordinateTransformationSettings.swapYZ;
		// getting all lipidtypes
		/*IO*/ this.dialog.setProgress("Getting all types of lipids ...", (int) (currentStep++ / steps * 100), true);
		if(sortByType) {
			lipidTypes = this.model.getAllLipidTypes();
		} else {
			lipidTypesExtra = this.model.getAllLipidTypesInMembraneExtra();
			lipidTypesIntra = this.model.getAllLipidTypesInMembraneIntra();
		}
		/* first loop - setting up min/ max for later transformation */
		Point3f min = new Point3f(this.MINVALUE, this.MINVALUE, this.MINVALUE);
		Point3f max = new Point3f(this.MAXVALUE, this.MAXVALUE, this.MAXVALUE);
		
		if (proteinTypes!=null) {
			for(String type : proteinTypes) {
				/*IO*/ this.dialog.setProgress("Setting up min/max: " + type, (int) (currentStep / steps * 100), true);
				Protein_3D[] proteinsOfSameType = this.model.getProteinsByPDBCode(type);
				for(Protein_3D protein : proteinsOfSameType) {

					// creating pdb file lines from the molecule and setting up min/ max
					Line[] moleculeLines = this.transform(protein, swapYZ);
					if (moleculeLines == null) continue;
					
					for(Line line : moleculeLines) {
						// min/ max update
						Point3f p = line.position;
						min.x = Math.min(min.x, p.x);
						min.y = Math.min(min.y, p.y);
						min.z = Math.min(min.z, p.z);
						max.x = Math.max(max.x, p.x);
						max.y = Math.max(max.y, p.y);
						max.z = Math.max(max.z, p.z);
					}
				}
			}
		}
		// -> is null if !sortByType
		if (lipidTypes!=null) {
			for(String type : lipidTypes) {
				int currentNumberOfActualType = 0;
				boolean foundAtomP = false;
				String keyAtoms = "";
				HashSet<String> typeAtoms = new HashSet<String>();
				
				/*IO*/ this.dialog.setProgress("Setting up min/max: " + type, (int) (currentStep / steps * 100), true);
				
				Lipid_3D[] lipidsOfSameType = this.model.getLipidsByPDBCodeExtra(type);
				for(Lipid_3D lipid : lipidsOfSameType) {

					// creating pdb file lines from the molecule and setting up min/ max
					Line[] moleculeLines = this.transform(lipid, swapYZ);
					if (moleculeLines == null) continue;
					
					for(Line line : moleculeLines) {
						// min/ max update
						Point3f p = line.position;
						min.x = Math.min(min.x, p.x);
						min.y = Math.min(min.y, p.y);
						min.z = Math.min(min.z, p.z);
						max.x = Math.max(max.x, p.x);
						max.y = Math.max(max.y, p.y);
						max.z = Math.max(max.z, p.z);
						// key atom update
						if (saveKeyAtomIDsForAPLVoro) {
							if (currentNumberOfActualType == 0 &&
								!foundAtomP &&
								( (line.name.contains("P") && !line.name.contains("O")) || 
								  (line.name.contains("O") && !line.name.contains("P")) ) ) {
								if (keyAtoms.isEmpty()) {
									keyAtoms = line.name;
									if (line.name.contains("P")) {
										foundAtomP = true;
									}
								} else {
									if (line.name.contains("P")) {
										keyAtoms = line.name;
										foundAtomP = true;
									} else {
										keyAtoms = keyAtoms+", "+line.name;
									}
								}
							}
						}
						if (!typeAtoms.contains(line.residueName))
							typeAtoms.add(line.residueName);
					}
					currentNumberOfActualType++;
				}
				
				lipidsOfSameType = this.model.getLipidsByPDBCodeIntra(type);
				for(Lipid_3D lipid : lipidsOfSameType) {

					// creating pdb file lines from the molecule and setting up min/ max
					Line[] moleculeLines = this.transform(lipid, swapYZ);
					if (moleculeLines == null) continue;
					
					for(Line line : moleculeLines) {
						// min/ max update
						Point3f p = line.position;
						min.x = Math.min(min.x, p.x);
						min.y = Math.min(min.y, p.y);
						min.z = Math.min(min.z, p.z);
						max.x = Math.max(max.x, p.x);
						max.y = Math.max(max.y, p.y);
						max.z = Math.max(max.z, p.z);
						// key atom update
						if (saveKeyAtomIDsForAPLVoro) {
							if (currentNumberOfActualType == 0 &&
								!foundAtomP &&
								( (line.name.contains("P") && !line.name.contains("O")) || 
								  (line.name.contains("O") && !line.name.contains("P")) ) ) {
								if (keyAtoms.isEmpty()) {
									keyAtoms = line.name;
									if (line.name.contains("P")) {
										foundAtomP = true;
									}
								} else {
									if (line.name.contains("P")) {
										keyAtoms = line.name;
										foundAtomP = true;
									} else {
										keyAtoms = keyAtoms+", "+line.name;
									}
								}
							}
						}
						if (!typeAtoms.contains(line.residueName))
							typeAtoms.add(line.residueName);
					}
					currentNumberOfActualType++;
				}
				// save the substring of the lipid type (which must match the topology identifier)
				// and the number of occurrences; this is only relevant for the GMX-Plugin
				// TODO it could be better to substitute type.substring(0,3) completely by typeAtoms, also in all other topologyVec
				//      sections, test this with the GMX Plugin
				for(String typeAtom:typeAtoms) {
					topologyVec.add(new String[]{type.substring(0,3), Integer.toString(currentNumberOfActualType)});
					aplVec.add(new String[]{typeAtom, keyAtoms});
				}
			}
		}
		// -> is null if sortByType
		if (lipidTypesExtra != null) {
			for(String type : lipidTypesExtra) {
				int currentNumberOfActualType = 0;	
				// TODO The Membrane Model seems to hold the old number of lipids after pressing "Clear Lipids" in the GUI
				//      there has to be some problem with the renewing of the MembraneModel in MembraneView.java:
				//       this.m3d.clearLipids(-1, DataPool.getCurrentLayerID());
				//      This can be solved AFTER implementing Philipp's last changes!
				/*IO*/ this.dialog.setProgress("Setting up min/max: " + type, (int) (currentStep / steps * 100), true);
				Lipid_3D[] lipidsExtraOfSameType = this.model.getLipidsByPDBCodeExtra(type);
				for(Lipid_3D lipid : lipidsExtraOfSameType) {

					// creating pdb file lines from the molecule and setting up min/ max
					Line[] moleculeLines = this.transform(lipid, swapYZ);
					if (moleculeLines == null) continue;
					
					for(Line line : moleculeLines) {
						// min/ max update
						Point3f p = line.position;
						min.x = Math.min(min.x, p.x);
						min.y = Math.min(min.y, p.y);
						min.z = Math.min(min.z, p.z);
						max.x = Math.max(max.x, p.x);
						max.y = Math.max(max.y, p.y);
						max.z = Math.max(max.z, p.z);
					}
					currentNumberOfActualType++;
				}
				// save the substring of the lipid type (which must match the topology identifier)
				// and the number of occurrences; this is only relevant for the GMX-Plugin
				topologyVec.add(new String[]{type.substring(0,3),Integer.toString(currentNumberOfActualType)});
			}
		}
		// -> is null if sortByType
		if (lipidTypesIntra != null) {
			for(String type : lipidTypesIntra) {
				int currentNumberOfActualType = 0;	
				/*IO*/ this.dialog.setProgress("Setting up min/max: " + type, (int) (currentStep / steps * 100), true);
				Lipid_3D[] lipidsIntraOfSameType = this.model.getLipidsByPDBCodeIntra(type);
				for(Lipid_3D lipid : lipidsIntraOfSameType) {

					// creating pdb file lines from the molecule and setting up min/ max
					Line[] moleculeLines = this.transform(lipid, swapYZ);
					if (moleculeLines == null) continue;
					
					for(Line line : moleculeLines) {
						// min/ max update
						Point3f p = line.position;
						min.x = Math.min(min.x, p.x);
						min.y = Math.min(min.y, p.y);
						min.z = Math.min(min.z, p.z);
						max.x = Math.max(max.x, p.x);
						max.y = Math.max(max.y, p.y);
						max.z = Math.max(max.z, p.z);
					}
					currentNumberOfActualType++;
				}
				// save the substring of the lipid type (which must match the topology identifier)
				// and the number of occurrences; this is only relevant for the GMX-Plugin
				topologyVec.add(new String[]{type.substring(0,3),Integer.toString(currentNumberOfActualType)});
			}
		}
		++currentStep;
		/* second loop - calculating cx, cy, cz; filling contents, transforming and saving molecules */
		this.cx = 0; this.cy = 0; this.cz = 0;
		if(((this.tHints != null) ? this.tHints.shiftXZ :
						DataPool.CoordinateTransformationSettings.shiftXZ) == 1 ||
							((this.tHints != null) ? this.tHints.shiftY :
								DataPool.CoordinateTransformationSettings.shiftY) > 0) {
			this.cx = ((this.tHints != null) ? this.tHints.shiftXZ :
						DataPool.CoordinateTransformationSettings.shiftXZ) == 1 ?
								(min.x + ((max.x - min.x) / 2)) : 0;
			this.cy = ((this.tHints != null) ? this.tHints.shiftY :
						DataPool.CoordinateTransformationSettings.shiftY) == 1 ?
								(min.y + ((max.y - min.y) / 2)) :
									((this.tHints != null) ? this.tHints.shiftY :
										DataPool.CoordinateTransformationSettings.shiftY) == 2 ?
												min.y : 0;
			this.cz = ((this.tHints != null) ? this.tHints.shiftXZ :
						DataPool.CoordinateTransformationSettings.shiftXZ) == 1 ?
								(min.z + ((max.z - min.z) / 2)) : 0;
		}
		// if shifting of XZ
		this.subX = 0; this.subZ = 0;
		if(((this.tHints != null) ? this.tHints.shiftXZ : 
						DataPool.CoordinateTransformationSettings.shiftXZ) == 2) {
			this.subX += this.model.getSize().width / 2;
			this.subZ += this.model.getSize().height / 2;
		}
		// calculating endOfSerialNumbers - important in saveMoleculeLines()
		if(this.useSerials == this.PROTSONLY) {
			this.endOfSerialNumbers = 	this.model.countAtoms(0, -1, -1, DataPool.PPESettings.saveOPMDummyLayersInPDB) +
										this.model.countTERs(0, -1, -1);
		} else {
			if(this.useSerials == this.BOTH) {
				this.endOfSerialNumbers = 	this.model.countAtoms(0, -1, -1, DataPool.PPESettings.saveOPMDummyLayersInPDB) +
									 		this.model.countAtoms(1, -1, -1, DataPool.PPESettings.saveOPMDummyLayersInPDB) +
									 		this.model.countAtoms(2, -1, -1, DataPool.PPESettings.saveOPMDummyLayersInPDB) +
									 		this.model.countTERs(0, -1, -1) +
									 		this.model.countTERs(1, -1, -1) +
									 		// TODO: this.model.countTERs(2, -1, -1) + ?
									 		this.model.countTERs(2, -1, -1);
			} else {
				this.endOfSerialNumbers = 0;
			}
		}
		
		// calculating CRYST values
		final float crystA  = max.x - min.x;
		final float crystB  = max.y - min.y;
		final float crystC  = max.z - min.z;
		
		// passing all proteins - transforming - filling content
		int resSeq = 1; 			// for counting the molecules -> if assignResSeqNumbers = true
		this.atomIndex = 0;			// for counting the atoms of all molecules in membrane
		this.terCount = 0;			// for counting TER atoms
		this.atomHetAtmCount = 0;	// for counting HetAtm atoms
		if (proteinTypes != null) {
			int progressIndex = 1;
			int sum = this.model.getProteinCount();
			for(String type : proteinTypes) {
				Protein_3D[] proteinsOfSameType = this.model.getProteinsByPDBCode(type);
				for(Protein_3D protein : proteinsOfSameType) {
					/*IO*/ this.dialog.setProgress("Transforming proteins: " + type 
									+ " (" + progressIndex++ + "/" + sum + ")" ,
													(int) (currentStep / steps * 100), true);
					// transforming molecule lines
					Line[] moleculeLines = this.transformAndConstructResidueHash(protein, resSeq, swapYZ, content);
					if(this.useSerials < this.NONE) {
						content.addConect(protein);
					}
					if(this.useChains < this.NONE) {
						try {
							content.setStructure(protein, moleculeLines);
						} catch (IOException e) {
							// handle exception here
							e.printStackTrace();
						}
					} else {
						content.removeChainsIDs(moleculeLines);
					}
					
					
					// =======================================================================
					// TODO: PDBWriter residue sequence update 04_2012						  |
					// =======================================================================
					// for the purpose of keeping secondary structures by enabling
					// 'assignResSeqNumbers' without touching the parallel integrity
					// of this 'writeMembrane()' method, the local variable resSeq has to be
					// updated after manipulating in 'this.transform()'.
					// Probably the use of a global 'resSeq' variable has the same effect, but
					// in this case the parallel class functionality is no longer warranted.
					if(DataPool.LimitationSettings.assignResSeqNumbers) {
						resSeq += content.resSeqMapping.size() - 1;
						// clear the sequence mapping for the next protein
						content.resSeqMapping.clear();
					}
					
					
					++resSeq;
					content.finishMolecule(protein);
					// saving transformed molecule lines in content
					this.saveMoleculeLines(moleculeLines, content, false);
				}
			}
			++currentStep;
		}
		// passing all lipids - transforming - filling content -> null if !sortByType
		if (lipidTypes != null) {
			int progressIndex = 1;
			int sum = this.model.getLipidCountExtra() + this.model.getLipidCountIntra();
			for(String type : lipidTypes) {
				Lipid_3D[] lipidsOfSameType = this.model.getLipidsByPDBCodeExtra(type);
				for(Lipid_3D lipid : lipidsOfSameType) {
					/*IO*/ this.dialog.setProgress("Transforming lipids: " + type 
							+ " (" + progressIndex++ + "/" + sum + ")" ,
											(int) (currentStep / steps * 100), true);
					Line[] moleculeLines = this.transformAndConstructResidueHash(lipid, resSeq, swapYZ, content);
					if(this.useSerials < this.PROTSONLY) {
						content.addConect(lipid);
					}
					if(this.useChains < this.PROTSONLY) {
						try {
							content.setStructure(lipid, moleculeLines);
						} catch (IOException e) {
							// handle exception here
							e.printStackTrace();
						}
					} else {
						content.removeChainsIDs(moleculeLines);
					}
					++resSeq;
					content.finishMolecule(lipid);
					// saving transformed molecule lines in content
					this.saveMoleculeLines(moleculeLines, content, true);
				}
				lipidsOfSameType = this.model.getLipidsByPDBCodeIntra(type);
				for(Lipid_3D lipid : lipidsOfSameType) {
					/*IO*/ this.dialog.setProgress("Transforming lipids: " + type 
							+ " (" + progressIndex++ + "/" + sum + ")" ,
											(int) (currentStep / steps * 100), true);
					Line[] moleculeLines = this.transformAndConstructResidueHash(lipid, resSeq, swapYZ, content);
					if(this.useSerials < this.PROTSONLY) {
						content.addConect(lipid);
					}
					if(this.useChains < this.PROTSONLY) {
						try {
							content.setStructure(lipid, moleculeLines);
						} catch (IOException e) {
							// handle exception here
							e.printStackTrace();
						}
					} else {
						content.removeChainsIDs(moleculeLines);
					}
					++resSeq;
					content.finishMolecule(lipid);
					// saving transformed molecule lines in content
					this.saveMoleculeLines(moleculeLines, content, true);
				}
			}
		}
		// passing all lipids - transforming - filling content -> null if sortByType
		if (lipidTypesExtra != null) {
			int progressIndex = 1;
			int sum = this.model.getLipidCountExtra();
			for(String type : lipidTypesExtra) {
				Lipid_3D[] lipidsExtraOfSameType = this.model.getLipidsByPDBCodeExtra(type);
				for(Lipid_3D lipid : lipidsExtraOfSameType) {
					/*IO*/ this.dialog.setProgress("Transforming lipids (extra): " + type 
							+ " (" + progressIndex++ + "/" + sum + ")" ,
											(int) (currentStep / steps * 100), true);
					Line[] moleculeLines = this.transformAndConstructResidueHash(lipid, resSeq, swapYZ, content);
					if(this.useSerials < this.PROTSONLY) {
						content.addConect(lipid);
					}
					if(this.useChains < this.PROTSONLY) {
						try {
							content.setStructure(lipid, moleculeLines);
						} catch (IOException e) {
							// handle exception here
							e.printStackTrace();
						}
					} else {
						content.removeChainsIDs(moleculeLines);
					}
					++resSeq;
					content.finishMolecule(lipid);
					// saving transformed molecule lines in content
					this.saveMoleculeLines(moleculeLines, content, true);
				}
			}
		}
		// passing all lipids - transforming - filling content -> null if sortByType
		if (lipidTypesIntra != null) {
			int progressIndex = 1;
			int sum = this.model.getLipidCountIntra();
			for(String type : lipidTypesIntra) {
				Lipid_3D[] lipidsIntraOfSameType = this.model.getLipidsByPDBCodeIntra(type);
				for(Lipid_3D lipid : lipidsIntraOfSameType) {
					/*IO*/ this.dialog.setProgress("Transforming lipids (intra): " + type 
							+ " (" + progressIndex++ + "/" + sum + ")" ,
											(int) (currentStep / steps * 100), true);
					Line[] moleculeLines = this.transformAndConstructResidueHash(lipid, resSeq, swapYZ, content);
					if(this.useSerials < this.PROTSONLY) {
						content.addConect(lipid);
					}
					if(this.useChains < this.PROTSONLY) {
						try {
							content.setStructure(lipid, moleculeLines);
						} catch (IOException e) {
							// handle exception here
							e.printStackTrace();
						}
					} else {
						content.removeChainsIDs(moleculeLines);
					}
					++resSeq;
					content.finishMolecule(lipid);
					// saving transformed molecule lines in content
					this.saveMoleculeLines(moleculeLines, content, true);
				}
			}
		}
		++currentStep;
		// beginning writing the header section into the output file
		/*IO*/ this.dialog.setProgress("Writing header ...", (int) (currentStep++ / steps * 100), true);
		this.remCount = 0;
		try {
			this.writeHeader(bWriter, content);
		} catch (IOException e) {
			// handle exception here
			e.printStackTrace();
		}
		// writing dbref
		/*IO*/ this.dialog.setProgress("Writing dbref ...", (int) (currentStep++ / steps * 100), true);
		for(String line : content.dbref) {
			try {
				bWriter.write(line);
				bWriter.newLine();
			} catch (IOException e) {
				// handle exception here
				e.printStackTrace();
			}
		}
		content.dbref.clear();
		// writing seqadv
		/*IO*/ this.dialog.setProgress("Writing seqadv ...", (int) (currentStep++ / steps * 100), true);
		for(String line : content.seqadv) {
			try {
				bWriter.write(line);
				bWriter.newLine();
			} catch (IOException e) {
				// handle exception here
				e.printStackTrace();
			}
		}
		content.seqadv.clear();
		// writing seqres
		/*IO*/ this.dialog.setProgress("Writing seqres ...", (int) (currentStep++ / steps * 100), true);
		for(String line : content.seqres) {
			try {
				bWriter.write(line);
				bWriter.newLine();
			} catch (IOException e) {
				// handle exception here
				e.printStackTrace();
			}
		}
		// writing modres
		/*IO*/ this.dialog.setProgress("Writing modres ...", (int) (currentStep++ / steps * 100), true);
		for(String line : content.modres) {
			try {
				bWriter.write(line);
				bWriter.newLine();
			} catch (IOException e) {
				// handle exception here
				e.printStackTrace();
			}
		}
		content.modres.clear();
		// writing het
		/*IO*/ this.dialog.setProgress("Writing het ...", (int) (currentStep++ / steps * 100), true);
		for(String line : content.het) {
			try {
				bWriter.write(line);
				bWriter.newLine();
			} catch (IOException e) {
				// handle exception here
				e.printStackTrace();
			}
		}
		// writing helix
		/*IO*/ this.dialog.setProgress("Writing helix ...", (int) (currentStep++ / steps * 100), true);
		for(String line : content.helix) {
			try {
				bWriter.write(line);
				bWriter.newLine();
			} catch (IOException e) {
				// handle exception here
				e.printStackTrace();
			}
		}
		// writing sheet
		/*IO*/ this.dialog.setProgress("Writing sheet ...", (int) (currentStep++ / steps * 100), true);
		for(String line : content.sheet) {
			try {
				bWriter.write(line);
				bWriter.newLine();
			} catch (IOException e) {
				// handle exception here
				e.printStackTrace();
			}
		}
		// writing turn
		/*IO*/ this.dialog.setProgress("Writing turn ...", (int) (currentStep++ / steps * 100), true);
		for(String line : content.turn) {
			try {
				bWriter.write(line);
				bWriter.newLine();
			} catch (IOException e) {
				// handle exception here
				e.printStackTrace();
			}
		}
		// writing ssbond
		/*IO*/ this.dialog.setProgress("Writing ssbond ...", (int) (currentStep++ / steps * 100), true);
		for(String line : content.ssbond) {
			try {
				bWriter.write(line);
				bWriter.newLine();
			} catch (IOException e) {
				// handle exception here
				e.printStackTrace();
			}
		}
		content.ssbond.clear();
		// writing link
		/*IO*/ this.dialog.setProgress("Writing link ...", (int) (currentStep++ / steps * 100), true);
		for(String line : content.link) {
			try {
				bWriter.write(line);
				bWriter.newLine();
			} catch (IOException e) {
				// handle exception here
				e.printStackTrace();
			}
		}
		content.link.clear();
		// writing cispep
		/*IO*/ this.dialog.setProgress("Writing cispep ...", (int) (currentStep++ / steps * 100), true);
		for(String line : content.cispep) {
			try {
				bWriter.write(line);
				bWriter.newLine();
			} catch (IOException e) {
				// handle exception here
				e.printStackTrace();
			}
		}
		content.cispep.clear();
		// writing site
		/*IO*/ this.dialog.setProgress("Writing site ...", (int) (currentStep++ / steps * 100), true);
		for(String line : content.site) {
			try {
				bWriter.write(line);
				bWriter.newLine();
			} catch (IOException e) {
				// handle exception here
				e.printStackTrace();
			}
		}
		// writing site
		/*IO*/ this.dialog.setProgress("Writing cryst section ...", (int) (currentStep++ / steps * 100), true);
		try {
			this.writeCRYST(bWriter, crystA, crystB, crystC);
		} catch (IOException e) {
			// handle exception here
			e.printStackTrace();
		}
		// writing molecule lines
		int progressIndex = 1;
		int sum = content.moleculeLines.size();
		for(String line : content.moleculeLines) {
			/*IO*/ this.dialog.setProgress("Writing molecule lines"
					+ " (" + progressIndex++ + "/" + sum + ")" ,
									(int) (currentStep / steps * 100), true);
			try {
				bWriter.write(line);
				bWriter.newLine();
			} catch (IOException e) {
				// handle exception here
				e.printStackTrace();
			}
		}
		++currentStep;
		content.moleculeLines.clear();
		// writing conects
		/*IO*/ this.dialog.setProgress("Writing conects ...", (int) (currentStep++ / steps * 100), true);
		for(String line : content.conect) {
			try {
				bWriter.write(line);
				bWriter.newLine();
			} catch (IOException e) {
				// handle exception here
				e.printStackTrace();
			}
		}
		// writing master section
		/*IO*/ this.dialog.setProgress("Writing master section ...", (int) (currentStep++ / steps * 100), true);
		try {
			this.writeMaster(bWriter, content);
			bWriter.write("END");
			bWriter.close();
		} catch (IOException e) {
			// handle exception here
			e.printStackTrace();
		}
		
		// check if the topology should be exported
		if (!tHints.exportAPLVoroFile && 
				(tHints.exportTopology || DataPool.LimitationSettings.exportTopology)) {
			
			if (!exportTopology(topologyVec))
				return false;
			
		}
		
		// check if the APLVoro config file should be exported
		if (tHints.exportAPLVoroFile || DataPool.LimitationSettings.exportAPLVoro){
			
			if (!exportAPLVoro(aplVec, tHints.exportAPLVoroFile))
				return false;
			
		}
		
		// System.gc();
		/*IO*/ this.dialog.setProgress("Finishing ...", (int) (currentStep++ / steps * 100), true);
		/*IO*/ this.dialog.setProgress("Finished", (int) (currentStep / steps * 100), true);
		
		return true;
		
	}

	/*
	 * GROMACS File Generation
	 * These methods are mainly intended for the GMX-Plugin. 
	 * Only the topology file may be written without this plug-in.
	 * 
	 * @author Sebastian Rubert
	 */
	public boolean exportTopology(Vector<String[]> topologyVec) {
		
		try {
			
			// create a new folder if this option was not chosen by the GMX-plugin
			if (!tHints.exportTopology && DataPool.LimitationSettings.exportTopology) {
				tHints = new CoordinateTransformationHints();
				tHints.projectFolder = outputFile.getAbsolutePath().substring(0, outputFile.getAbsolutePath().length()-4);
				File projectDir = new File(tHints.projectFolder);
				if (projectDir.exists())
					Cm2_Toolkit.deleteDir(projectDir,true);
				projectDir.mkdir();
			}

			// write Topology File
			BufferedWriter out = new BufferedWriter( new FileWriter(tHints.projectFolder  + File.separator + "topol.top"));
			out.write("; Include forcefield parameters\n");
			out.write("#include \"ffG45a3.itp\"\n");
			out.write("\n");
			out.write("; Include Content\n");
			out.write("#include \"lipidsFFG45a3.itp\"\n");
			out.write("#include \"ions.itp\"\n");
			out.write("\n");
			out.write("; Include water\n");
			out.write("#include \"flexspce.itp\"\n");
			out.write("\n");
			out.write("; System Name\n");
			out.write("[System]\n");
			out.write("lipids in water\n");
			out.write("\n");
			out.write("; Numbers of Molecules in their original order as found in the PDB file\n"); // "Anzahl der Mol in der pdb in gleicher reihenfolge"
			out.write("[molecules]\n");
			for(String[] lipidString : topologyVec){
				out.write(lipidString[0] + "              " + lipidString[1] + "\n");
			}     
            out.close();	

            
            if (tHints.exportTopology) {
      
            	// write Mdp Files
            
	            // - 1. write minimize.mdp
	            BufferedReader in = new BufferedReader( new FileReader(tHints.projectFolder  + File.separator + "minimize.mdp"));
//	            System.out.println("Read File : " + tHints.projectFolder  + File.separator + "minimize.mdp");
	            String minMdp = "";
	            String line;
	            line = in.readLine();
	            if (line != null){
	            	
	            	do{
	                	minMdp = minMdp + line + "\n";
	                	line = in.readLine();
	                }while(line != null);
	            }
	            in.close();
	            
	            //find >energygrps< and split the string
	            int energyPos = minMdp.indexOf(";ENERGYGRPS");
	            //search of "energygroups" and insert the lipidgroups
	            String mdpBegin = minMdp.substring(0,energyPos);
	            String mdpEnd = minMdp.substring(energyPos + "ENERGYGRPS".length() + 1);
	            String mdpLipids = "";
	            //paste in the lipid types
	            for(String[] lipidString : topologyVec){
					mdpLipids = mdpLipids + " " + lipidString[0] ;
				}  
	            
	            // - 2. write minimize.mdp file
	            String output = mdpBegin + "energygrps                =" + mdpLipids + "\n" + mdpEnd;
	            out = new BufferedWriter( new FileWriter(tHints.projectFolder  + File.separator + "minimize.mdp"));
	            out.write(output);
	            out.close();
	            
	            // - 3. write NPT.mdp
	            in = new BufferedReader( new FileReader(tHints.projectFolder  + File.separator + "NPT.mdp"));
	            minMdp = "";
	            line = in.readLine();
	            if (line != null){
	            	do{
	            		minMdp = minMdp + line + "\n";
	                	line = in.readLine();
	                }while(line != null);
	            }
	            in.close();
	           
	            int numbLipids = 0;
	            energyPos = minMdp.indexOf(";ENERGYGRPS");
	            //search of "energygroups" and insert the lipidgroups
	            mdpBegin = minMdp.substring(0,energyPos);
	            mdpEnd = minMdp.substring(energyPos + "ENERGYGRPS".length() + 1);
	            mdpLipids = "";
	            for(String[] lipidString : topologyVec){
					mdpLipids = mdpLipids + " " + lipidString[0] ;
					numbLipids++;
				}  
	            output = mdpBegin + "energygrps                =" + mdpLipids + "\n" + mdpEnd;
	            
	            //search for tc-groups, tau_t and ref_t and insert standard values
	            int tcgrpsPos = output.indexOf(";TCGRPS");
	            mdpBegin = output.substring(0,tcgrpsPos);
//	            System.out.println(output.substring(0,tcgrpsPos) + "<-- MDPBEGIN");
//	            System.out.println(output.substring(tcgrpsPos) + "<-- MDPEND");
	            mdpEnd = output.substring(tcgrpsPos + "TCGRPS".length() +1);
	            output = mdpBegin + "tc-grps                 =" + mdpLipids  + " SOL " + mdpEnd;
	            
	            String taut = "0.1";
	            int tautPos = output.indexOf(";TAUT");
	            mdpBegin = output.substring(0,tautPos);
	            mdpEnd = output.substring(tautPos  + "TAUT".length() +1);
	            for (int i=0; i<numbLipids; i++){
	            	taut = taut + " 0.1"; 
	            }
	            output = mdpBegin + "tau_t                = " + taut + mdpEnd;
	            
	            String reft = "300";
	            int reftPos = output.indexOf(";REFT");
	            mdpBegin = output.substring(0,reftPos);
	            mdpEnd = output.substring(reftPos  + "REFT".length() +1);
	            for (int i=0; i<numbLipids; i++){
	            	reft = reft + " 300"; 
	            }
	            output = mdpBegin + "ref_t                = " + reft + mdpEnd;
	            
	            out = new BufferedWriter( new FileWriter(tHints.projectFolder  + File.separator + "NPT.mdp"));
	            out.write(output);
	            out.close();
	            
	            // - 4. write dynamics.mdp
	            in = new BufferedReader( new FileReader(tHints.projectFolder  + File.separator + "dynamics.mdp"));
	            minMdp = "";
	            line = in.readLine();
	            if (line != null){
	            	do{
	            		minMdp = minMdp + line + "\n";
	                	line = in.readLine();
	                }while(line != null);
	            }
	            in.close();
	            
	            energyPos = minMdp.indexOf(";ENERGYGRPS");
	            // search for "energygroups" and insert the lipidgroups
	            mdpBegin = minMdp.substring(0,energyPos);
	            mdpEnd = minMdp.substring(energyPos + "ENERGYGRPS".length() + 1);
	            mdpLipids = "";
	            for(String[] lipidString : topologyVec){
					mdpLipids = mdpLipids + " " + lipidString[0] ;
				}  
	            output = mdpBegin + "energygrps                =" + mdpLipids + "\n" + mdpEnd;
	            
	            // search for tc-groups, tau_t and ref_t and insert standard values
	            tcgrpsPos = output.indexOf(";TCGRPS");
	            mdpBegin = output.substring(0,tcgrpsPos);
//	            System.out.println(output.substring(0,tcgrpsPos) + "<-- MDPBEGIN");
//	            System.out.println(output.substring(tcgrpsPos) + "<-- MDPEND");
	            mdpEnd = output.substring(tcgrpsPos + "TCGRPS".length() +1);
	            output = mdpBegin + "tc-grps                 =" + mdpLipids  + " SOL " + mdpEnd;
	            
	            tautPos = output.indexOf(";TAUT");
	            mdpBegin = output.substring(0,tautPos);
	            mdpEnd = output.substring(tautPos  + "TAUT".length() +1);
	            output = mdpBegin + "tau_t                = " + taut + mdpEnd;
	            
	            reftPos = output.indexOf(";REFT");
	            mdpBegin = output.substring(0,reftPos);
	            mdpEnd = output.substring(reftPos  + "REFT".length() +1);
	            output = mdpBegin + "ref_t                = " + reft + "\n" + mdpEnd;
	            
	            out = new BufferedWriter( new FileWriter(tHints.projectFolder  + File.separator + "dynamics.mdp"));
	            out.write(output);
	            out.close();
	            
	            // write charge of given membrane: lipid_charge.txt
	            in = new BufferedReader( new FileReader(tHints.projectFolder  + File.separator + "lipid_charge.txt"));
	            Vector<String> lipidChargeVec = new Vector<String>();
	            
	            //this variable will hold the membrane charge
	            int lipid_charge = 0;
	            while ((line = in.readLine()) != null){
	            	lipidChargeVec.add(line);
	            }
	            
	            HashMap<String,String> lipidChargeMap = new HashMap<String,String>();
	            for (String s : lipidChargeVec){
	            	String [] lineArray = s.split(" ");
	            	lipidChargeMap.put(lineArray[0], lineArray[1]);
	            }
	            in.close();
	            
	            // search for lipids and calculate charge
	            HashSet<String> errorLipid = new HashSet<String>();
	        	for(String[] lipidString : topologyVec){
	        		if (lipidChargeMap.get(lipidString[0])!=null) {
	        			lipid_charge = lipid_charge + (Integer.parseInt(lipidString[1])*Integer.parseInt(lipidChargeMap.get(lipidString[0])));
	        		} else {
	        			if (!errorLipid.contains(lipidString[0])) {
	        				Cm2_Toolkit.complain("PDBWriter Problem: There was no lipid charge defined for "+lipidString[0]+
	        						" in the file lipid_charge.txt.\nSetting for "+lipidString[0]+" charge to 0.","Missing Lipid Charge");
	        				errorLipid.add(lipidString[0]);
	        			}
	        		}
	        	}
	            
//	            System.out.println("Lipid Charge " + lipid_charge);
	            
	        	// write charge of given membrane: membrane_charge.txt
	            out = new BufferedWriter( new FileWriter(tHints.projectFolder  + File.separator + "membrane_charge.txt"));
	            out.write(String.valueOf(lipid_charge));
	            out.close();
	            
            }
            
            return true;
            
		} catch (Exception e) {
			e.printStackTrace();
			Cm2_Toolkit.complain(e.getMessage(), "PDBWriter Error: Gromacs Files could not be written");
			return false;
			
		}
        
	}
	
	/*
	 * APL@Voro Config File Generation
	 * These methods are required to analyze a membrane with APL@Voro.
	 * @author Bjoern Sommer
	 */
	public boolean exportAPLVoro(Vector<String[]> aplVec, boolean aplPreviewButtonPressed) {

		try {
			
			BufferedWriter out = null;

			// save there the APLVoroFile if the Analyze Button was pressed				
			if (aplPreviewButtonPressed) {
				out = new BufferedWriter( new FileWriter(CM2_CONSTANTS.IO.TEMP_DIRECTORY + File.separator + CM2_CONSTANTS.IO.TEMP_APLVORO_FILE.getName()));
			
			// create a new folder and save there the APLVoroFile if chosen by the PDB Properties
			} else if (DataPool.LimitationSettings.exportAPLVoro){
				tHints = new CoordinateTransformationHints();
				tHints.projectFolder = outputFile.getAbsolutePath().substring(0, outputFile.getAbsolutePath().length()-4);
				String outputFileName = this.outputFile.getName().substring(0, outputFile.getName().length()-4);
				
				File projectDir = new File(tHints.projectFolder);
				if (projectDir.exists() && !(tHints.exportTopology || DataPool.LimitationSettings.exportTopology))
					Cm2_Toolkit.deleteDir(projectDir,true);
				projectDir.mkdir();
				out = new BufferedWriter( new FileWriter(tHints.projectFolder  + File.separator + outputFileName + ".aplvoro"));
			}	

			// write APL@Voro File
			out.write("Lipids =");
			for(int i=0; i<aplVec.size(); i++){
				if (i==0)
					out.write(" " + aplVec.get(i)[0]);
				else
					out.write(", " + aplVec.get(i)[0]);
			}
			out.write("\n");
			for(String[] lipidString : aplVec){
				out.write(lipidString[0] + " = " + lipidString[1] + "\n");
			}     
            out.close();	
            
            return true;

		} catch (Exception e) {
			e.printStackTrace();
			Cm2_Toolkit.complain(e.getMessage(), "PDBWriter Error: APL@Voro Files could not be written");
			return false;
			
		}
        
	}

	/**
	 * This method generates an intern copy of the DataPool MembraneModel - but
	 * copies only(!) the necessary elements.
	 * 
	 * @param domainID - the identification number of the domain
	 * @param layerID - the identification number of the layer
	 * @return Returns a reduced copy of the original MembraneModel.
	 */
	private MembraneModel getAlteredModel(final int domainID, final int layerID) {
		// generating two membrane models
		final MembraneModel model = DataPool.getMembraneModel();
		final MembraneModel altered = new MembraneModel();
		// setting up the altered model - a copy from the DataPool model
		altered.setAlgorithmdescription(model.getAlgorithmDescription());
		altered.setAlgorithmname(model.getAlgorithmName());
		altered.setAlgorithmStringRepresentation(model.getAlgorithmStringRepresentation());
		altered.setAuthor(model.getAuthor());
		altered.setCode(model.getCode());
		altered.setDate(model.getDate());
		altered.setExpdta(model.getEXPDTA());
		altered.setHeader(model.getHeader());
		altered.setJrnl(model.getJRNL());
		altered.setKeywds(model.getKEYWDS());
		altered.setRemark(model.getRemark());
		altered.setSize(model.getSize());
		altered.setDecimalPlaces(model.getDecimalPlaces());
		altered.setComputeAbsoluteValues(model.getComputeAbsoluteValues());
		altered.setTitle(model.getTitle());
		// adding the layer
		for (int i = 0; i < model.getLayerCount(); i++) {
			altered.addLayer(model.getLayer(i));
		}
		// adding the microdomain
		for (int i = 0; i < model.countMicroDomains(); i++) {
			altered.addMicroDomain(model.getMicroDomain(i));
		}
		// adding the molecules
		Molecule_3D[] mols = model.getAllProteinsInDomain(domainID, layerID);
		for (int i = 0; i < mols.length; i++) {
			altered.addProtein((Protein_3D) mols[i].copy(true, false),
					mols[i].getLayerID());
		}
		mols = model.getAllLipidsExtraInDomain(domainID, layerID);
		for (int i = 0; i < mols.length; i++) {
			altered.addLipidExtra((Lipid_3D) mols[i].copy(true, false),
					mols[i].getDomainID(), mols[i].getLayerID());
		}
		mols = model.getAllLipidsIntraInDomain(domainID, layerID);
		for (int i = 0; i < mols.length; i++) {
			altered.addLipidIntra((Lipid_3D) mols[i].copy(true, false),
					mols[i].getDomainID(), mols[i].getLayerID());
		}
		// adding the ExampleLipids
		for (int i = 0; i < model.getExampleLipidsCountExtra(); i++)
			altered.addExampleLipidExtra(model.getExampleLipidAtExtra(i),
					model.getExampleLipidAtExtra(i).getDomainID(),
					model.getExampleLipidAtExtra(i).getLayerID());
		for (int i = 0; i < model.getExampleLipidsCountIntra(); i++)
			altered.addExampleLipidIntra(model.getExampleLipidAtIntra(i),
					model.getExampleLipidAtIntra(i).getDomainID(),
					model.getExampleLipidAtIntra(i).getLayerID());
		// checking for more than one layer
		if (layerID > 1) {
			final float y = model.getLayerByID(layerID).getYHeight();
			final Molecule_3D[] m = altered.getAllMoleculesInMembrane();
			for (int i = 0; i < m.length; i++) {
				m[i].setYTranslation(m[i].getYTranslation() - y);
			}
		}
		// sorting molecules and ExampleLipids
		Molecule_3D[] mm = altered.getAllMoleculesInMembrane();
		for(int i = 0; i < mm.length; i++) {
			mm[i].sortAtoms();
		}
		for(int i = 0; i < altered.getExampleLipidsCountExtra(); i++) {
			altered.getExampleLipidAtExtra(i).sortAtoms();
		}
		for(int i = 0; i < altered.getExampleLipidsCountIntra(); i++) {
			altered.getExampleLipidAtIntra(i).sortAtoms();
		}
		// returning the reduced copy of model
		return altered;
	}
	
	/**
	 * This Method performs necessary transformations with the representing
	 * PDB file lines of the given molecule. In addition, a hash is constructed
	 * containing the residue sequences.
	 * 
	 * @param molecule - the molecule to convert into PDB file lines and to transform
	 * @param resSeq - the index number of the given molecule (related to the membrane its taken from)
	 * @return Returns the representing PDB file lines - transformed - of the given molecule
	 */
	private Line[] transformAndConstructResidueHash(final Molecule_3D molecule, final int resSeq, final boolean swapYZ, Content content) {
		// transform position and matrix of molecule
		Line[] moleculeLines = null;
		try {
			moleculeLines = this.createLines(molecule);
		} catch (IOException e) {
			// handle exception here
			e.printStackTrace();
			return moleculeLines;
		}
		
		
		// constructs a HashMap for updating the residue sequence number if
		// 'assignResSeq' is enabled.
		// ===========================================================================
		// TODO: PDBWriter residue sequence update 04_2012							  |
		// ===========================================================================
		// local access must be warranted
		List<Integer> resSeqList = null;
		if(DataPool.LimitationSettings.assignResSeqNumbers) {
			if(molecule instanceof Protein_3D) {
				// molecule is a protein
				resSeqList = new ArrayList<Integer>();
				for(Line line : moleculeLines) {
					// going through every line and register every residue
					// sequence number of the protein
					int currentResSeq = new Integer(line.resSeq);
					if(!resSeqList.contains(currentResSeq)) {
						resSeqList.add(currentResSeq);
					}
				}
				// arrange the residue sequence list
				Integer[] array = new Integer[resSeqList.size()];
				resSeqList.toArray(array);
				Arrays.sort(array);
				// overwrite unsorted residue sequence list
				resSeqList = Arrays.asList(array);
			}
		}
		
		
		Matrix3f matrix = molecule.getRotationMatrix();
		Point3f position = molecule.getPosition();
		for (Line line : moleculeLines) {
			if(line != null) {
				
				
				// replace the original resSeq number, taken from the original
				// molecule file, with a running number, counting the molecules.
				// ===========================================================================
				// TODO: PDBWriter residue sequence update 04_2012							  |
				// ===========================================================================
				// if the current molecule is a protein and in case of 'assignResSeqNumbers'
				// is true/enabled the secondary structure must be retained.
				// so 'resSeq' has to be internally updated.
				if(DataPool.LimitationSettings.assignResSeqNumbers) {
					if(molecule instanceof Protein_3D) {
						// molecule is a protein
						final int currentResSeq = new Integer(line.resSeq);
						if(!content.resSeqMapping.containsKey(currentResSeq)) {
							// sequence mapping does not contain the current original
							// residue sequence number of the protein (key)
							int index = resSeqList.indexOf(currentResSeq);
							line.resSeq = "" + (resSeq + index);
							content.resSeqMapping.put(currentResSeq, (resSeq + index));
						} else {
							// sequence mapping contains the current original residue
							// sequence number of the protein (key)
							line.resSeq = "" + content.resSeqMapping.get(currentResSeq);
						}
					} else {
						// molecule is a lipid
						line.resSeq = "" + resSeq;
					}
				}
				
				
				line.position = Cm2_Toolkit.transformToNewPoint(matrix, line.position);
				line.position = new Point3f(line.position.x + position.x - this.subX - this.cx,
											line.position.y + position.y - this.cy,
											line.position.z + position.z - this.subZ - this.cz);
				if(swapYZ) {
					line.position = new Point3f(line.position.x,
												line.position.z,
												line.position.y);
				}
				++content.listSize;
				if(line.atomOfMol != null) {
					line.atomOfMol.indexForPDBWriter = content.listSize;
				}
			}
		}
		return moleculeLines;
	}
	
	/**
	 * This Method performs necessary transformations with the representing
	 * PDB file lines of the given molecule.
	 * 
	 * @param molecule - the molecule to convert into PDB file lines and to transform
	 * @return Returns the representing PDB file lines - transformed - of the given molecule
	 */
	private Line[] transform(final Molecule_3D molecule, final boolean swapYZ) {
		// transform position and matrix of molecule
		Line[] moleculeLines = null;
		try {
			moleculeLines = this.createLines(molecule);
		} catch (IOException e) {
			// handle exception here
			e.printStackTrace();
			return moleculeLines;
		}
		
		Matrix3f matrix = molecule.getRotationMatrix();
		Point3f position = molecule.getPosition();
		for (Line line : moleculeLines) {
			if(line != null) {
				
				line.position = Cm2_Toolkit.transformToNewPoint(matrix, line.position);
				line.position = new Point3f(line.position.x + position.x - this.subX - this.cx,
											line.position.y + position.y - this.cy,
											line.position.z + position.z - this.subZ - this.cz);
				if(swapYZ) {
					line.position = new Point3f(line.position.x,
												line.position.z,
												line.position.y);
				}
			}
		}
		return moleculeLines;
	}
	
	/**
	 * 
	 * @param moleculeLines
	 * @param content
	 */
	private void saveMoleculeLines(final Line[] moleculeLines, Content content, boolean atomEntryOnly ) {
		// writing/ saving a line
		for(Line line : moleculeLines) {
			String out = "";
			out += Cm2_Toolkit.extendRight(atomEntryOnly?"ATOM":line.recordName, 6);
			if(this.atomIndex < this.endOfSerialNumbers || !DataPool.LimitationSettings.all99999) {
				out += Cm2_Toolkit.extendLeft("" + (this.atomIndex + 1), 5);
			} else {
				out += Cm2_Toolkit.extendLeft("99999", 5);
			}
			out += Cm2_Toolkit.extendLeft(line.name, 5);
			line.altLoc = "";
			out += Cm2_Toolkit.extendLeft(line.altLoc, 1);
			out += Cm2_Toolkit.extendLeft(line.residueName, 3);
			out += Cm2_Toolkit.extendLeft(line.chainId, 2);
			out += Cm2_Toolkit.extendLeft(line.resSeq, 4);
			out += Cm2_Toolkit.extendLeft(line.iCode, 1);
			out += "   ";
			if(line.recordName.startsWith("TER")) {
				out += Cm2_Toolkit.extendLeft("", 24);
				++this.terCount;
			} else {
				out += Cm2_Toolkit.extendLeft(this.createNumberString(line.position.x), 8);
				out += Cm2_Toolkit.extendLeft(this.createNumberString(line.position.y), 8);
				out += Cm2_Toolkit.extendLeft(this.createNumberString(line.position.z), 8);
				++this.atomHetAtmCount;
			}
			out += Cm2_Toolkit.extendLeft(line.occupancy, 6);
			out += " ";
			out += Cm2_Toolkit.extendLeft(line.tempFactor, 5);
			out += "       ";
			out += Cm2_Toolkit.extendLeft(line.segID, 3);
			out += Cm2_Toolkit.extendLeft(line.element, 2);
			out += Cm2_Toolkit.extendLeft(line.charge, 2);
			++this.atomIndex;
			if(this.atomIndex > 99998 && DataPool.LimitationSettings.all99999 == false) {
				if (!DataPool.LimitationSettings.restartNumberingWith0)
					this.atomIndex = 0;
				else 
					this.atomIndex = -1;
			}
			// adding out string to content
			content.addMoleculeLine(out);
		}
	}
	
	
	/**
	 * Writes the MetaInfo into the given output file.
	 * 
	 * @throws IOException
	 */
	private void writeHeader(BufferedWriter bWriter, Content content) throws IOException {
		String firstone = "HEADER    ";
		String header = "";
		if (this.model.getHeader() != null) {
			header = this.model.getHeader();
		}
		firstone += Cm2_Toolkit.extendRight(header, 40);
		String date = "";
		if (this.model.getDate() != null) {
			date = this.model.getDate();
		}
		firstone += Cm2_Toolkit.extendRight(date, 12);
		String code = "";
		if (this.model.getCode() != null) {
			code = this.model.getCode();
		}
		firstone += Cm2_Toolkit.extendRight(code, 4);
		firstone += "    ";
		// writing into output file
		bWriter.write(firstone.toUpperCase());
		bWriter.newLine();
		if (this.model.getTitle() != null) {
			if (this.model.getTitle().length() > 0) {
				final String[] strings = this.model.getTitle().split("\n");
				for (int i = 0; i < strings.length; i++) {
					String s = "";
					s += "TITLE ";
					s += strings[i];
					bWriter.write(s.toUpperCase());
					bWriter.newLine();
				}
			}
		}

		for (int i = 0; i < content.compnd.size(); i++) {
			bWriter.write(content.compnd.get(i));
			bWriter.newLine();
		}
		if (this.model.getSOURCE() != null) {
			if (this.model.getSOURCE().length() > 0) {
				final String[] strings = this.model.getSOURCE().split("\n");
				for (int i = 0; i < strings.length; i++) {
					String species = "";
					species += "SOURCE";
					species += strings[i];
					bWriter.write(species.toUpperCase());
					bWriter.newLine();
					if (i > 9999) {
						break;
					}
				}
			}
		}
		if (this.model.getKEYWDS() != null) {
			if (this.model.getKEYWDS().length() > 0) {
				final String[] strings = this.model.getKEYWDS().split("\n");
				for (int i = 0; i < strings.length; i++) {
					String s = "";
					s += "KEYWDS ";
					s += strings[i];
					bWriter.write(s.toUpperCase());
					bWriter.newLine();
					if (i > 9999) {
						break;
					}
				}
			}
		}
		if (this.model.getEXPDTA() != null) {
			if (this.model.getEXPDTA().length() > 0) {
				final String[] strings = this.model.getEXPDTA().split("\n");
				for (int i = 0; i < strings.length; i++) {
					String s = "";
					s += "EXPDTA";
					s += strings[i];
					bWriter.write(s.toUpperCase());
					bWriter.newLine();
					if (i > 9999) {
						break;
					}
				}
			}
		}
		if (this.model.getAuthor() != null) {
			if (this.model.getAuthor().length() > 0) {
				final String[] strings = this.model.getAuthor().split("\n");
				for (int i = 0; i < strings.length; i++) {
					String author = "";
					author += "AUTHOR";
					author += strings[i];
					bWriter.write(author.toUpperCase());
					bWriter.newLine();
					if (i > 9999) {
						break;
					}
				}
			}
		}
		if (this.model.getJRNL() != null) {
			if (this.model.getJRNL().length() > 0) {
				final String[] strings = this.model.getJRNL().split("\n");
				for (int i = 0; i < strings.length; i++) {
					String s = "";
					s += "JRNL  ";
					s += strings[i];
					bWriter.write(s.toUpperCase());
					bWriter.newLine();
					if (i > 9999) {
						break;
					}
				}
			}
		}
		final String remark = this.createAutoRemark(content);
		final String[] strings = remark.split("\n");
		for (int i = 0; i < strings.length; i++) {
			String c = "";
			c += "REMARK";
			c += "    ";
			c += strings[i];
			bWriter.write(c);
			bWriter.newLine();
			this.remCount++;
		}
		if (content.positionList.isEmpty() == false) {
			bWriter.write("REMARK");
			bWriter.newLine();
			bWriter.write("REMARK");
			bWriter.newLine();
			bWriter.write("REMARK    POSITIONS OF MOLECULE INSTANCES BY ATOM SERIALS");
			bWriter.newLine();
			bWriter.write("REMARK");
			bWriter.newLine();
			this.remCount += 4;
			for (int i = 0; i < content.positionList.size(); i++) {
				String s = "";
				s += "REMARK    ";
				s += content.positionList.get(i);
				bWriter.write(s);
				bWriter.newLine();
				this.remCount++;
			}
		}
		if (this.model.getRemark() != null) {
			if (this.model.getRemark().length() > 0) {
				bWriter.write("REMARK");
				bWriter.newLine();
				bWriter.write("REMARK    START OF USER DEFINED REMARKS");
				bWriter.newLine();
				bWriter.write("REMARK");
				bWriter.newLine();
				this.remCount += 3;
				final String[] string = this.model.getRemark().split("\n");
				for (int i = 0; i < string.length; i++) {
					String s = "";
					s += "REMARK";
					s += string[i];
					bWriter.write(s.toUpperCase());
					bWriter.newLine();
					this.remCount++;
				}
			}
		}
		if (content.xmlContent.isEmpty() == false) {
			bWriter.write("REMARK");
			bWriter.newLine();
			bWriter.write("REMARK    COPY OF THE MEMBRANE MODEL BEGINS HERE");
			bWriter.newLine();
			bWriter.write("REMARK");
			bWriter.newLine();
			for (int i = 0; i < content.xmlContent.size(); i++) {
				bWriter.write("REMARK " + content.xmlContent.get(i));
				bWriter.newLine();
				this.remCount++;
			}
			bWriter.write("REMARK    COPY OF THE MEMBRANE MODEL ENDS HERE");
			bWriter.newLine();
			bWriter.write("REMARK");
			bWriter.newLine();
			this.remCount += 5;
			content.xmlContent.clear();
			if (tHints != null) {
				bWriter.write("REMARK    SHIFTXZ=" + tHints.shiftXZ);
				bWriter.newLine();
				bWriter.write("REMARK    SHIFTY=" + tHints.shiftY);
				bWriter.newLine();
				bWriter.write("REMARK    SWAPYZ=" + tHints.swapYZ);
				bWriter.newLine();
				bWriter.write("REMARK    SORTBYTYPE=" + tHints.sortByType);
				bWriter.newLine();
				bWriter.write("REMARK    COMPUTEABSOLUTEVALUES=" + model.getComputeAbsoluteValues());
				bWriter.newLine();
				bWriter.write("REMARK");
				bWriter.newLine();
				this.remCount += 5;
			}
			System.gc();
		}
		if (content.adopts.isEmpty() == false) {
			bWriter.write("REMARK");
			bWriter.newLine();
			bWriter.write("REMARK    SECTION OF ADOPTED TITLE SECTIONS BEGINS HERE");
			bWriter.newLine();
			bWriter.write("REMARK");
			bWriter.newLine();
			this.remCount += 3;
			for (int i = 0; i < content.adopts.size(); i++) {
				bWriter.write(content.adopts.get(i));
				bWriter.newLine();
				this.remCount++;
			}
			content.adopts.clear();
			System.gc();
		}
	}
	
	
	/**
	 * Writes the CRYST info into a given file.
	 * 
	 * @param bWriter
	 * @param a
	 * @param b
	 * @param c
	 * @throws IOException
	 */
	private void writeCRYST(BufferedWriter bWriter, final float a, final float b, final float c)
		throws IOException {
		final String s = "CRYST1" 	+ Cm2_Toolkit.extendLeft(this.createNumberString(a), 9)
									+ Cm2_Toolkit.extendLeft(this.createNumberString(b), 9)
									+ Cm2_Toolkit.extendLeft(this.createNumberString(c), 9)
									+ "  90.00  90.00  90.00 P 1           1";
		bWriter.write(s);
		bWriter.newLine();
		bWriter.write("ORIGX1      1.000000  0.000000  0.000000        0.00000                         ");
		bWriter.newLine();
		bWriter.write("ORIGX2      0.000000  1.000000  0.000000        0.00000                         ");
		bWriter.newLine();
		bWriter.write("ORIGX3      0.000000  0.000000  1.000000        0.00000                         ");
		bWriter.newLine();
	}
	
	
	/**
	 * Writes the master line at the end of the file.
	 * 
	 * @throws IOException
	 */
	private void writeMaster(BufferedWriter bWriter, Content content) throws IOException {
		String master = "MASTER";
		master += Cm2_Toolkit.extendLeft(this.getNumString(this.remCount, 9), 9);
		master += Cm2_Toolkit.extendLeft("" + 0, 5);
		master += Cm2_Toolkit.extendLeft(this.getNumString(content.het.size(), 5), 5);
		content.het.clear();
		master += Cm2_Toolkit.extendLeft(this.getNumString(content.helix.size(), 5), 5);
		content.helix.clear();
		master += Cm2_Toolkit.extendLeft(this.getNumString(content.sheet.size(), 5), 5);
		content.sheet.clear();
		master += Cm2_Toolkit.extendLeft(this.getNumString(content.turn.size(), 5), 5);
		content.turn.clear();
		master += Cm2_Toolkit.extendLeft(this.getNumString(content.site.size(), 5), 5);
		content.site.clear();
		master += Cm2_Toolkit.extendLeft("" + 0, 5);
		master += Cm2_Toolkit.extendLeft(this.getNumString(this.atomHetAtmCount, 5), 5);
		master += Cm2_Toolkit.extendLeft(this.getNumString(this.terCount, 5), 5);
		master += Cm2_Toolkit.extendLeft(this.getNumString(content.conect.size(), 5), 5);
		content.conect.clear();
		master += Cm2_Toolkit.extendLeft(this.getNumString(content.seqres.size(), 5), 5);
		content.seqres.clear();
		bWriter.write(master);
		bWriter.newLine();
	}
	
	/**
	 * Creates the auto remarks.
	 * 
	 * @return
	 * @throws IOException
	 */
	private String createAutoRemark(Content content) throws IOException {
		String remark = "APPLICATION REMARKS:\n \n ";
		remark += "BUILD WITH : " + CM2_CONSTANTS.GENERAL.NAME_OF_THIS_PROGRAM + "\n ";
		remark += "             " + CM2_CONSTANTS.GENERAL.VERSION + "\n ";
		remark += "MORE INFO  : " + CM2_CONSTANTS.GENERAL.CM2_HOMEPAGE + "\n \n";
		if (this.model.getAlgorithmDescription() != null
				|| this.model.getAlgorithmName() != null
				|| this.model.getAlgorithmStringRepresentation() != null) {
			remark += "ALGORITM USED FOR BILAYER CALCULATION  : " + "\n ";
			if (this.model.getAlgorithmName() != null) {
				remark += "NAME\n";
				remark += this.model.getAlgorithmName();
				remark += "\n\n";
			}
			if (this.model.getAlgorithmDescription() != null) {
				remark += "DESCRIPTION\n";
				remark += this.model.getAlgorithmDescription();
				remark += "\n\n";
			}
			if (this.model.getAlgorithmStringRepresentation() != null) {
				remark += "SETUP INFORMATION\n";
				remark += this.model.getAlgorithmStringRepresentation();
				remark += "\n\n";
			}
		}
		if (this.enableReverseParsing) {
			this.storeMembraneModel(content);
		}
		final String[] allprots = this.model.getAllProteinTypes();
		final String[] allEC = this.model.getAllLipidTypesInMembraneExtra();
		final String[] allIC = this.model.getAllLipidTypesInMembraneIntra();
		final Hashtable<String, String> hash = new Hashtable<String, String>();
		for (int i = 0; i < allEC.length; i++) {
			hash.put(allEC[i], "");
		}
		for (int i = 0; i < allIC.length; i++) {
			hash.put(allIC[i], "");
		}
		final Enumeration<String> e = hash.keys();
		remark += "INCLUDED AS PROTEINS: \n \n";
		if (allprots.length > 0) {
			remark += "CODE                   HEADER                   COUNT \n";
			for (int i = 0; i < allprots.length; i++) {
				String line = "";
				final Protein_3D[] prots = this.model.getProteinsByPDBCode(allprots[i]);
				if (prots.length == 0) {
					continue;
				}
				final PdbAttributes info = new PdbReader_3D().readAttributes(DataPool.getRawPdbData(prots[0]
								.getPDBFile().getAbsolutePath()));
				if (info.getCode() != null) {
					line += Cm2_Toolkit.extendRight(info.getCode(), 4);
				} else {
					line += " ?? ";
				}
				line += "  ";
				if (info.getHeader() != null) {
					line += Cm2_Toolkit.extendRight(info.getHeader(), 40);
				} else {
					line += "                   ??                   ";
				}
				line += "  ";
				final int count = prots.length;

				if (count < 100000) {
					line += Cm2_Toolkit.extendLeft("" + count, 5);
				} else {
					line += "99999+";
				}
				remark += line + "\n";
				if (this.adoptTitleSections) {
					this.adoptTitleSection(prots[0], content);
				}
			}
		} else {
			remark += "None";
		}
		remark += " \n \n";
		remark += "INCLUDED AS LIPIDS: \n \n";
		if (e.hasMoreElements()) {
			remark += "CODE                   HEADER                   COUNT \n";
			while (e.hasMoreElements()) {
				String line = "";
				final String key = e.nextElement();
				Lipid_3D[] lips = this.model.getLipidsByPDBCodeExtra(key);
				final Lipid_3D[] lips2 = this.model.getLipidsByPDBCodeIntra(key);
				final int count = lips.length + lips2.length;
				if (lips.length == 0) {
					lips = lips2;
				}
				if (lips.length == 0) {
					continue;
				}
				final PdbAttributes info = new PdbReader_3D().readAttributes(DataPool.getRawPdbData(lips[0]
								.getPDBFile().getAbsolutePath()));
				if (info.getCode() != null) {
					line += Cm2_Toolkit.extendRight(info.getCode(), 4);
				} else {
					line += " ?? ";
				}
				line += "  ";
				if (info.getHeader() != null) {
					line += Cm2_Toolkit.extendRight(info.getHeader(), 40);
				} else {
					line += "                   ??                   ";
				}
				line += "  ";
				if (count < 100000) {
					line += Cm2_Toolkit.extendLeft("" + count, 5);
				} else {
					line += "99999+";
				}
				remark += line + "\n";
				if (this.adoptTitleSections) {
					this.adoptTitleSection(lips[0], content);
				}
			}
		} else {
			remark += "None";
		}
		remark += " \n \n";
		return remark;
	}
	
	
	
	/**
	 * Copies part of the title section from a molecule's file.
	 * 
	 * @param mol
	 */
	private void adoptTitleSection(final Molecule_3D mol, Content content) {
		String top = "";
		if (mol instanceof Protein_3D)
			top = "REMARK MEMBER " + mol.getPDBcode() + " ";
		else
			top = "REMARK MEMBER " + mol.getPDBcode() + " ";
		final PdbReader_3D reader = new PdbReader_3D();
		try {
			String[] s = reader.readSection(mol.getPDBFile(), "TITLE ");
			for (int i = 0; i < s.length; i++) {
				content.addAdopt(top + s[i]);
			}
			s = reader.readSection(mol.getPDBFile(), "COMPND");
			for (int i = 0; i < s.length; i++) {
				content.addAdopt(top + s[i]);
			}
			s = reader.readSection(mol.getPDBFile(), "SOURCE");
			for (int i = 0; i < s.length; i++) {
				content.addAdopt(top + s[i]);
			}
			s = reader.readSection(mol.getPDBFile(), "KEYWDS");
			for (int i = 0; i < s.length; i++) {
				content.addAdopt(top + s[i]);
			}
			s = reader.readSection(mol.getPDBFile(), "EXPDTA");
			for (int i = 0; i < s.length; i++) {
				content.addAdopt(top + s[i]);
			}
			s = reader.readSection(mol.getPDBFile(), "AUTHOR");
			for (int i = 0; i < s.length; i++) {
				content.addAdopt(top + s[i]);
			}
			s = reader.readSection(mol.getPDBFile(), "JRNL  ");
			for (int i = 0; i < s.length; i++) {
				content.addAdopt(top + s[i]);
			}
			s = reader.readSection(mol.getPDBFile(), "REMARK");
			for (int i = 0; i < s.length; i++) {
				content.addAdopt(top + s[i]);
			}
			s = reader.readSection(mol.getPDBFile(), "FORMUL");
			for (int i = 0; i < s.length; i++) {
				content.addAdopt(top + s[i]);
			}
		} catch (final IOException e) {
			// handle exception here
			return;
		}
	}
	
	
	/**
	 * 
	 * @param content
	 */
	private void storeMembraneModel(Content content) {
		final File temp = new File(FileIOHandler.home + "tmp.xml");
		this.model.translateAllMolecules(-DataPool.getCurrentLayerHeight(), false, false);
		try {
			new MembraneXmlWriter(DataPool.workspace).writeMembrane(temp, this.model);
			final BufferedReader br = new BufferedReader(new FileReader(temp));
			while (true) {
				final String s = br.readLine();
				if (s == null) {
					break;
				}
				content.addXmlContent(s);
			}
			br.close();
		} catch (final IOException e) {
			e.printStackTrace();
		}
		this.model.translateAllMolecules(DataPool.getCurrentLayerHeight(), false, false);
		temp.deleteOnExit();
		temp.delete();
	}

	
	/**
	 * This method creates lines representing the molecule in a pdb file.
	 * 
	 * @param mol - the molecule to represent
	 * @return Returns an array of lines representing the molecule in a pdb file.
	 * @throws IOException
	 */
	private Line[] createLines(final Molecule_3D mol) throws IOException {
		// setting up some variables
		Vector<Line> linesTemp = new Vector<Line>();
		Vector<Particle> alreadyIn = new Vector<Particle>();
		final String[][] readin = new PdbReader_3D().readModel(DataPool.getRawPdbData(
				mol.getPDBFile().getAbsolutePath()),mol.getModelSet());
		for (int i = 0; i < (DataPool.PPESettings.saveOPMDummyLayersInPDB ? 2 : 1); i++) {
			int c = 0;
			final Atom_3D[] atoms = (i == 0) ? mol.allAtomsSorted : mol.opmAtoms;
			for (int j = 0; j < readin[i].length; j++) {
				Atom_3D atm;
				try {
					new Atom_3D(readin[i][j]);
					atm = atoms[c];
					c++;
					Particle p = mol.getHostParticle(atm);
					if (p != null) {
						if (alreadyIn.contains(p)) {
							continue;
						}
						alreadyIn.addElement(p);
						final Line line = new Line(p, readin[i][j], mol.getDomainID());
						line.name = p.name;
						line.comparator=readin.length+j;
						linesTemp.addElement(line);
					} else {
						final Line line = new Line(atm, readin[i][j], mol.getDomainID());
						linesTemp.addElement(line);
						line.comparator=j;
					}
				} catch (final Exception e) {
					final Line line = new Line(null, readin[i][j], mol.getDomainID());
					linesTemp.addElement(line);
					line.comparator=j;
				}
			}
		}
		Line prevL = null;
		for (int i = 0; i < linesTemp.size(); i++) {
			Atom_3D a = linesTemp.get(i).atomOfMol;
			if (a == null) {
				continue;
			}
			if (a instanceof Particle) {
				Line ref = null;
				if (prevL != null) {
					ref = prevL;
				} else {
					for (int k = i + 1; k < linesTemp.size(); k++) {
						if (linesTemp.get(k).atomOfMol != null
								&& (linesTemp.get(k).atomOfMol instanceof Particle) == false) {
							ref = linesTemp.get(k);
							break;
						}
					}
				}
				if (ref == null) {
					continue;
				}
				Line line = linesTemp.get(i);
				line.recordName = ref.recordName;
				line.occupancy = ref.occupancy;
				line.tempFactor = ref.tempFactor;
				line.charge = ref.charge;
				line.residueName = ref.residueName;
				line.altLoc = ref.altLoc;
				line.chainId = ref.chainId;
				line.resSeq = ref.resSeq;
				line.iCode = ref.iCode;
				line.segID = ref.segID;
				String element = Cm2_Toolkit.convertAtomNameToElementName(line.name);
				if (element != null) {
					line.element = element;
				} else {
					line.element = "";
				}
			} else {
				prevL = linesTemp.get(i);
			}
		}
		// if domainIDs should be added to the ResName, check at which position (1, 2, or 3)
		if (DataPool.LimitationSettings.domainID2ResNameAtPosition > 0) {
			for (int i = 0; i < linesTemp.size(); i++) {
				if (linesTemp.get(i).recordName.isEmpty())
					continue;
				linesTemp.get(i).residueName = linesTemp.get(i).residueName.substring(0, DataPool.LimitationSettings.domainID2ResNameAtPosition-1)
												+ mol.getDomainID()
												+ linesTemp.get(i).residueName.substring(DataPool.LimitationSettings.domainID2ResNameAtPosition, 3);
			}
		}
		// if layerIDs should be added to the ResName, check at which position (1, 2, or 3)
		if (DataPool.LimitationSettings.layerID2ResNameAtPosition > 0) {
			for (int i = 0; i < linesTemp.size(); i++) {
				if (linesTemp.get(i).recordName.isEmpty())
					continue;
				linesTemp.get(i).residueName = linesTemp.get(i).residueName.substring(0, DataPool.LimitationSettings.layerID2ResNameAtPosition-1)
												+ mol.getLayerID()
												+ linesTemp.get(i).residueName.substring(DataPool.LimitationSettings.layerID2ResNameAtPosition, 3);
			}
		}
		return linesTemp.toArray(new Line[0]);
	}
	
	
	/**
	 * Creates a string of each number, ensuring that it has at the proper
	 * number of zeros behind the dot
	 * 
	 * @param x - the given number to convert into a String
	 * @return Returns the converted Number as String.
	 */
	private String createNumberString(final float x) {
		String X = "" + x;
		if(X.contains(".") == false) {
			X += ".000";
		} else {
			final int indexof = X.indexOf(".");
			if(indexof == X.length() - 1) {
				X += "000";
			} else { 
				if(indexof == X.length() - 2) {
					X += "00";
				} else {
					if(indexof == X.length() - 3) {
						X += "0";
					} else {
						if(indexof < X.length() - 4) {
							X = X.substring(0, indexof + 4);
						}
					}
				}
			}
		}
		return X;
	}
	
	
	/**
	 * Converts a passed number into a right-side-adjusted string of ste
	 * specified length If the string is longer than that, it will be cut of,
	 * will all characters beeing a '9'
	 * 
	 * @param number
	 * @param length
	 * @return
	 */
	private String getNumString(final int number, final int length) {
		String s = "" + number;
		if (s.length() > length) {
			s = "";
			for (int i = 0; i < length; i++) {
				s += "9";
			}
		} else
			s = Cm2_Toolkit.extendLeft("" + s, length);
		return s;
	}
	
	
	public boolean coordinatesWouldExceedValidNumberRangeCurrently(MembraneModel model) {

		float maxx = -1000000, maxy = -1000000, maxz = -1000000, minx = 1000000, miny = 1000000, minz = 1000000;

		final Molecule_3D[] mols = model.getAllMoleculesInMembrane();

		for (int i = 0; i < mols.length; i++) {

			Atom_3D[] atoms = mols[i].getAtoms(true, false);

			for (int j = 0; j < atoms.length; j++) {

				final Point3f p = atoms[j].position;
				minx = Math.min(minx, p.x);
				miny = Math.min(miny, p.y);
				minz = Math.min(minz, p.z);

				maxx = Math.max(maxx, p.x);
				maxy = Math.max(maxy, p.y);
				maxz = Math.max(maxz, p.z);

			}
		}

		float subX = 0;
		float subZ = 0;
		float subY = 0;
		if (this.tHints.shiftXZ == 2) {
			subX += model.getSize().width / 2;
			subZ += model.getSize().height / 2;

		} else if (this.tHints.shiftXZ == 1) {
			subX = minx + ((maxx - minx) / 2);
			subZ = minz + ((maxz - minz) / 2);
		}

		if (this.tHints.shiftY == 2) {
			subY = miny;
		} else if (this.tHints.shiftY == 1) {
			subY = miny + ((maxy - miny) / 2);
		}

		Vector<Atom_3D> finalAtoms = new Vector<Atom_3D>();
		for (int i = 0; i < mols.length; i++) {
			Point3f p = new Point3f(mols[i].getPosition());
			p.sub(new Point3f(subX, subY, subZ));
			mols[i].setPosition(p);

			Atom_3D[] atoms = mols[i].getAtoms(true, true);

			for (int j = 0; j < atoms.length; j++) {
				finalAtoms.addElement(atoms[j]);
			}

			if (! DataPool.PPESettings.saveOPMDummyLayersInPDB)
				continue;

			for (int j = 0; j < mols[i].opmAtoms.length; j++) {
				Atom_3D a = mols[i].opmAtoms[j].copy();
				a.position = Cm2_Toolkit.transformToNewPoint(mols[i]
						.getRotationMatrix(), a.position);
				a.position.sub(new Point3f(subX, subY, subZ));
				finalAtoms.addElement(a);
			}

		}

		for (int i = 0; i < finalAtoms.size(); i++) {
			Atom_3D a = finalAtoms.get(i);
			if ((a.position.x < -999.999f || a.position.x > 9999.999f)
					|| (a.position.y < -999.999f || a.position.y > 9999.999f)
					|| (a.position.z < -999.999f || a.position.z > 9999.999f))
				return true;
		}

		return false;
	}
	
	
//	*****************************************************************************
//	*  INNER CLASS LINE() - FOR LINE REPRESENTATION       				  		*
//	*****************************************************************************
	/**
	 * A class representing an atom line from a pdb file
	 * 
	 * @author Tim Dingersen <i>dackelnose@gmx.net</i> <br>
	 * 		   Christian Gamroth <i>c.gamroth@hotmail.de</i>
	 */
	class Line implements Comparable{

		// global variables
		public String recordName = "";
		public Point3f position = new Point3f();
		public String name = "";
		public String altLoc = "";
		public String residueName = "";
		public String chainId = "";
		public String resSeq = "";
		public String iCode = "";
		public String occupancy = "";
		public String tempFactor = "";
		public String segID = "";
		public String element = "";
		public String charge = "";
		public int domainID = 0;
		public Atom_3D atomOfMol;
		private int comparator = -1;

		/**
		 * Constructor of class Line.
		 * 
		 * @param atomOfMolecule - the atom of the molecule
		 * @param line - a string representation of a pdb file
		 * @param domainID - the identification number of the domain
		 */
		public Line(final Atom_3D atomOfMolecule, final String line, final int domainID) {
			if (line == null) return;
			// parameters getting global
			this.domainID = domainID;
			this.atomOfMol = atomOfMolecule;
			try {
				this.recordName = line.substring(0, 7).trim().toUpperCase();
			} catch (IndexOutOfBoundsException e) {
				// handle the exception here
			}
			if (this.atomOfMol == null) {
				try {
					final String sx = line.substring(31, 38);
					final String sy = line.substring(39, 46);
					final String sz = line.substring(47, 54);
					final float x = Float.parseFloat(sx.trim());
					final float y = Float.parseFloat(sy.trim());
					final float z = Float.parseFloat(sz.trim());
					this.position = new Point3f(x, y, z);
				} catch (IndexOutOfBoundsException e) {
					// handle the exception here
				} catch (NumberFormatException e) {
					// handle the exception here
				} catch (Exception e) {
					e.printStackTrace();
				}
			} else {
				this.position = new Point3f(atomOfMolecule.position);
			}
			try {
				this.name = line.substring(11, 16);
				this.altLoc = line.substring(16, 17).trim();
				this.residueName = line.substring(17, 20).trim();
				this.chainId = line.substring(21, 22).trim();
				this.resSeq = line.substring(22, 26).trim();
				this.iCode = line.substring(26, 27).trim();
				this.occupancy = line.substring(55, 60).trim();
				this.tempFactor = line.substring(61, 66).trim();
				this.segID = line.substring(73, 76).trim();
				if(this.atomOfMol!=null) {
					this.element = this.atomOfMol.element;
				}
				this.element = line.substring(76, 78).trim();
				this.charge = line.substring(78, 80).trim();
			} catch (IndexOutOfBoundsException e) {
				// line was shorter than 80 chars, 
				// this is normally anyway valid PDB code
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		/**
		 * What is going on here(?). Fill in the text if you know it.
		 * 
		 * @param arg0 - an object to compare to
		 */
		public int compareTo(Object arg0) {
			if(arg0 instanceof Line == false) {
				return 0;
			} else {
				int c = ((Line) arg0).comparator;
				if(this.comparator < c) {
					return -1;
				} else {
					if(this.comparator > c) {
						return 1;
					} else {
						System.out.println("Comparable: Same Comparator");
						System.out.println(this.name + "  " + ((Line) arg0).name);
						return 0;
					}
				}
			}
		}	
	}
//	*****************************************************************************
//	*  END OF INNER CLASS LINE()					        					*
//	*****************************************************************************
	
	
//	*****************************************************************************
//	*  INNER CLASS CONTENT() - HOLDING INFORMATION FOR WRITING INTO OUTPUT FILE *
//	*****************************************************************************
	/**
	 * 
	 */
	class Content {
		
		public Hashtable<String, String> ids;
		public HashMap<Integer, Integer> resSeqMapping;
		public Vector<String> conect;
		public Vector<String> seqres;
		public Vector<String> helix;
		public Vector<String> sheet;
		public Vector<String> turn;
		public Vector<String> ssbond;
		public Vector<String> site;
		public Vector<String> dbref;
		public Vector<String> seqadv;
		public Vector<String> modres;
		public Vector<String> het;
		public Vector<String> link;
		public Vector<String> cispep;
		public Vector<String> compnd;
		public Vector<String> positionList;
		public Vector<String> moleculeLines;
		public Vector<String> xmlContent;
		public Vector<String> adopts;
		public int currentPos, listPos, listSize;
		
		/**
		 * Constructor of class Content
		 * 
		 */
		public Content() {
			this.ids = new Hashtable<String, String>();
			this.resSeqMapping = new HashMap<Integer, Integer>();
			this.conect = new Vector<String>();
			this.seqres = new Vector<String>();
			this.helix = new Vector<String>();
			this.sheet = new Vector<String>();
			this.turn = new Vector<String>();
			this.ssbond = new Vector<String>();
			this.site = new Vector<String>();
			this.dbref = new Vector<String>();
			this.seqadv = new Vector<String>();
			this.modres = new Vector<String>();
			this.het = new Vector<String>();
			this.link = new Vector<String>();
			this.cispep = new Vector<String>();
			this.compnd = new Vector<String>();
			this.positionList = new Vector<String>();
			this.moleculeLines = new Vector<String>();
			this.xmlContent = new Vector<String>();
			this.adopts = new Vector<String>();
			this.currentPos = 0;
			this.listPos = 1;
			this.listSize = 0;
		}
		
		/**
		 * Adding a connection to the global vector conect.
		 * 
		 * @param mol - the molecule which connections are to add
		 */
		public void addConect(Molecule_3D mol) {
			for(int i = 0; i < mol.atoms.length; i++) {
				int[] sorted = new int[mol.atoms[i].currentNeighbors.size()];
				for (int j = 0; j < sorted.length; j++) {
					sorted[j] = mol.atoms[i].currentNeighbors.get(j).indexForPDBWriter.intValue();
				}
				Arrays.sort(sorted);
				Vector<Integer> sortedSerials = new Vector<Integer>();
				for (int j = 0; j < sorted.length; j++) {
					sortedSerials.addElement(sorted[j]);
				}
				Enumeration<Integer> en = sortedSerials.elements();
				while (en.hasMoreElements()) {
					String string = "";
					string = "CONECT";
					int refInd = mol.atoms[i].indexForPDBWriter.intValue();
					string += Cm2_Toolkit.extendLeft("" + refInd, 5);
					while (true) {
						if (!en.hasMoreElements()) {
							this.conect.addElement("" + string);
							break;
						} else {
							int serial = en.nextElement().intValue();
							string += Cm2_Toolkit.extendLeft("" + serial, 5);
						}
					}
				}
			}
		}
		
		/**
		 * 
		 * 
		 * @param mol
		 * @param moleculeLines
		 * @throws IOException
		 */
		public void setStructure(final Molecule_3D mol, Line[] moleculeLines) throws IOException {
			// clearing older entries of the hashtable IDs
			this.ids.clear();
			final Vector<String> chars2use = new Vector<String>();
			chars2use.setSize(Cm2_Toolkit.CHARS.length);
			for(int i = 0; i < mol.getChainIDs().length; i++) {
				final char s = mol.getChainIDs()[i].charAt(0);
				final int pos = Cm2_Toolkit.getCharacterPosOf(s);
				chars2use.set(pos, mol.getChainIDs()[i]);
			}
			// filling the hashtable IDs
			int c = 0;
			for (int i = 0; i < chars2use.size(); i++) {
				if (chars2use.get(i) == null) {
					continue;
				}
				this.ids.put(chars2use.get(i), "" + this.getCHARAt(c));
				c++;
			}
			// updating chainID in moleculeLines
			for(Line line : moleculeLines) {
				if(mol.getChainCount() < 1) {
					line.chainId = "";
					continue;
				}
				if(line.chainId.length() < 1) {
					line.chainId = "";
					continue;
				}
				line.chainId = this.getCharFor(line.chainId);
			}
			// filling vector seqres
			String[] input = new PdbReader_3D().readSection(mol.getPDBFile(), "SEQRES");
			for (int i = 0; i < input.length; i++) {
				String current = input[i];
				try {
					final String orig = "" + current.charAt(11);
					current = current.substring(0, 11) + this.getCharFor(orig) + current.substring(12);
					this.seqres.addElement(current);
				} catch (final Exception e) {
					// handle exception here
				}
			}
			// filling vector helix
			input = new PdbReader_3D().readSection(mol.getPDBFile(), "HELIX");
			for (int i = 0; i < input.length; i++) {
				String current = input[i];
				try {
					current = current.substring(0, 6) +
						Cm2_Toolkit.extendLeft("" + (this.helix.size() + 1), 4) + current.substring(10);
					final String orig = "" + current.charAt(19);
					current = current.substring(0, 19) + this.getCharFor(orig) + current.substring(20);
					current = current.substring(0, 31) + this.getCharFor(orig) + current.substring(32);
					
					
					// update residue sequence numbers if 'assignResSeq' is activated
					// ===================================================================
					// TODO: PDBWriter residue sequence update 04_2012					  |
					// ===================================================================
					if(DataPool.LimitationSettings.assignResSeqNumbers) {
						if(mol instanceof Protein_3D) {
							// molecule is a protein
							final int initSeqNumReplacement = this.resSeqMapping.get(new Integer(current.substring(21, 25).trim()));
							final int endSeqNumReplacement = this.resSeqMapping.get(new Integer(current.substring(33, 37).trim()));
							current = current.substring(0, 21) +
										Cm2_Toolkit.extendLeft(String.valueOf(initSeqNumReplacement), 4) +
											current.substring(25, 33) +
												Cm2_Toolkit.extendLeft(String.valueOf(endSeqNumReplacement), 4) +
													current.substring(37);
						}
					}
					
					
					this.helix.addElement(current);
				} catch (final Exception e) {
					e.printStackTrace();
					// handle exception here
				}

			}
			// filling vector sheet
			input = new PdbReader_3D().readSection(mol.getPDBFile(), "SHEET");
			for (int i = 0; i < input.length; i++) {
				String current = input[i];
				try {
					final String s = "" + current.charAt(21);
					current = current.substring(0, 21) + this.getCharFor(s) + current.substring(22);
				} catch (final Exception e) {
					// handle exception here
				}
				try {
					final String s = "" + current.charAt(32);
					current = current.substring(0, 32) + this.getCharFor(s) + current.substring(33);
				} catch (final Exception e) {
					// handle exception here
				}
				try {
					final String s = "" + current.charAt(49);
					current = current.substring(0, 49) + this.getCharFor(s) + current.substring(50);
				} catch (final Exception e) {
					// handle exception here
				}
				try {
					final String s = "" + current.charAt(64);
					current = current.substring(0, 64) + this.getCharFor(s) + current.substring(65);
				} catch (final Exception e) {
					// handle exception here
				}
				
				
				// update residue sequence numbers if 'assignResSeq' is activated
				// ===================================================================
				// TODO: PDBWriter residue sequence update 04_2012					  |
				// ===================================================================
				try {
					if(DataPool.LimitationSettings.assignResSeqNumbers) {
						if(mol instanceof Protein_3D) {
							// molecule is a protein
							final int sense = new Integer(current.substring(38, 40).trim());
							final int initSeqNumReplacement = this.resSeqMapping.get(new Integer(current.substring(22, 26).trim()));
							final int endSeqNumReplacement = this.resSeqMapping.get(new Integer(current.substring(33, 37).trim()));
							final int currentSeqNumReplacement = (sense != 0) ? this.resSeqMapping.get(new Integer(current.substring(50, 54).trim())) : 0;
							final int prevSeqNumReplacement = (sense != 0) ? this.resSeqMapping.get(new Integer(current.substring(65, 69).trim())) : 0;
							current = current.substring(0, 22) +
										Cm2_Toolkit.extendLeft(String.valueOf(initSeqNumReplacement), 4) +
											current.substring(26, 33) +
												Cm2_Toolkit.extendLeft(String.valueOf(endSeqNumReplacement), 4) +
													current.substring(37);
							if(sense != 0) {
								// if sense equals '0' this is the starting strand without current and previous
								// sequence number
								current = current.substring(0, 50) +
											Cm2_Toolkit.extendLeft(String.valueOf(currentSeqNumReplacement), 4) +
												current.substring(54, 65) +
													Cm2_Toolkit.extendLeft(String.valueOf(prevSeqNumReplacement), 4) +
														current.substring(69);
							}
						}
					}
				} catch(Exception e) {
					// handle exception here
					e.printStackTrace();
				}
				
				
				this.sheet.addElement(current);
			}
			// filling vector turn
			input = new PdbReader_3D().readSection(mol.getPDBFile(), "TURN");
			for (int i = 0; i < input.length; i++) {
				String current = input[i];
				try {
					current = current.substring(0, 6) +
						Cm2_Toolkit.extendLeft("" + (this.turn.size() + 1), 4) + current.substring(10);
					String orig = "" + current.charAt(19);
					current = current.substring(0, 19) + this.getCharFor(orig) + current.substring(20);
					orig = "" + current.charAt(30);
					current = current.substring(0, 30) + this.getCharFor(orig) + current.substring(31);
					this.turn.addElement(current);
				} catch (final Exception e) {
					// handle exception here
				}

			}
			// filling vector ssbond
			input = new PdbReader_3D().readSection(mol.getPDBFile(), "SSBOND");
			for (int i = 0; i < input.length; i++) {
				String current = input[i];
				try {
					final String s = "" + current.charAt(15);
					current = current.substring(0, 15) + this.getCharFor(s) + current.substring(16);
					current = current.substring(0, 6) +
						Cm2_Toolkit.extendLeft("" + (this.ssbond.size() + 1), 4) + current.substring(10);
				} catch (final Exception e) {
					// handle exception here
					continue;
				}
				try {
					final String s = "" + current.charAt(29);
					current = current.substring(0, 29) + this.getCharFor(s) + current.substring(30);
				} catch (final Exception e) {
					// handle exception here
					continue;
				}
				this.ssbond.addElement(current);
			}
			// filling vector site
			input = new PdbReader_3D().readSection(mol.getPDBFile(), "SITE");
			for (int i = 0; i < input.length; i++) {
				String current = input[i];
				try {
					final String s = "" + current.charAt(22);
					current = current.substring(0, 22) + this.getCharFor(s) + current.substring(23);
				} catch (final Exception e) {
					// handle exception here
				}
				try {
					final String s = "" + current.charAt(33);
					current = current.substring(0, 33) + this.getCharFor(s) + current.substring(34);
				} catch (final Exception e) {
					// handle exception here
				}
				try {
					final String s = "" + current.charAt(44);
					current = current.substring(0, 44) + this.getCharFor(s) + current.substring(45);
				} catch (final Exception e) {
					// handle exception here
				}
				try {
					final String s = "" + current.charAt(55);
					current = current.substring(0, 55) + this.getCharFor(s) + current.substring(56);
				} catch (final Exception e) {
					// handle exception here
				}
				this.site.addElement(current);
			}
			// filling vector dbref
			input = new PdbReader_3D().readSection(mol.getPDBFile(), "DBREF");
			for (int i = 0; i < input.length; i++) {
				String current = input[i];
				try {
					final String orig = "" + current.charAt(12);
					current = current.substring(0, 12) + this.getCharFor(orig) + current.substring(13);
					this.dbref.addElement(current);
				} catch (final Exception e) {
					// handle exception here
				}
			}
			// filling vector seqadv
			input = new PdbReader_3D().readSection(mol.getPDBFile(), "SEQADV");
			for (int i = 0; i < input.length; i++) {
				String current = input[i];
				try {
					final String orig = "" + current.charAt(16);
					current = current.substring(0, 16) + this.getCharFor(orig) + current.substring(17);
					this.seqadv.addElement(current);
				} catch (final Exception e) {
					// handle exception here
				}
			}
			// filling vector modres
			input = new PdbReader_3D().readSection(mol.getPDBFile(), "MODRES");
			for (int i = 0; i < input.length; i++) {
				String current = input[i];
				try {
					final String orig = "" + current.charAt(16);
					current = current.substring(0, 16) + this.getCharFor(orig) + current.substring(17);
					this.modres.addElement(current);
				} catch (final Exception e) {
					// handle exception here
				}
			}
			// filling vector het
			input = new PdbReader_3D().readSection(mol.getPDBFile(), "HET");
			for (int i = 0; i < input.length; i++) {
				String current = input[i];
				try {
					final String orig = "" + current.charAt(12);
					current = current.substring(0, 12) + this.getCharFor(orig) + current.substring(13);
					this.het.addElement(current);
				} catch (final Exception e) {
					// handle exception here
				}
			}
			// filling vector link
			input = new PdbReader_3D().readSection(mol.getPDBFile(), "LINK");
			for (int i = 0; i < input.length; i++) {
				String current = input[i];
				try {
					String orig = "" + current.charAt(21);
					current = current.substring(0, 21) + this.getCharFor(orig) + current.substring(22);
					orig = "" + current.charAt(51);
					current = current.substring(0, 51) + this.getCharFor(orig) + current.substring(52);
					this.link.addElement(current);
				} catch (final Exception e) {
					// handle exception here
				}
			}
			// filling vector cispep
			input = new PdbReader_3D().readSection(mol.getPDBFile(), "CISPEP");
			for (int i = 0; i < input.length; i++) {
				String current = input[i];
				try {
					current = current.substring(0, 6) +
						Cm2_Toolkit.extendLeft("" + (this.cispep.size() + 1), 4) + current.substring(10);
					String orig = "" + current.charAt(15);
					current = current.substring(0, 15) + this.getCharFor(orig) + current.substring(16);
					orig = "" + current.charAt(29);
					current = current.substring(0, 29) + this.getCharFor(orig) + current.substring(30);
					this.cispep.addElement(current);
				} catch (final Exception e) {
					// handle exception here
				}
			}
			// filling vector compnd
			if (mol.getChainCount() > 0) {
				String n = "    ";
				if (this.compnd.size() > 1)
					n = Cm2_Toolkit.extendLeft("" + this.compnd.size(), 4) + " ";
				this.compnd.add("COMPND" + n + "MOL_ID:"+
						Cm2_Toolkit.extendLeft("" + ((this.compnd.size() / 3) + 1), 2) + ";");
				n = Cm2_Toolkit.extendLeft("" + this.compnd.size(), 4) + " ";
				this.compnd.add("COMPND" + n + "MOLECULE: " + mol.getHeader().trim() + ";");
				n = Cm2_Toolkit.extendLeft("" + this.compnd.size(), 4) + " ";
				String chains = "";
				final String[] ids = mol.getChainIDs();
				for (int i = 0; i < ids.length; i++) {
					final char ch = Cm2_Toolkit.CHARS[this.currentPos + i];
					chains += ch;
					if (i < ids.length - 1) {
						chains += ", ";
					}
				}
				this.compnd.add("COMPND" + n + "CHAIN: " + chains + ";");
			}
			this.currentPos += mol.getChainCount();
		}
		
		/**
		 * 
		 * @param mol
		 */
		public void finishMolecule(final Molecule_3D mol) {
			if (PDBWriter.this.listPositions) {
				if (PDBWriter.this.useSerials == PDBWriter.this.NONE && mol instanceof Protein_3D) {
					return;
				}	
				else if (PDBWriter.this.useSerials > PDBWriter.this.BOTH && mol instanceof Lipid_3D) {
					return;
				}
				String out = mol instanceof Protein_3D ? "PROTEIN" : "LIPID  ";
				out += "  ";
				out += Cm2_Toolkit.extendRight(mol.getPDBcode(), 4);
				out += "   START:  " + Cm2_Toolkit.extendLeft("" + this.listPos, 5);
				out += "   END:  " + Cm2_Toolkit.extendLeft("" + this.listSize, 5);
				out += "   LENGTH:  " + Cm2_Toolkit.extendLeft("" + (this.listSize - this.listPos - 1), 5);
				this.positionList.addElement(out);
				this.listPos = this.listSize + 1;
			}
		}
		
		/**
		 * 
		 * @param moleculeLine
		 */
		public void addMoleculeLine(String moleculeLine) {
			// adding molecule line to vector moleculeLines
			this.moleculeLines.addElement(moleculeLine);
		}
		
		/**
		 * 
		 * @param content
		 */
		public void addXmlContent(String content) {
			// adding xml content to vector xmlContent
			this.xmlContent.addElement(content);
		}
		
		/**
		 * 
		 * @param string
		 */
		public void addAdopt(String string) {
			// adding adopted title from the molecule to the vector adopts
			this.adopts.addElement(string);
		}
		
		/**
		 * 
		 * @param moleculeLines
		 */
		public void removeChainsIDs(Line[] moleculeLines) {
			for(Line line : moleculeLines) {
				line.chainId = "";
			}
		}
		
		/**
		 * 
		 * @param i
		 * @return
		 */
		private String getCHARAt(final int i) {
			try {
				return "" + Cm2_Toolkit.CHARS[this.currentPos + i];
			} catch (final Exception e) {
				// handle exception here
				return " ";
			}
		}
		
		/**
		 * 
		 * @param s
		 * @return
		 */
		private String getCharFor(final String s) {
			try {
				final String ret = this.ids.get(s);
				return ret != null ? ret : " ";
			} catch (final Exception e) {
				// handle exception here
				return " ";
			}
		}
		
	}
//	*****************************************************************************
//	*  END OF INNER CLASS CONTENT()					        					*
//	*****************************************************************************
	
	
}




// !THIS IS THE ORIGINAL PDBWriter ONCE WRITTEN BY TIM DINGERSEN!
// The new version is still based on Tim's PDBWriter, but should
// be a little bit more efficient and more structured. Anyway, 
// this old writer was proven to be very stable.
//
// The method
//  coordinatesWouldExceedValidNumberRangeCurrently()
// used formerly in the Engine is contained in this older source code.

//package org.cellmicrocosmos.cm2.io;
//
//import java.io.BufferedReader;
//import java.io.BufferedWriter;
//import java.io.File;
//import java.io.FileReader;
//import java.io.FileWriter;
//import java.io.IOException;
//import java.util.Arrays;
//import java.util.Enumeration;
//import java.util.Hashtable;
//import java.util.Vector;
//
//import javax.vecmath.Matrix3f;
//import javax.vecmath.Point3f;
//
//import org.cellmicrocosmos.cm2.constants.CM2_CONSTANTS;
//import org.cellmicrocosmos.cm2.dataobjects.Atom_3D;
//import org.cellmicrocosmos.cm2.dataobjects.Lipid_3D;
//import org.cellmicrocosmos.cm2.dataobjects.MembraneModel;
//import org.cellmicrocosmos.cm2.dataobjects.Molecule_3D;
//import org.cellmicrocosmos.cm2.dataobjects.Particle;
//import org.cellmicrocosmos.cm2.dataobjects.Protein_3D;
//import org.cellmicrocosmos.cm2.engine.DataPool;
//import org.cellmicrocosmos.cm2.gui.dialogs.StatusDialog;
//import org.cellmicrocosmos.cm2.io.PdbReader_3D.PdbAttributes;
//import org.cellmicrocosmos.cm2.membranepackingalgorithm.CoordinateTransformationHints;
//import org.cellmicrocosmos.cm2.misc.Cm2_Toolkit;
//
//
///**
// * Does what this programm is made for: Setting up all relevant data from the
// * membrane model, computing it into a proper pdb syntax and writing it into a
// * file.
// * 
// * @author Tim Dingersen dackelnose@gmx.net, Bjoern Sommer
// *         bjoern@CELLmicrocosmos.org
// * 
// */
//public class PDBWriter {
//
//	private final int BOTH = 0;
//
//	private final int PROTSONLY = 1;
//
//	private final int NONE = 2;
//
//	private final StatusDialog dialog;
//
////	private final boolean attachElementCharAt77 = true;
//
//	private final boolean listPositions = true;
//
//	private final File outputFile;
//
//	private BufferedWriter BW;
//
//	private Content cont;
//
//	private int useSerials = this.BOTH;
//
//	private int useChains = this.BOTH;
//
//	private boolean adoptTitleSections = false;
//
//	private final Vector<String> adopts = new Vector<String>();
//
//	private int remcount = 0;
//
//	private int atomhetatmcount = 0;
//
//	private int tercount = 0;
//
//	MembraneModel model;
//
//	private boolean enableReverseParsing = false;
//	
//	private final Vector<String> xmlContents = new Vector<String>();
//
//	private CoordinateTransformationHints tHints = null;
//
//	/**
//	 * Writes the membrane
//	 * 
//	 * @param domainID
//	 * 
//	 * @throws IOException
//	 */
//	public boolean writeMembrane(final int domainID, final int layerID,
//			CoordinateTransformationHints tHints) throws IOException {
//
//		this.tHints = tHints;
//
//		this.model = PDBWriter.getAlteredModel(domainID, layerID);
//
//		// this.model.sortMolecules();
//
//		this.BW = new BufferedWriter(new FileWriter(this.outputFile));
//
//		final int totalCount = this.model.countAtoms(-1, -1, -1,
//				DataPool.PPESettings.saveOPMDummyLayersInPDB);
//
//		float step = 30f / (float) totalCount;
//		float counter = 0f;
//
//		float maxx = -1000000, maxy = -1000000, maxz = -1000000, minx = 1000000, miny = 1000000, minz = 1000000;
//		
//		Vector<String[]> topologyVec = new Vector<String[]>();
//
//		final boolean resSeqCount = DataPool.LimitationSettings.assignResSeqNumbers;
//		final boolean swapYZ = tHints != null ? tHints.swapYZ
//				: DataPool.CoordinateTransformationSettings.swapYZ;
//		final boolean sortByType = tHints != null ? tHints.sortByType
//				: DataPool.CoordinateTransformationSettings.sortByType;
//
//		this.cont = new Content();
//
//		// add all proteins to the content
//		for (int i = 0; i < this.model.getProteinCount(); i++) {
//			final Protein_3D prot = this.model.getProteinAt(i);
//
//			if (this.dialog != null)
//				this.dialog.setProgress("Reading Protein " + prot.getID(),
//						(int) (counter * step), true);
//
//			final Line[] readins = this.createLines(prot);
//
//			for (int j = 0; j < readins.length; j++) {
//
//				this.cont.add(readins[j]);
//
//				counter++;
//
//			}
//			this.cont.finishMolecule(prot);
//
//		}
//		final int end = this.model.getLipidCountExtra()
//				+ this.model.getLipidCountIntra();
//		int current = 1;
//
//		final int[] originalLipidExtraNumber = new int[this.model
//				.getLipidCountExtra()];
//		final int[] originalLipidIntraNumber = new int[this.model
//				.getLipidCountIntra()];
//		
//		// add all lipids to the content
//		// sort the lipids by membrane side
//		if (!sortByType) {
//		
//			int currentLipidPerLayer = 0;
//			
//			for (int i = 0; i < 2; i++) {
//	
//				int lipidCount;
//				String[] allLipidTypes;
//				if (i == 0) {
//					lipidCount = this.model.getLipidCountExtra();
//					allLipidTypes = this.model.getAllLipidTypesInMembraneExtra();
//				} else {
//					lipidCount = this.model.getLipidCountIntra();
//					allLipidTypes = this.model.getAllLipidTypesInMembraneIntra();
//				}
//				currentLipidPerLayer = 0;
//	
//				for (int lipidTypeCount = 0; lipidTypeCount < allLipidTypes.length; lipidTypeCount++) {
//	
//					for (int j = 0; j < lipidCount; j++) {
//	
//						Lipid_3D lip;
//						if (i == 0) {
//							lip = this.model.getLipidAtExtra(j);
//						} else {
//							lip = this.model.getLipidAtIntra(j);
//						}
//	
//						if (lip.getPDBcode().equals(allLipidTypes[lipidTypeCount])) {
//	
//							if (this.dialog != null)
//								this.dialog.setProgress("Reading Lipids ( "
//										+ current + " / " + end + " )",
//										(int) (counter * step), true);
//	
//							final Line[] readins = this.createLines(lip);
//	
//							for (int k = 0; k < readins.length; k++) {
//	
//								this.cont.add(readins[k]);
//	
//								counter++;
//	
//							}
//							this.cont.finishMolecule(lip);
//							if (i == 0)
//								originalLipidExtraNumber[currentLipidPerLayer] = j;
//							else
//								originalLipidIntraNumber[currentLipidPerLayer] = j;
//							current++;
//							currentLipidPerLayer++;
//						}
//	
//					}
//				}
//			}
//			
//		// add all lipids to the content
//		// sort the lipids by type
//		} else {
//			
//			String[] allLipidTypes 	= this.model.getAllLipidTypes();
//			int lipidCountExtra 	= this.model.getLipidCountExtra();
//			int lipidCountIntra 	= this.model.getLipidCountIntra();
//			int currentLipidExtra 	= 0;
//			int currentLipidIntra 	= 0;
//			
//			for (int lipidTypeCount = 0; lipidTypeCount < allLipidTypes.length; lipidTypeCount++) {
//				
//				int currentNumberOfActualType = 0;
//	
//				for (int j = 0; j < lipidCountExtra; j++) {
//
////					System.out.println("lipidTypeCount "+lipidTypeCount+" lipidCountExtra "+lipidCountExtra+":"+j);
//					
//					Lipid_3D lip;
//					lip = this.model.getLipidAtExtra(j);
//
//					if (lip.getPDBcode().equals(allLipidTypes[lipidTypeCount])) {
////						System.out.println("Hit for "+allLipidTypes[lipidTypeCount]+" "+lip.getDomainID()+" "+"extra"+j);
//						if (this.dialog != null)
//							this.dialog.setProgress("Reading Lipids ( "
//									+ current + " / " + end + " )",
//									(int) (counter * step), true);
//
//						final Line[] readins = this.createLines(lip);
//
//						for (int k = 0; k < readins.length; k++) {
//
//							this.cont.add(readins[k]);
//
//							counter++;
//
//						}
//						this.cont.finishMolecule(lip);
////						originalLipidExtraNumber[currentLipidExtra] = j;
//						current++;
//						currentLipidExtra++;
//						currentNumberOfActualType++;
//					}
//	
//				}
//				
//				for (int j = 0; j < lipidCountIntra; j++) {
//
////					System.out.println("lipidTypeCount "+lipidTypeCount+" lipidCountIntra "+lipidCountIntra+":"+j);
//					
//					Lipid_3D lip;
//					lip = this.model.getLipidAtIntra(j);
//
//					if (lip.getPDBcode().equals(allLipidTypes[lipidTypeCount])) {
////						System.out.println("Hit for "+allLipidTypes[lipidTypeCount]+" "+lip.getDomainID()+" "+"intra"+j);
//						if (this.dialog != null)
//							this.dialog.setProgress("Reading Lipids ( "
//									+ current + " / " + end + " )",
//									(int) (counter * step), true);
//
//						final Line[] readins = this.createLines(lip);
//
//						for (int k = 0; k < readins.length; k++) {
//
//							this.cont.add(readins[k]);
//
//							counter++;
//
//						}
//						this.cont.finishMolecule(lip);
////						originalLipidIntraNumber[currentLipidIntra] = j;
//						current++;
//						currentLipidIntra++;
//						currentNumberOfActualType++;
//					}
//	
//				}
//				
//				topologyVec.add(new String[]{allLipidTypes[lipidTypeCount].substring(0,3),Integer.toString(currentNumberOfActualType)});
//				
//			}
//
//		}
//
//		this.cont.finish();
//
//		int subX = 0;
//		int subZ = 0;
//
//		if ((tHints != null ? tHints.shiftXZ
//				: DataPool.CoordinateTransformationSettings.shiftXZ) == 2) {
//			subX += this.model.getSize().width / 2;
//			subZ += this.model.getSize().height / 2;
//		}
//
//		if ((tHints != null ? tHints.shiftXZ
//				: DataPool.CoordinateTransformationSettings.shiftXZ) == 1
//				|| (tHints != null ? tHints.shiftY
//						: DataPool.CoordinateTransformationSettings.shiftY) == 1)
//			step = 5f / (float) totalCount;
//		else
//			step = 10f / (float) totalCount;
//
//		counter = 0f;
//
//		// transform positions and matrices of all proteins
//		for (int i = 0; i < this.model.getProteinCount(); i++) {
//			final Line[] lines = this.cont.getAll(i, 0);
//			final Matrix3f matrix = this.model.getProteinAt(i)
//					.getRotationMatrix();
//
//			final Point3f position = this.model.getProteinAt(i).getPosition();
//			for (int j = 0; j < lines.length; j++) {
//				if (this.dialog != null)
//					this.dialog.setProgress("Transforming...",
//							(int) (counter * step) + 30, true);
//
//				if (lines[j] != null) {
//
//					// replace the original resSeq number taken from the
//					// original
//					// lipid file
//					// with a running number counting the lipids
//					if (resSeqCount)
//						lines[j].resSeq = "" + (i + 1);
//
//					lines[j].position = Cm2_Toolkit.transformToNewPoint(matrix,
//							lines[j].position);
//
//					lines[j].position = new Point3f(lines[j].position.x
//							+ position.x - subX, lines[j].position.y
//							+ position.y, lines[j].position.z + position.z
//							- subZ);
//
//					counter++;
//
//					final Point3f p = lines[j].position;
//					minx = Math.min(minx, p.x);
//					miny = Math.min(miny, p.y);
//					minz = Math.min(minz, p.z);
//
//					maxx = Math.max(maxx, p.x);
//					maxy = Math.max(maxy, p.y);
//					maxz = Math.max(maxz, p.z);
//					// if (maxAtomPosX < lines[j].position.x) maxAtomPosX =
//					// lines[j].position.x;
//					// if (maxAtomPosY < lines[j].position.y) maxAtomPosY =
//					// lines[j].position.y;
//					// if (maxAtomPosZ < lines[j].position.z) maxAtomPosZ =
//					// lines[j].position.z;
//				}
//			}
//		}
//
//		// transform positions and matrices of all lipids
//		// sort the lipids by membrane side
//		if (!sortByType) {
//
//			for (int i = 0; i < this.model.getLipidCountExtra(); i++) {
//				final Line[] lines = this.cont.getAll(i, 1);
//				final Matrix3f matrix = this.model.getLipidAtExtra(
//						originalLipidExtraNumber[i]).getRotationMatrix();
//	
//				final Point3f position = this.model.getLipidAtExtra(
//						originalLipidExtraNumber[i]).getPosition();
//				for (int j = 0; j < lines.length; j++) {
//					if (this.dialog != null)
//						this.dialog.setProgress("Transforming...",
//								(int) (counter * step) + 30, true);
//	
//					// replace the original resSeq number taken from the original
//					// lipid file
//					// with a running number counting the lipids
//					if (resSeqCount)
//						lines[j].resSeq = ""
//								+ (i + 1 + this.model.getProteinCount());
//	
//					lines[j].position = Cm2_Toolkit.transformToNewPoint(matrix,
//							lines[j].position);
//	
//					lines[j].position = new Point3f(lines[j].position.x
//							+ position.x - subX, lines[j].position.y + position.y,
//							lines[j].position.z + position.z - subZ);
//	
//					counter++;
//	
//					final Point3f p = lines[j].position;
//					minx = Math.min(minx, p.x);
//					miny = Math.min(miny, p.y);
//					minz = Math.min(minz, p.z);
//	
//					maxx = Math.max(maxx, p.x);
//					maxy = Math.max(maxy, p.y);
//					maxz = Math.max(maxz, p.z);
//	
//					// if (maxAtomPosX < lines[j].position.x) maxAtomPosX =
//					// lines[j].position.x;
//					// if (maxAtomPosY < lines[j].position.y) maxAtomPosY =
//					// lines[j].position.y;
//					// if (maxAtomPosZ < lines[j].position.z) maxAtomPosZ =
//					// lines[j].position.z;
//				}
//			}
//			for (int i = 0; i < this.model.getLipidCountIntra(); i++) {
//				final Line[] lines = this.cont.getAll(i, 2);
//				final Matrix3f matrix = this.model.getLipidAtIntra(
//						originalLipidIntraNumber[i]).getRotationMatrix();
//	
//				final Point3f position = this.model.getLipidAtIntra(
//						originalLipidIntraNumber[i]).getPosition();
//				for (int j = 0; j < lines.length; j++) {
//					if (this.dialog != null)
//						this.dialog.setProgress("Transforming...",
//								(int) (counter * step) + 30, true);
//	
//					// replace the original resSeq number taken from the original
//					// lipid file
//					// with a running number counting the lipids
//					if (resSeqCount)
//						lines[j].resSeq = ""
//								+ (i + 1 + this.model.getProteinCount() + this.model
//										.getLipidCountExtra());
//	
//					lines[j].position = Cm2_Toolkit.transformToNewPoint(matrix,
//							lines[j].position);
//	
//					lines[j].position = new Point3f(lines[j].position.x
//							+ position.x - subX, lines[j].position.y + position.y,
//							lines[j].position.z + position.z - subZ);
//	
//					counter++;
//	
//					final Point3f p = lines[j].position;
//					minx = Math.min(minx, p.x);
//					miny = Math.min(miny, p.y);
//					minz = Math.min(minz, p.z);
//	
//					maxx = Math.max(maxx, p.x);
//					maxy = Math.max(maxy, p.y);
//					maxz = Math.max(maxz, p.z);
//					// if (maxAtomPosX < lines[j].position.x) maxAtomPosX =
//					// lines[j].position.x;
//					// if (maxAtomPosY < lines[j].position.y) maxAtomPosY =
//					// lines[j].position.y;
//					// if (maxAtomPosZ < lines[j].position.z) maxAtomPosZ =
//					// lines[j].position.z;
//				}
//			}
//			
//		// transform positions and matrices of all lipids
//		// sort the lipids by type
//		} else {
//			
//			String[] allLipidTypes =this.model.getAllLipidTypes();
//			int lipidCountExtra = this.model.getLipidCountExtra();
//			int lipidCountIntra = this.model.getLipidCountIntra();
////			int currentLipidExtra = 0;
////			int currentLipidIntra = 0;
//	
//			int lipidCount = 0;
//			
//			for (int lipidTypeCount = 0; lipidTypeCount < allLipidTypes.length; lipidTypeCount++) {
//				
//				for (int i = 0; i < lipidCountExtra; i++) {
////					System.out.println("Transform extra "+i+"/"+lipidCountExtra+" "+lipidTypeCount);
//					Lipid_3D lip;
//					lip = this.model.getLipidAtExtra(i);
//					
//					if (lip.getPDBcode().equals(allLipidTypes[lipidTypeCount])) {
////						System.out.println("Hit for "+allLipidTypes[lipidTypeCount]+" "+lipidCount);
////						final Line[] lines = this.cont.getAll(i, 1);
//						final Line[] lines = this.cont.getAll(lipidCount++, 1);
//						final Matrix3f matrix = lip.getRotationMatrix();
//			
//						final Point3f position = lip.getPosition();
//						for (int j = 0; j < lines.length; j++) {
//							if (this.dialog != null)
//								this.dialog.setProgress("Transforming...",
//										(int) (counter * step) + 30, true);
//			
//							// replace the original resSeq number taken from the original
//							// lipid file
//							// with a running number counting the lipids
//							if (resSeqCount)
////								lines[j].resSeq = ""
////										+ (i + 1 + this.model.getProteinCount());
//								lines[j].resSeq = ""
//									+ (lipidCount + this.model.getProteinCount());
//			
//							lines[j].position = Cm2_Toolkit.transformToNewPoint(matrix,
//									lines[j].position);
//			
//							lines[j].position = new Point3f(lines[j].position.x
//									+ position.x - subX, lines[j].position.y + position.y,
//									lines[j].position.z + position.z - subZ);
//			
//							counter++;
//			
//							final Point3f p = lines[j].position;
//							minx = Math.min(minx, p.x);
//							miny = Math.min(miny, p.y);
//							minz = Math.min(minz, p.z);
//			
//							maxx = Math.max(maxx, p.x);
//							maxy = Math.max(maxy, p.y);
//							maxz = Math.max(maxz, p.z);
//						}
//						
//					}
//	
//				}
//				
//				for (int i = 0; i < lipidCountIntra; i++) {
////					System.out.println("Transform intra "+i+"/"+lipidCountIntra+" "+lipidTypeCount);
//					Lipid_3D lip;
//					lip = this.model.getLipidAtIntra(i);
//					
//					if (lip.getPDBcode().equals(allLipidTypes[lipidTypeCount])) {
////						System.out.println("Hit for "+allLipidTypes[lipidTypeCount]+" "+lipidCount);
////						final Line[] lines = this.cont.getAll(i, 2);
//						final Line[] lines = this.cont.getAll(lipidCount++, 1);
//						final Matrix3f matrix = lip.getRotationMatrix();
//			
//						final Point3f position = lip.getPosition();
//						for (int j = 0; j < lines.length; j++) {
//							if (this.dialog != null)
//								this.dialog.setProgress("Transforming...",
//										(int) (counter * step) + 30, true);
//			
//							// replace the original resSeq number taken from the original
//							// lipid file
//							// with a running number counting the lipids
//							if (resSeqCount)
////								lines[j].resSeq = ""
////										+ (i + 1 + this.model.getProteinCount());
//								lines[j].resSeq = ""
//									+ (lipidCount + this.model.getProteinCount());
//			
//							lines[j].position = Cm2_Toolkit.transformToNewPoint(matrix,
//									lines[j].position);
//			
//							lines[j].position = new Point3f(lines[j].position.x
//									+ position.x - subX, lines[j].position.y + position.y,
//									lines[j].position.z + position.z - subZ);
//			
//							counter++;
//			
//							final Point3f p = lines[j].position;
//							minx = Math.min(minx, p.x);
//							miny = Math.min(miny, p.y);
//							minz = Math.min(minz, p.z);
//			
//							maxx = Math.max(maxx, p.x);
//							maxy = Math.max(maxy, p.y);
//							maxz = Math.max(maxz, p.z);
//						}
//	
//					}
//					
//				}
//				
//			}
//			
//		}
//
//		if ((tHints != null ? tHints.shiftXZ
//				: DataPool.CoordinateTransformationSettings.shiftXZ) == 1
//				|| (tHints != null ? tHints.shiftY
//						: DataPool.CoordinateTransformationSettings.shiftY) > 0) {
//			step = 5f / (float) totalCount;
//			counter = 0f;
//
//			final float cx = (tHints != null ? tHints.shiftXZ
//					: DataPool.CoordinateTransformationSettings.shiftXZ) == 1 ? (minx + ((maxx - minx) / 2))
//					: 0;
//			float cy = (tHints != null ? tHints.shiftY
//					: DataPool.CoordinateTransformationSettings.shiftY) == 1 ? (miny + ((maxy - miny) / 2))
//					: 0;
//			final float cz = (tHints != null ? tHints.shiftXZ
//					: DataPool.CoordinateTransformationSettings.shiftXZ) == 1 ? (minz + ((maxz - minz) / 2))
//					: 0;
//
//			if ((tHints != null ? tHints.shiftY
//					: DataPool.CoordinateTransformationSettings.shiftY) == 2)
//				cy = miny;
//
//			step = 5f / (float) totalCount;
//			counter = 0f;
//
//			// transform the positions of all proteins
//			for (int i = 0; i < this.model.getProteinCount(); i++) {
//				final Line[] lines = this.cont.getAll(i, 0);
//
//				for (int j = 0; j < lines.length; j++) {
//					if (this.dialog != null)
//						this.dialog.setProgress("Transforming...",
//								(int) (counter * step) + 40, true);
//					lines[j].position = new Point3f(lines[j].position.x - cx,
//							lines[j].position.y - cy, lines[j].position.z - cz);
//					counter++;
//				}
//
//			}
//			
//			// transform the positions of all lipids
//			// sort the lipids by membrane side
//			if (!sortByType) {
//				for (int i = 0; i < this.model.getLipidCountExtra(); i++) {
//					final Line[] lines = this.cont.getAll(i, 1);
//	
//					for (int j = 0; j < lines.length; j++) {
//						if (this.dialog != null)
//							this.dialog.setProgress("Transforming...",
//									(int) (counter * step) + 40, true);
//						lines[j].position = new Point3f(lines[j].position.x - cx,
//								lines[j].position.y - cy, lines[j].position.z - cz);
//						counter++;
//					}
//	
//				}
//				for (int i = 0; i < this.model.getLipidCountIntra(); i++) {
//					final Line[] lines = this.cont.getAll(i, 2);
//	
//					for (int j = 0; j < lines.length; j++) {
//						if (this.dialog != null)
//							this.dialog.setProgress("Transforming...",
//									(int) (counter * step) + 40, true);
//						lines[j].position = new Point3f(lines[j].position.x - cx,
//								lines[j].position.y - cy, lines[j].position.z - cz);
//						counter++;
//					}
//	
//				}
//			
//			// transform the positions of all lipids
//			// sort the lipids by type
//			} else {
//				
//				String[] allLipidTypes =this.model.getAllLipidTypes();
//				int lipidCountExtra = this.model.getLipidCountExtra();
//				int lipidCountIntra = this.model.getLipidCountIntra();
////				int currentLipidExtra = 0;
////				int currentLipidIntra = 0;
//				
//				int lipidCount = 0;
//		
//				for (int lipidTypeCount = 0; lipidTypeCount < allLipidTypes.length; lipidTypeCount++) {
//					
//					for (int i = 0; i < lipidCountExtra; i++) {
////						System.out.println("Positioning extra "+i+"/"+lipidCountExtra+" "+lipidTypeCount);
//						Lipid_3D lip;
//						lip = this.model.getLipidAtExtra(i);
//						
//						if (lip.getPDBcode().equals(allLipidTypes[lipidTypeCount])) {
////							System.out.println("Hit for "+allLipidTypes[lipidTypeCount]+" "+lipidCount);
////							final Line[] lines = this.cont.getAll(i, 1);
//							final Line[] lines = this.cont.getAll(lipidCount++, 1);
//							
//							for (int j = 0; j < lines.length; j++) {
//								if (this.dialog != null)
//									this.dialog.setProgress("Transforming...",
//											(int) (counter * step) + 40, true);
//								lines[j].position = new Point3f(lines[j].position.x - cx,
//										lines[j].position.y - cy, lines[j].position.z - cz);
//								counter++;
//							}
//							
//						}
//		
//					}
//					
//					for (int i = 0; i < lipidCountIntra; i++) {
////						System.out.println("Positioning intra "+i+"/"+lipidCountIntra+" "+lipidTypeCount);
//						Lipid_3D lip;
//						lip = this.model.getLipidAtIntra(i);
//						
//						if (lip.getPDBcode().equals(allLipidTypes[lipidTypeCount])) {
////							System.out.println("Hit for "+allLipidTypes[lipidTypeCount]+" "+lipidCount);
////							final Line[] lines = this.cont.getAll(i, 2);
//							final Line[] lines = this.cont.getAll(lipidCount++, 1);
//							
//							for (int j = 0; j < lines.length; j++) {
//								if (this.dialog != null)
//									this.dialog.setProgress("Transforming...",
//											(int) (counter * step) + 40, true);
//								lines[j].position = new Point3f(lines[j].position.x - cx,
//										lines[j].position.y - cy, lines[j].position.z - cz);
//								counter++;
//							}
//							
//						}
//		
//					}
//					
//				}
//			}
//
//		}
//
//		if (swapYZ) {
//			for (int i = 0; i < this.cont.getLength(); i++) {
//				final Line line = this.cont.getAt(i);
//				line.position = new Point3f(line.position.x, line.position.z,
//						line.position.y);
//			}
//		}
//
//		if (this.dialog != null)
//			this.dialog.setProgress("Reading Conects...", 45, true);
//
//		this.readInConects(dialog);
//
//		if (this.dialog != null)
//			this.dialog.setProgress("Generating Structures...", 60, true);
//
//		this.generateStructures();
//
//		if (this.dialog != null) {
//			this.dialog.setProgress("Writing Header Section...", 65, true);
//		}
//		this.writeHeader();
//
//		if (this.dialog != null) {
//			this.dialog.setProgress("Writing Structures...", 68, true);
//		}
//
//		for (int i = 0; i < this.cont.dbref.size(); i++) {
//			this.BW.write(this.cont.dbref.get(i));
//			this.BW.newLine();
//		}
//		for (int i = 0; i < this.cont.seqadv.size(); i++) {
//			this.BW.write(this.cont.seqadv.get(i));
//			this.BW.newLine();
//		}
//		for (int i = 0; i < this.cont.seqres.size(); i++) {
//			this.BW.write(this.cont.seqres.get(i));
//			this.BW.newLine();
//		}
//		for (int i = 0; i < this.cont.modres.size(); i++) {
//			this.BW.write(this.cont.modres.get(i));
//			this.BW.newLine();
//		}
//		for (int i = 0; i < this.cont.het.size(); i++) {
//			this.BW.write(this.cont.het.get(i));
//			this.BW.newLine();
//		}
//		for (int i = 0; i < this.cont.helix.size(); i++) {
//			this.BW.write(this.cont.helix.get(i));
//			this.BW.newLine();
//		}
//		for (int i = 0; i < this.cont.sheet.size(); i++) {
//			this.BW.write(this.cont.sheet.get(i));
//			this.BW.newLine();
//		}
//		for (int i = 0; i < this.cont.turn.size(); i++) {
//			this.BW.write(this.cont.turn.get(i));
//			this.BW.newLine();
//		}
//		for (int i = 0; i < this.cont.ssbond.size(); i++) {
//			this.BW.write(this.cont.ssbond.get(i));
//			this.BW.newLine();
//		}
//		for (int i = 0; i < this.cont.link.size(); i++) {
//			this.BW.write(this.cont.link.get(i));
//			this.BW.newLine();
//		}
//		for (int i = 0; i < this.cont.cispep.size(); i++) {
//			this.BW.write(this.cont.cispep.get(i));
//			this.BW.newLine();
//		}
//		for (int i = 0; i < this.cont.site.size(); i++) {
//			this.BW.write(this.cont.site.get(i));
//			this.BW.newLine();
//		}
//
//		final float CrystA = maxx - minx;
//		final float CrystB = swapYZ ? maxz - minz : maxy - miny;
//		final float CrystC = swapYZ ? maxy - miny : maxz - minz;
//
//		this.writeCRYST(CrystA, CrystB, CrystC);
//
//		step = 28f / (float) totalCount;
//
//		counter = 0f;
//
//		int endofserialnumbers;
//
//		if (this.useSerials == this.PROTSONLY) {
//			endofserialnumbers = this.model.countAtoms(0, -1, -1,
//					DataPool.PPESettings.saveOPMDummyLayersInPDB);
//			endofserialnumbers+=this.model.countTERs(0,-1,-1);
//		}else if (this.useSerials == this.BOTH) {
//			endofserialnumbers = this.model.countAtoms(0, -1, -1,
//					DataPool.PPESettings.saveOPMDummyLayersInPDB)
//					+ this.model.countAtoms(1, -1, -1,
//							DataPool.PPESettings.saveOPMDummyLayersInPDB)
//					+ this.model.countAtoms(2, -1, -1,
//							DataPool.PPESettings.saveOPMDummyLayersInPDB);
//			endofserialnumbers+=this.model.countTERs(0,-1,-1);
//			endofserialnumbers+=this.model.countTERs(2,-1,-1);
//			endofserialnumbers+=this.model.countTERs(1,-1,-1);
//			endofserialnumbers+=this.model.countTERs(2,-1,-1);
//		}else
//			endofserialnumbers = 0;
//		
//		int index = 0;
//		for (int i = 0; i < this.cont.getLength(); i++) {
//
//			if (this.dialog != null)
//				this.dialog.setProgress("Writing...",
//						(int) (counter * step) + 70, true);
//
//			final Line line = this.cont.getAt(i);
//
//			if (line != null) {
//				String out = "";
//				out += Cm2_Toolkit.extendRight(line.recordName, 6);
//				if (index < endofserialnumbers
//						|| !DataPool.LimitationSettings.all99999)
//					out += Cm2_Toolkit.extendLeft("" + (index + 1), 5);
//				else
//					out += Cm2_Toolkit.extendLeft("99999", 5);
//				out += Cm2_Toolkit.extendLeft(line.name, 5);
//				line.altLoc = "";
//				out += Cm2_Toolkit.extendLeft(line.altLoc, 1);
//				out += Cm2_Toolkit.extendLeft(line.residueName, 3);
//				out += Cm2_Toolkit.extendLeft(line.chainId, 2);
//				out += Cm2_Toolkit.extendLeft(line.resSeq, 4);
//				out += Cm2_Toolkit.extendLeft(line.iCode, 1);
//				out += "   ";
//
//				if (line.recordName.startsWith("TER")) {
//					out += Cm2_Toolkit.extendLeft("", 24);
//
//					this.tercount++;
//				} else {
//					out += Cm2_Toolkit.extendLeft(this
//							.createNumberString(line.position.x), 8);
//					out += Cm2_Toolkit.extendLeft(this
//							.createNumberString(line.position.y), 8);
//					out += Cm2_Toolkit.extendLeft(this
//							.createNumberString(line.position.z), 8);
//					this.atomhetatmcount++;
//				}
//				out += Cm2_Toolkit.extendLeft(line.occupancy, 6);
//				out += " ";
//				out += Cm2_Toolkit.extendLeft(line.tempFactor, 5);
//				out += "       ";
//				out += Cm2_Toolkit.extendLeft(line.segID, 3);
//
//				
//				out += Cm2_Toolkit.extendLeft(line.element, 2);
//				out += Cm2_Toolkit.extendLeft(line.charge, 2);
//
//				counter++;
//				index++;
//				if (index > 99998
//						&& (DataPool.LimitationSettings.all99999 == false))
//					index = 0;
//
//				this.BW.write(out);
//				this.BW.newLine();
//			}
//
//		}
//
//		if (this.dialog != null)
//			this.dialog.setProgress("Writing Conects...", 98, true);
//
//		for (int i = 0; i < this.cont.conects.size(); i++) {
//			this.BW.write(this.cont.conects.get(i));
//			this.BW.newLine();
//		}
//		this.writeMaster();
//		this.BW.write("END");
//
//		if (this.dialog != null)
//			this.dialog.setProgress("Finished", 100, true);
//
//		if (tHints.exportToplogy){
//			//Write Topology File
////			System.out.println("Write Topology here: ");
////			
//			BufferedWriter out;
//			out = new BufferedWriter( new FileWriter(tHints.projectFolder  + File.separator + "topol.top"));
//			
//			out.write("; Include forcefield parameters\n");
//			out.write("#include \"ffG45a3.itp\"\n");
//			out.write("\n");
//			out.write("; Include Content\n");
//			out.write("#include \"lipidsFFG45a3.itp\"\n");
//			out.write("#include \"ions.itp\"\n");
//			out.write("\n");
//			out.write("; include water\n");
//			out.write("#include \"flexspce.itp\"\n");
//			out.write("\n");
//			out.write(";System Name\n");
//			out.write("[System]\n");
//			out.write("lipids in water\n");
//			out.write("\n");
//			out.write("; Anzahl der Mol in der pdb in gleicher reihenfolge\n");
//			out.write("[molecules]\n");
//			for(String[] lipidString : topologyVec){
//				out.write(lipidString[0] + "              " + lipidString[1] + "\n");
//			}     
//			
//            out.close();	
//
//            //Write Mdp Files
//            //first minimize.mdp
//            BufferedReader in = new BufferedReader( new FileReader(tHints.projectFolder  + File.separator + "minimize.mdp"));
////          System.out.println("Read File : " + tHints.projectFolder  + File.separator + "minimize.mdp");
//            String minMdp = "";
//            String line;
//            line = in.readLine();
//            if (line != null){
//            	
//            	do{
//                	minMdp = minMdp + line + "\n";
//                	line = in.readLine();
//                }while(line != null);
//            }
//            //find >energygrps< and split the string
//            int energyPos = minMdp.indexOf(";ENERGYGRPS");
//            //search of "energygroups" and insert the lipidgroups
//            String mdpBegin = minMdp.substring(0,energyPos);
//            String mdpEnd = minMdp.substring(energyPos + "ENERGYGRPS".length() + 1);
//            String mdpLipids = "";
//            //paste in the lipid types
//            for(String[] lipidString : topologyVec){
//				mdpLipids = mdpLipids + " " + lipidString[0] ;
//			}  
//            //write the new file
//            String output = mdpBegin + "energygrps                =" + mdpLipids + "\n" + mdpEnd;
//
//            out = new BufferedWriter( new FileWriter(tHints.projectFolder  + File.separator + "minimize.mdp"));
//            out.write(output);
//            out.close();
//            
//            //now NPT.mdp
//            in = new BufferedReader( new FileReader(tHints.projectFolder  + File.separator + "NPT.mdp"));
//            minMdp = "";
//            line = in.readLine();
//            if (line != null){
//            	do{
//            		minMdp = minMdp + line + "\n";
//                	line = in.readLine();
//                }while(line != null);
//            }
//           
//            int numbLipids = 0;
//            energyPos = minMdp.indexOf(";ENERGYGRPS");
//            //search of "energygroups" and insert the lipidgroups
//            mdpBegin = minMdp.substring(0,energyPos);
//            mdpEnd = minMdp.substring(energyPos + "ENERGYGRPS".length() + 1);
//            mdpLipids = "";
//            for(String[] lipidString : topologyVec){
//				mdpLipids = mdpLipids + " " + lipidString[0] ;
//				numbLipids++;
//			}  
//            output = mdpBegin + "energygrps                =" + mdpLipids + "\n" + mdpEnd;
//            
//            //search for tc-groups, tau_t and ref_t and insert standard values
//            int tcgrpsPos = output.indexOf(";TCGRPS");
//            mdpBegin = output.substring(0,tcgrpsPos);
////            System.out.println(output.substring(0,tcgrpsPos) + "<-- MDPBEGIN");
////            System.out.println(output.substring(tcgrpsPos) + "<-- MDPEND");
//            mdpEnd = output.substring(tcgrpsPos + "TCGRPS".length() +1);
//            output = mdpBegin + "tc-grps                 =" + mdpLipids  + " SOL " + mdpEnd;
//            
//            String taut = "0.1";
//            int tautPos = output.indexOf(";TAUT");
//            mdpBegin = output.substring(0,tautPos);
//            mdpEnd = output.substring(tautPos  + "TAUT".length() +1);
//            for (int i=0; i<numbLipids; i++){
//            	taut = taut + " 0.1"; 
//            }
//            output = mdpBegin + "tau_t                = " + taut + mdpEnd;
//            
//            String reft = "300";
//            int reftPos = output.indexOf(";REFT");
//            mdpBegin = output.substring(0,reftPos);
//            mdpEnd = output.substring(reftPos  + "REFT".length() +1);
//            for (int i=0; i<numbLipids; i++){
//            	reft = reft + " 300"; 
//            }
//            output = mdpBegin + "ref_t                = " + reft + mdpEnd;
//            
//            out = new BufferedWriter( new FileWriter(tHints.projectFolder  + File.separator + "NPT.mdp"));
//            out.write(output);
//            out.close();
//            
//            //at last dynamics.mdp
//            in = new BufferedReader( new FileReader(tHints.projectFolder  + File.separator + "dynamics.mdp"));
//            minMdp = "";
//            line = in.readLine();
//            if (line != null){
//            	do{
//            		minMdp = minMdp + line + "\n";
//                	line = in.readLine();
//                }while(line != null);
//            }
//            energyPos = minMdp.indexOf(";ENERGYGRPS");
//            //search of "energygroups" and insert the lipidgroups
//            mdpBegin = minMdp.substring(0,energyPos);
//            mdpEnd = minMdp.substring(energyPos + "ENERGYGRPS".length() + 1);
//            mdpLipids = "";
//            for(String[] lipidString : topologyVec){
//				mdpLipids = mdpLipids + " " + lipidString[0] ;
//			}  
//            output = mdpBegin + "energygrps                =" + mdpLipids + "\n" + mdpEnd;
//            
//          //search for tc-groups, tau_t and ref_t and insert standard values
//            tcgrpsPos = output.indexOf(";TCGRPS");
//            mdpBegin = output.substring(0,tcgrpsPos);
////            System.out.println(output.substring(0,tcgrpsPos) + "<-- MDPBEGIN");
////            System.out.println(output.substring(tcgrpsPos) + "<-- MDPEND");
//            mdpEnd = output.substring(tcgrpsPos + "TCGRPS".length() +1);
//            output = mdpBegin + "tc-grps                 =" + mdpLipids  + " SOL " + mdpEnd;
//            
//            tautPos = output.indexOf(";TAUT");
//            mdpBegin = output.substring(0,tautPos);
//            mdpEnd = output.substring(tautPos  + "TAUT".length() +1);
//            output = mdpBegin + "tau_t                = " + taut + mdpEnd;
//            
//            reftPos = output.indexOf(";REFT");
//            mdpBegin = output.substring(0,reftPos);
//            mdpEnd = output.substring(reftPos  + "REFT".length() +1);
//            output = mdpBegin + "ref_t                = " + reft + "\n" + mdpEnd;
//            
//            out = new BufferedWriter( new FileWriter(tHints.projectFolder  + File.separator + "dynamics.mdp"));
//            out.write(output);
//            out.close();
//            
//		}
//		
//		this.BW.close();
//		System.gc();
//		
//		return true;
//		
//	}
//
//	private Line[] createLines(final Molecule_3D mol) throws IOException {
//
//		final String[][] readin = new PdbReader_3D().readModel(DataPool
//				.getRawPdbData(mol.getPDBFile().getAbsolutePath()), mol
//				.getModelSet());
//		// final Line[] lines = new
//		// Line[readin[0].length+(DataPool.PPESettings.saveOPMDummyLayersInPDB?readin[1].length:0)];
//
//		Vector<Line> linesTemp = new Vector<Line>();
//
//		Vector<Particle> alreadyIn = new Vector<Particle>();
//
//		for (int i = 0; i < (DataPool.PPESettings.saveOPMDummyLayersInPDB ? 2
//				: 1); i++) {
//			int c = 0;
//			final Atom_3D[] atoms = i == 0 ? mol.allAtomsSorted : mol.opmAtoms;
//
//			for (int j = 0; j < readin[i].length; j++) {
//				
//				Atom_3D atm;
//
//				try {
//					new Atom_3D(readin[i][j]);
//					atm = atoms[c];
//					c++;
//
//					Particle p = mol.getHostParticle(atm);
//					if (p != null) {
//						if (alreadyIn.contains(p)) {
//							continue;
//						}
//						alreadyIn.addElement(p);
//						final Line line = new Line(p, readin[i][j], mol
//								.getDomainID());
//
//						line.name = p.name;
//					line.comparator=readin.length+j;
//
//						linesTemp.addElement(line);
//					} else {
//						final Line line = new Line(atm, readin[i][j], mol
//								.getDomainID());
//						linesTemp.addElement(line);
//						line.comparator=j;
//					}
//
//				} catch (final Exception e) {
//					// e.printStackTrace();
//					final Line line = new Line(null, readin[i][j], mol
//							.getDomainID());
//					linesTemp.addElement(line);
//					line.comparator=j;
//				}
//
//			}
//
//		}
//
//		Line prevL = null;
//		for (int i = 0; i < linesTemp.size(); i++) {
//			Atom_3D a = linesTemp.get(i).atomOfMol;
//
//			if (a == null)
//				continue;
//
//			if (a instanceof Particle) {
//				Line ref = null;
//				if (prevL != null) {
//					ref = prevL;
//				} else {
//					for (int k = i + 1; k < linesTemp.size(); k++) {
//						if (linesTemp.get(k).atomOfMol != null
//								&& (linesTemp.get(k).atomOfMol instanceof Particle) == false) {
//							ref = linesTemp.get(k);
//							break;
//						}
//					}
//				}
//
//				if (ref == null)
//					continue;
//
//				Line line = linesTemp.get(i);
//
//				line.recordName = ref.recordName;
//				line.occupancy = ref.occupancy;
//				line.tempFactor = ref.tempFactor;
//
//				line.charge = ref.charge;
//				line.residueName = ref.residueName;
//
//				line.altLoc = ref.altLoc;
//				line.chainId = ref.chainId;
//				line.resSeq = ref.resSeq;
//				line.iCode = ref.iCode;
//				line.segID = ref.segID;
//
//				String element = Cm2_Toolkit
//						.convertAtomNameToElementName(line.name);
//				if (element != null)
//					line.element = element;
//				else
//					line.element = "";
//
//			} else {
//				prevL = linesTemp.get(i);
//			}
//		}
//		
////		Collections.sort(linesTemp);
//
//		return linesTemp.toArray(new Line[0]);
//	}
//
//	private void writeCRYST(final float a, final float b, final float c)
//			throws IOException {
//
//		final String s = "CRYST1"
//				+ Cm2_Toolkit.extendLeft(this.createNumberString(a), 9)
//				+ Cm2_Toolkit.extendLeft(this.createNumberString(b), 9)
//				+ Cm2_Toolkit.extendLeft(this.createNumberString(c), 9)
//				+ "  90.00  90.00  90.00 P 1           1";
//		this.BW.write(s);
//		this.BW.newLine();
//
//		this.BW
//				.write("ORIGX1      1.000000  0.000000  0.000000        0.00000                         ");
//		this.BW.newLine();
//		this.BW
//				.write("ORIGX2      0.000000  1.000000  0.000000        0.00000                         ");
//		this.BW.newLine();
//		this.BW
//				.write("ORIGX3      0.000000  0.000000  1.000000        0.00000                         ");
//		this.BW.newLine();
//
//	}
//
//	private static MembraneModel getAlteredModel(final int domainID,
//			final int layerID) {
//		final MembraneModel model = DataPool.getMembraneModel();
//
//		final MembraneModel altered = new MembraneModel();
//
//		altered.setAlgorithmdescription(model.getAlgorithmDescription());
//		altered.setAlgorithmname(model.getAlgorithmName());
//		altered.setAlgorithmStringRepresentation(model
//				.getAlgorithmStringRepresentation());
//		altered.setAuthor(model.getAuthor());
//		altered.setCode(model.getCode());
//		altered.setDate(model.getDate());
//		altered.setExpdta(model.getEXPDTA());
//		altered.setHeader(model.getHeader());
//		altered.setJrnl(model.getJRNL());
//		altered.setKeywds(model.getKEYWDS());
//		altered.setRemark(model.getRemark());
//		altered.setSize(model.getSize());
//		altered.setDecimalPlaces(model.getDecimalPlaces());
//		altered.setTitle(model.getTitle());
//
//		for (int i = 0; i < model.getLayerCount(); i++) {
//			altered.addLayer(model.getLayer(i));
//		}
//		for (int i = 0; i < model.countMicroDomains(); i++) {
//			altered.addMicroDomain(model.getMicroDomain(i));
//		}
//
//		Molecule_3D[] mols = model.getAllProteinsInDomain(domainID, layerID);
//		for (int i = 0; i < mols.length; i++) {
//			altered.addProtein((Protein_3D) mols[i].copy(true, false), mols[i]
//					.getLayerID());
//		
//		}
//		mols = model.getAllLipidsExtraInDomain(domainID, layerID);
//		for (int i = 0; i < mols.length; i++) {
//			altered.addLipidExtra((Lipid_3D) mols[i].copy(true, false), mols[i]
//					.getDomainID(), mols[i].getLayerID());
//		}
//		mols = model.getAllLipidsIntraInDomain(domainID, layerID);
//		for (int i = 0; i < mols.length; i++) {
//			altered.addLipidIntra((Lipid_3D) mols[i].copy(true, false), mols[i]
//					.getDomainID(), mols[i].getLayerID());
//		}
//
//		for (int i = 0; i < model.getExampleLipidsCountExtra(); i++)
//			altered.addExampleLipidExtra(model.getExampleLipidAtExtra(i), model
//					.getExampleLipidAtExtra(i).getDomainID(), model
//					.getExampleLipidAtExtra(i).getLayerID());
//
//		for (int i = 0; i < model.getExampleLipidsCountIntra(); i++)
//			altered.addExampleLipidIntra(model.getExampleLipidAtIntra(i), model
//					.getExampleLipidAtIntra(i).getDomainID(), model
//					.getExampleLipidAtIntra(i).getLayerID());
//
//		if (layerID > 1) {
//			final float y = model.getLayerByID(layerID).getYHeight();
//			final Molecule_3D[] m = altered.getAllMoleculesInMembrane();
//			for (int i = 0; i < m.length; i++) {
//				m[i].setYTranslation(m[i].getYTranslation() - y);
//			}
//
//		}
//
//		
//		Molecule_3D[]mm=altered.getAllMoleculesInMembrane();
//		for(int i=0;i<mm.length;i++) {
//			mm[i].sortAtoms();
//		}
//		
//		for(int i=0;i<altered.getExampleLipidsCountExtra();i++) {
//			altered.getExampleLipidAtExtra(i).sortAtoms();
//		}
//		for(int i=0;i<altered.getExampleLipidsCountIntra();i++) {
//			altered.getExampleLipidAtIntra(i).sortAtoms();
//		}
//		return altered;
//	}
//
//	/**
//	 * Writes the master line at the end of the file
//	 * 
//	 * @throws IOException
//	 */
//	private void writeMaster() throws IOException {
//		String master = "MASTER";
//		master += Cm2_Toolkit
//				.extendLeft(this.getNumString(this.remcount, 9), 9);
//		master += Cm2_Toolkit.extendLeft("" + 0, 5);
//		master += Cm2_Toolkit.extendLeft(this.getNumString(
//				this.cont.het.size(), 5), 5);
//		master += Cm2_Toolkit.extendLeft(this.getNumString(this.cont.helix
//				.size(), 5), 5);
//		master += Cm2_Toolkit.extendLeft(this.getNumString(this.cont.sheet
//				.size(), 5), 5);
//		master += Cm2_Toolkit.extendLeft(this.getNumString(this.cont.turn
//				.size(), 5), 5);
//		master += Cm2_Toolkit.extendLeft(this.getNumString(this.cont.site
//				.size(), 5), 5);
//		master += Cm2_Toolkit.extendLeft("" + 0, 5);
//		master += Cm2_Toolkit.extendLeft(this.getNumString(
//				this.atomhetatmcount, 5), 5);
//		master += Cm2_Toolkit
//				.extendLeft(this.getNumString(this.tercount, 5), 5);
//		master += Cm2_Toolkit.extendLeft(this.getNumString(this.cont.conects
//				.size(), 5), 5);
//		master += Cm2_Toolkit.extendLeft(this.getNumString(this.cont.seqres
//				.size(), 5), 5);
//
//		this.BW.write(master);
//		this.BW.newLine();
//
//	}
//
//	/**
//	 * Converts a passed number into a right-side-adjusted string of ste
//	 * specified length If the string is longer than that, it will be cut of,
//	 * will all characters beeing a '9'
//	 * 
//	 * @param number
//	 * @param length
//	 * @return
//	 */
//	private String getNumString(final int number, final int length) {
//
//		String s = "" + number;
//		if (s.length() > length) {
//			s = "";
//			for (int i = 0; i < length; i++) {
//				s += "9";
//			}
//		} else
//			s = Cm2_Toolkit.extendLeft("" + s, length);
//		return s;
//	}
//
//	/**
//	 * Reading in the Conects from each file
//	 * @param dialog 
//	 * 
//	 * @throws IOException
//	 */
//	private void readInConects(StatusDialog dialog) throws IOException {
//
//
//		if (this.dialog != null)
//			this.dialog.setProgress("Reading Conects...", 45, true);
//		
//		int total = model.getProteinCount()+model.getLipidCountExtra()+model.getLipidCountIntra();
//		
//		float step = 15f/total;
//		float pos=45;
//		
//		
//		for (int i = 0; i < this.model.getProteinCount(); i++) {
//			
//			if (this.useSerials < this.NONE) {
//
//
//				this.cont.addConect(this.model.getProteinAt(i));
//				
//				if (this.dialog != null)
//					this.dialog.setProgress("Creating Connectivities...", (int)pos, true);
//				pos+=step;
//			}
//
//		}
//		for (int i = 0; i < this.model.getLipidCountExtra(); i++) {
//			if (this.useSerials < this.PROTSONLY) {
//
//
//				this.cont.addConect(this.model.getLipidAtExtra(i));
//				
//				if (this.dialog != null)
//					this.dialog.setProgress("Creating Connectivities...", (int)pos, true);
//				pos+=step;
//			}
//
//		}
//		for (int i = 0; i < this.model.getLipidCountIntra(); i++) {
//			if (this.useSerials < this.PROTSONLY) {
//
//
//				this.cont.addConect(this.model.getLipidAtIntra(i));
//				
//				if (this.dialog != null)
//					this.dialog.setProgress("Creating Connectivities...", (int)pos, true);
//				pos+=step;
//			}
//
//		}
//
//	}
//
//	/**
//	 * Generates the structures for each molecule
//	 * 
//	 * @throws IOException
//	 */
//	private void generateStructures() throws IOException {
//
//		float step = 8f / (float) this.model.getProteinCount();
//
//		float counter = 0f;
//
//		if (this.useChains < this.NONE) {
//
//			for (int i = 0; i < this.model.getProteinCount(); i++) {
//				if (this.dialog != null)
//					this.dialog.setProgress("Generating Structures",
//							(int) (counter * step) + 50, true);
//				this.cont.setStructure(this.model.getProteinAt(i), i, 0);
//				counter++;
//			}
//		} else {
//			for (int i = 0; i < this.model.getProteinCount(); i++) {
//				if (this.dialog != null)
//					this.dialog.setProgress("Generating Structures",
//							(int) (counter * step) + 50, true);
//				this.cont.removeChainsIDs(this.model.getProteinAt(i), i, 0);
//				counter++;
//			}
//		}
//		System.gc();
//
//		step = 7f / (float) (this.model.getLipidCountExtra() + this.model
//				.getLipidCountIntra());
//
//		counter = 0f;
//		if (this.useChains < this.PROTSONLY) {
//
//			for (int i = 0; i < this.model.getLipidCountExtra(); i++) {
//				if (this.dialog != null)
//					this.dialog.setProgress("Generating Structures",
//							(int) (counter * step) + 58, true);
//				this.cont.setStructure(this.model.getLipidAtExtra(i), i, 1);
//				counter++;
//			}
//
//			for (int i = 0; i < this.model.getLipidCountIntra(); i++) {
//				if (this.dialog != null)
//					this.dialog.setProgress("Generating Structures",
//							(int) (counter * step) + 58, true);
//				this.cont.setStructure(this.model.getLipidAtIntra(i), i, 2);
//				counter++;
//			}
//		} else {
//			for (int i = 0; i < this.model.getLipidCountExtra(); i++) {
//				if (this.dialog != null)
//					this.dialog.setProgress("Generating Structures",
//							(int) (counter * step) + 58, true);
//				this.cont.removeChainsIDs(this.model.getLipidAtExtra(i), i, 1);
//				counter++;
//			}
//
//			for (int i = 0; i < this.model.getLipidCountIntra(); i++) {
//				if (this.dialog != null)
//					this.dialog.setProgress("Generating Structures",
//							(int) (counter * step) + 58, true);
//				this.cont.removeChainsIDs(this.model.getLipidAtIntra(i), i, 2);
//				counter++;
//			}
//		}
//		System.gc();
//	}
//
//	public PDBWriter(final File file, final StatusDialog dialog) {
//		this.outputFile = file;
//		this.dialog = dialog;
//
//		this.useSerials = DataPool.LimitationSettings.atomserials;
//		this.useChains = DataPool.LimitationSettings.chains;
//		this.adoptTitleSections = DataPool.LimitationSettings.adoptTitleSections;
//		this.enableReverseParsing = DataPool.LimitationSettings.reverseParsing;
//
//	}
//
//	/**
//	 * Creates a string of each number, ensuring that it has at the proper
//	 * number of zeros behind the dot
//	 * 
//	 * @param x
//	 * @return
//	 */
//	private String createNumberString(final float x) {
//
//		String X = "" + x;
//		if (X.contains(".") == false) {
//			X += ".000";
//		} else {
//			final int indexof = X.indexOf(".");
//			if (indexof == X.length() - 1)
//				X += "000";
//			else if (indexof == X.length() - 2)
//				X += "00";
//			else if (indexof == X.length() - 3)
//				X += "0";
//			else if (indexof < X.length() - 4)
//				X = X.substring(0, indexof + 4);
//
//		}
//
//		return X;
//	}
//
//	/**
//	 * Closes the bufferedwriter from outside
//	 * 
//	 */
//	public void closeBufferedWriter() {
//		if (this.BW != null)
//			try {
//				this.BW.close();
//			} catch (final IOException e) {
//
//			}
//	}
//
//	/**
//	 * Writes the MetaInfo
//	 * 
//	 * @throws IOException
//	 */
//	private void writeHeader() throws IOException {
//
//		String firstone = "";
//
//		firstone += "HEADER    ";
//
//		String header = "";
//		if (this.model.getHeader() != null)
//			header = this.model.getHeader();
//		firstone += Cm2_Toolkit.extendRight(header, 40);
//
//		String date = "";
//		if (this.model.getDate() != null)
//			date = this.model.getDate();
//		firstone += Cm2_Toolkit.extendRight(date, 12);
//
//		String code = "";
//		if (this.model.getCode() != null)
//			code = this.model.getCode();
//		firstone += Cm2_Toolkit.extendRight(code, 4);
//		firstone += "    ";
//		this.BW.write(firstone.toUpperCase());
//
//		this.BW.newLine();
//
//		if (this.model.getTitle() != null) {
//			if (this.model.getTitle().length() > 0) {
//
//				final String[] strings = this.model.getTitle().split("\n");
//				for (int i = 0; i < strings.length; i++) {
//
//					String s = "";
//					s += "TITLE ";
//
//					s += strings[i];
//
//					this.BW.write(s.toUpperCase());
//					this.BW.newLine();
//
//				}
//
//			}
//		}
//
//		for (int i = 0; i < this.cont.compnd.size(); i++) {
//			this.BW.write(this.cont.compnd.get(i));
//			this.BW.newLine();
//		}
//
//		if (this.model.getSOURCE() != null) {
//			if (this.model.getSOURCE().length() > 0) {
//
//				final String[] strings = this.model.getSOURCE().split("\n");
//				for (int i = 0; i < strings.length; i++) {
//
//					String species = "";
//					species += "SOURCE";
//
//					species += strings[i];
//
//					this.BW.write(species.toUpperCase());
//					this.BW.newLine();
//
//					if (i > 9999)
//						break;
//
//				}
//
//			}
//		}
//		if (this.model.getKEYWDS() != null) {
//			if (this.model.getKEYWDS().length() > 0) {
//
//				final String[] strings = this.model.getKEYWDS().split("\n");
//				for (int i = 0; i < strings.length; i++) {
//
//					String s = "";
//					s += "KEYWDS ";
//
//					s += strings[i];
//
//					this.BW.write(s.toUpperCase());
//					this.BW.newLine();
//
//					if (i > 9999)
//						break;
//
//				}
//
//			}
//		}
//		if (this.model.getEXPDTA() != null) {
//			if (this.model.getEXPDTA().length() > 0) {
//
//				final String[] strings = this.model.getEXPDTA().split("\n");
//				for (int i = 0; i < strings.length; i++) {
//
//					String s = "";
//					s += "EXPDTA";
//
//					s += strings[i];
//
//					this.BW.write(s.toUpperCase());
//					this.BW.newLine();
//
//					if (i > 9999)
//						break;
//
//				}
//
//			}
//		}
//
//		if (this.model.getAuthor() != null) {
//			if (this.model.getAuthor().length() > 0) {
//
//				final String[] strings = this.model.getAuthor().split("\n");
//				for (int i = 0; i < strings.length; i++) {
//
//					String author = "";
//					author += "AUTHOR";
//
//					author += strings[i];
//
//					this.BW.write(author.toUpperCase());
//					this.BW.newLine();
//
//					if (i > 9999)
//						break;
//
//				}
//
//			}
//		}
//		if (this.model.getJRNL() != null) {
//			if (this.model.getJRNL().length() > 0) {
//
//				final String[] strings = this.model.getJRNL().split("\n");
//				for (int i = 0; i < strings.length; i++) {
//
//					String s = "";
//					s += "JRNL  ";
//
//					s += strings[i];
//
//					this.BW.write(s.toUpperCase());
//					this.BW.newLine();
//
//					if (i > 9999)
//						break;
//
//				}
//
//			}
//		}
//
//		final String remark = this.createAutoRemark();
//
//		final String[] strings = remark.split("\n");
//
//		for (int i = 0; i < strings.length; i++) {
//
//			String c = "";
//			c += "REMARK";
//
//			c += "    ";
//
//			c += strings[i];
//
//			this.BW.write(c);
//			this.BW.newLine();
//			this.remcount++;
//		}
//		if (this.cont.positionList.isEmpty() == false) {
//
//			this.BW.write("REMARK");
//			this.BW.newLine();
//			this.BW.write("REMARK");
//			this.BW.newLine();
//			this.BW
//					.write("REMARK    POSITIONS OF MOLECULE INSTANCES BY ATOM SERIALS");
//			this.BW.newLine();
//			this.BW.write("REMARK");
//			this.BW.newLine();
//			this.remcount += 4;
//
//			for (int i = 0; i < this.cont.positionList.size(); i++) {
//
//				String s = "";
//				s += "REMARK    ";
//
//				s += this.cont.positionList.get(i);
//
//				this.BW.write(s);
//				this.BW.newLine();
//				this.remcount++;
//
//			}
//
//		}
//
//		if (this.model.getRemark() != null) {
//			if (this.model.getRemark().length() > 0) {
//
//				this.BW.write("REMARK");
//				this.BW.newLine();
//
//				this.BW.write("REMARK    START OF USER DEFINED REMARKS");
//				this.BW.newLine();
//				this.BW.write("REMARK");
//				this.BW.newLine();
//				this.remcount += 3;
//
//				final String[] string = this.model.getRemark().split("\n");
//				for (int i = 0; i < string.length; i++) {
//
//					String s = "";
//					s += "REMARK";
//
//					s += string[i];
//
//					this.BW.write(s.toUpperCase());
//					this.BW.newLine();
//					this.remcount++;
//
//				}
//
//			}
//		}
//
//		if (this.xmlContents.isEmpty() == false) {
//			this.BW.write("REMARK");
//			this.BW.newLine();
//
//			this.BW.write("REMARK    COPY OF THE MEMBRANE MODEL BEGINS HERE");
//			this.BW.newLine();
//			this.BW.write("REMARK");
//			this.BW.newLine();
//			this.remcount += 3;
//			for (int i = 0; i < this.xmlContents.size(); i++) {
//				this.BW.write("REMARK " + this.xmlContents.get(i));
//				this.BW.newLine();
//				this.remcount++;
//			}
//
//			this.BW.write("REMARK    COPY OF THE MEMBRANE MODEL ENDS HERE");
//			this.BW.newLine();
//			this.BW.write("REMARK");
//			this.BW.newLine();
//			this.xmlContents.clear();
//
//			if (tHints != null) {
//				this.BW.write("REMARK    SHIFTXZ=" + tHints.shiftXZ);
//				this.BW.newLine();
//				this.BW.write("REMARK    SHIFTY=" + tHints.shiftY);
//				this.BW.newLine();
//				this.BW.write("REMARK    SWAPYZ=" + tHints.swapYZ);
//				this.BW.newLine();
//				this.BW.write("REMARK    SORTBYTYPE=" + tHints.sortByType);
//				this.BW.newLine();
//				this.BW.write("REMARK");
//				this.BW.newLine();
//			}
//
//			System.gc();
//		}
//
//		if (this.adopts.isEmpty() == false) {
//			this.BW.write("REMARK");
//			this.BW.newLine();
//
//			this.BW
//					.write("REMARK    SECTION OF ADOPTED TITLE SECTIONS BEGINS HERE");
//			this.BW.newLine();
//			this.BW.write("REMARK");
//			this.BW.newLine();
//			this.remcount += 3;
//			for (int i = 0; i < this.adopts.size(); i++) {
//				this.BW.write(this.adopts.get(i));
//				this.BW.newLine();
//				this.remcount++;
//			}
//			this.adopts.clear();
//			System.gc();
//		}
//	}
//
//	/**
//	 * Creates the auto remarks
//	 * 
//	 * @return
//	 * @throws IOException
//	 */
//	private String createAutoRemark() throws IOException {
//
//		String remark = "";
//
//		remark += "APPLICATION REMARKS:\n \n ";
//
//		remark += "BUILD WITH : " + CM2_CONSTANTS.GENERAL.NAME_OF_THIS_PROGRAM
//				+ "\n ";
//		remark += "             " + CM2_CONSTANTS.GENERAL.VERSION + "\n ";
//		remark += "MORE INFO  : " + CM2_CONSTANTS.GENERAL.CM2_HOMEPAGE
//				+ "\n \n";
//
//		if (this.model.getAlgorithmDescription() != null
//				|| this.model.getAlgorithmName() != null
//				|| this.model.getAlgorithmStringRepresentation() != null) {
//
//			remark += "ALGORITM USED FOR BILAYER CALCULATION  : " + "\n ";
//
//			if (this.model.getAlgorithmName() != null) {
//				remark += "NAME\n";
//				remark += this.model.getAlgorithmName();
//				remark += "\n\n";
//			}
//			if (this.model.getAlgorithmDescription() != null) {
//				remark += "DESCRIPTION\n";
//				remark += this.model.getAlgorithmDescription();
//				remark += "\n\n";
//			}
//			if (this.model.getAlgorithmStringRepresentation() != null) {
//				remark += "SETUP INFORMATION\n";
//				remark += this.model.getAlgorithmStringRepresentation();
//				remark += "\n\n";
//			}
//		}
//
//		if (this.enableReverseParsing)
//			this.storeMembraneModel();
//
//		final String[] allprots = this.model.getAllProteinTypes();
//		final String[] allEC = this.model.getAllLipidTypesInMembraneExtra();
//		final String[] allIC = this.model.getAllLipidTypesInMembraneIntra();
//
//		final Hashtable<String, String> hash = new Hashtable<String, String>();
//
//		for (int i = 0; i < allEC.length; i++)
//			hash.put(allEC[i], "");
//		for (int i = 0; i < allIC.length; i++)
//			hash.put(allIC[i], "");
//
//		final Enumeration<String> e = hash.keys();
//
//		remark += "INCLUDED AS PROTEINS: \n \n";
//
//		if (allprots.length > 0) {
//			remark += "CODE                   HEADER                   COUNT \n";
//
//			for (int i = 0; i < allprots.length; i++) {
//				String line = "";
//				final Protein_3D[] prots = this.model
//						.getProteinsByPDBCode(allprots[i]);
//
//				if (prots.length == 0)
//					continue;
//
//				final PdbAttributes info = new PdbReader_3D()
//						.readAttributes(DataPool.getRawPdbData(prots[0]
//								.getPDBFile().getAbsolutePath()));
//				if (info.getCode() != null) {
//					line += Cm2_Toolkit.extendRight(info.getCode(), 4);
//				} else {
//					line += " ?? ";
//				}
//
//				line += "  ";
//				if (info.getHeader() != null) {
//					line += Cm2_Toolkit.extendRight(info.getHeader(), 40);
//				} else {
//					line += "                   ??                   ";
//				}
//
//				line += "  ";
//
//				final int count = prots.length;
//
//				if (count < 100000) {
//					line += Cm2_Toolkit.extendLeft("" + count, 5);
//				} else {
//					line += "99999+";
//				}
//
//				remark += line + "\n";
//
//				if (this.adoptTitleSections)
//					this.adoptTitleSection(prots[0]);
//			}
//		} else {
//			remark += "None";
//		}
//		remark += " \n \n";
//
//		remark += "INCLUDED AS LIPIDS: \n \n";
//		if (e.hasMoreElements()) {
//			remark += "CODE                   HEADER                   COUNT \n";
//
//			while (e.hasMoreElements()) {
//				String line = "";
//				final String key = e.nextElement();
//				Lipid_3D[] lips = this.model.getLipidsByPDBCodeExtra(key);
//				final Lipid_3D[] lips2 = this.model
//						.getLipidsByPDBCodeIntra(key);
//
//				final int count = lips.length + lips2.length;
//				if (lips.length == 0)
//					lips = lips2;
//
//				if (lips.length == 0)
//					continue;
//
//				final PdbAttributes info = new PdbReader_3D()
//						.readAttributes(DataPool.getRawPdbData(lips[0]
//								.getPDBFile().getAbsolutePath()));
//				if (info.getCode() != null) {
//					line += Cm2_Toolkit.extendRight(info.getCode(), 4);
//				} else {
//					line += " ?? ";
//				}
//
//				line += "  ";
//				if (info.getHeader() != null) {
//					line += Cm2_Toolkit.extendRight(info.getHeader(), 40);
//				} else {
//					line += "                   ??                   ";
//				}
//
//				line += "  ";
//
//				if (count < 100000) {
//					line += Cm2_Toolkit.extendLeft("" + count, 5);
//				} else {
//					line += "99999+";
//				}
//				remark += line + "\n";
//
//				if (this.adoptTitleSections)
//					this.adoptTitleSection(lips[0]);
//			}
//
//		} else {
//			remark += "None";
//		}
//
//		remark += " \n \n";
//
//		return remark;
//	}
//
//	/**
//	 * Copies part of the title section from a molecule's file
//	 * 
//	 * @param mol
//	 */
//	private void adoptTitleSection(final Molecule_3D mol) {
//
//		String top;
//		if (mol instanceof Protein_3D)
//			top = "REMARK MEMBER " + mol.getPDBcode() + " ";
//		else
//			top = "REMARK MEMBER " + mol.getPDBcode() + " ";
//
//		final PdbReader_3D reader = new PdbReader_3D();
//		try {
//			String[] s = reader.readSection(mol.getPDBFile(), "TITLE ");
//			for (int i = 0; i < s.length; i++) {
//				this.adopts.addElement(top + s[i]);
//			}
//			s = reader.readSection(mol.getPDBFile(), "COMPND");
//			for (int i = 0; i < s.length; i++) {
//				this.adopts.addElement(top + s[i]);
//			}
//			s = reader.readSection(mol.getPDBFile(), "SOURCE");
//			for (int i = 0; i < s.length; i++) {
//				this.adopts.addElement(top + s[i]);
//			}
//			s = reader.readSection(mol.getPDBFile(), "KEYWDS");
//			for (int i = 0; i < s.length; i++) {
//				this.adopts.addElement(top + s[i]);
//			}
//			s = reader.readSection(mol.getPDBFile(), "EXPDTA");
//			for (int i = 0; i < s.length; i++) {
//				this.adopts.addElement(top + s[i]);
//			}
//			s = reader.readSection(mol.getPDBFile(), "AUTHOR");
//			for (int i = 0; i < s.length; i++) {
//				this.adopts.addElement(top + s[i]);
//			}
//			s = reader.readSection(mol.getPDBFile(), "JRNL  ");
//			for (int i = 0; i < s.length; i++) {
//				this.adopts.addElement(top + s[i]);
//			}
//			s = reader.readSection(mol.getPDBFile(), "REMARK");
//			for (int i = 0; i < s.length; i++) {
//				this.adopts.addElement(top + s[i]);
//			}
//			s = reader.readSection(mol.getPDBFile(), "FORMUL");
//			for (int i = 0; i < s.length; i++) {
//				this.adopts.addElement(top + s[i]);
//			}
//		} catch (final IOException e) {
//			return;
//		}
//	}
//
//	private void storeMembraneModel() {
//
//		final File temp = new File(FileIOHandler.home + "tmp.xml");
//
//		this.model.translateAllMolecules(-DataPool.getCurrentLayerHeight(),
//				false, false);
//
//		try {
//			new MembraneXmlWriter(DataPool.workspace).writeMembrane(temp,
//					this.model);
//
//			final BufferedReader br = new BufferedReader(new FileReader(temp));
//
//			while (true) {
//				final String s = br.readLine();
//
//				if (s == null)
//					break;
//
//				this.xmlContents.addElement(s);
//			}
//
//			br.close();
//
//		} catch (final IOException e) {
//
//			e.printStackTrace();
//		}
//		this.model.translateAllMolecules(DataPool.getCurrentLayerHeight(),
//				false, false);
//		temp.deleteOnExit();
//		temp.delete();
//	}
//
//	/**
//	 * 
//	 * A class representing an atom line from a pdb file
//	 * 
//	 * @author Tim Dingersen dackelnose@gmx.net
//	 * 
//	 */
//	@SuppressWarnings("unchecked")
//	class Line implements Comparable{
//
//		public String recordName = "";
//
//		// public int serial = -1;
//
//		public Point3f position = new Point3f();
//
//		public String name = "";
//
//		public String altLoc = "";
//
//		public String residueName = "";
//
//		public String chainId = "";
//
//		public String resSeq = "";
//
//		public String iCode = "";
//
//		public String occupancy = "";
//
//		public String tempFactor = "";
//
//		public String segID = "";
//
//		public String element = "";
//
//		public String charge = "";
//
//		public int domainID = 0;
//
//		public Atom_3D atomOfMol;
//
//		private int comparator=-1;
//
//		public Line(final Atom_3D atomOfMolecule, final String line,
//				final int domainID) {
//
//			if (line == null) {
//
//				return;
//			}
//
//			this.domainID = domainID;
//			this.atomOfMol = atomOfMolecule;
//			try {
//				this.recordName = line.substring(0, 7).trim().toUpperCase();
//
//			} catch (final Exception e) {
//			}
//			// try {
//			// final String id = line.substring(7, 11);
//			// this.serial = Integer.parseInt(id.trim());
//			// } catch (final Exception e) {
//			// }
//
//			if (atomOfMolecule == null) {
//				try {
//					final String sx = line.substring(31, 38);
//					final String sy = line.substring(39, 46);
//					final String sz = line.substring(47, 54);
//
//					final float x = Float.parseFloat(sx.trim());
//					final float y = Float.parseFloat(sy.trim());
//					final float z = Float.parseFloat(sz.trim());
//
//					this.position = new Point3f(x, y, z);
//
//				} catch (final Exception e) {
//
//				}
//			} else
//				this.position = new Point3f(atomOfMolecule.position);
//
//			try {
//				this.name = line.substring(11, 16);
//			} catch (final Exception e) {
//
//			}
//
//			try {
//				this.altLoc = line.substring(16, 17).trim();
//			} catch (final Exception e) {
//
//			}
//			try {
//				this.residueName = line.substring(17, 20).trim();
//			} catch (final Exception e) {
//
//			}
//			try {
//				this.chainId = line.substring(21, 22).trim();
//			} catch (final Exception e) {
//
//			}
//			try {
//				this.resSeq = line.substring(22, 26).trim();
//			} catch (final Exception e) {
//
//			}
//			try {
//				this.iCode = line.substring(26, 27).trim();
//			} catch (final Exception e) {
//
//			}
//			try {
//				this.occupancy = line.substring(55, 60).trim();
//			} catch (final Exception e) {
//
//			}
//			try {
//				this.tempFactor = line.substring(61, 66).trim();
//			} catch (final Exception e) {
//
//			}
//			try {
//				this.segID = line.substring(73, 76).trim();
//			} catch (final Exception e) {
//
//			}
//			try {
//				if(this.atomOfMol!=null)this.element=this.atomOfMol.element;
//				this.element = line.substring(76, 78).trim();
//			} catch (final Exception e) {
//
//			}
//			try {
//				this.charge = line.substring(78, 80).trim();
//			} catch (final Exception e) {
//
//			}
//
//		}
//		public int compareTo(Object arg0) {
//			if(arg0 instanceof Line ==false) {
//				return 0;
//			}
//			else {
//				
//				int c = ((Line)arg0).comparator;
//				if(this.comparator<c)return -1;
//				else if(this.comparator>c)return 1;
//				else {
//					
//					System.out.println("Comparable: Same Comparator");
//					System.out.println(this.name+"  "+((Line)arg0).name);
//					return 0;
//				}
//
//			}
//		}
//	}
//
//	/**
//	 * Stores and organizes all contents that has to be written later. Gets
//	 * filled during the collection of data process
//	 * 
//	 * @author Tim Dingersen dackelnose@gmx.net
//	 * 
//	 */
//	class Content {
//
//		int currentPos = 0;
//
//		int listPos = 1;
//
//		Hashtable<String, String> IDs = new Hashtable<String, String>();
//
//		Vector<Line> list = new Vector<Line>();
//
//		Vector<Line> objects = new Vector<Line>();
//
//		Vector<Line[]> contents = new Vector<Line[]>();
//
//		Vector<String> conects = new Vector<String>();
//
//		Line[][] all;
//
//		Line[] all2;
//
//		Vector<String> seqres = new Vector<String>();
//
//		Vector<String> helix = new Vector<String>();
//
//		Vector<String> sheet = new Vector<String>();
//
//		Vector<String> turn = new Vector<String>();
//
//		Vector<String> ssbond = new Vector<String>();
//
//		Vector<String> site = new Vector<String>();
//
//		Vector<String> dbref = new Vector<String>();
//
//		Vector<String> seqadv = new Vector<String>();
//
//		Vector<String> modres = new Vector<String>();
//
//		Vector<String> het = new Vector<String>();
//
//		Vector<String> link = new Vector<String>();
//
//		Vector<String> cispep = new Vector<String>();
//
//		Vector<String> positionList = new Vector<String>();
//
////		public Vector<Atom> forConects = new Vector<Atom>();
//
//		public Vector<String> compnd = new Vector<String>();
//
//		public void add(final Line line) {
//			this.objects.addElement(line);
//			this.list.addElement(line);
//			if(line.atomOfMol!=null)line.atomOfMol.indexForPDBWriter=list.size();
//		}
//
//		public void finishMolecule(final Molecule_3D mol) {
//			this.contents.addElement((this.objects.toArray(new Line[0]))
//					.clone());
//			this.objects.clear();
//
//			if (PDBWriter.this.listPositions) {
//				if (PDBWriter.this.useSerials == PDBWriter.this.NONE
//						&& mol instanceof Protein_3D)
//					return;
//				else if (PDBWriter.this.useSerials > PDBWriter.this.BOTH
//						&& mol instanceof Lipid_3D)
//					return;
//
//				String out = "";
//
//				out += mol instanceof Protein_3D ? "PROTEIN" : "LIPID  ";
//				out += "  ";
//				out += Cm2_Toolkit.extendRight(mol.getPDBcode(), 4);
//				out += "   START:  "
//						+ Cm2_Toolkit.extendLeft("" + this.listPos, 5);
//				out += "   END:  "
//						+ Cm2_Toolkit.extendLeft("" + this.list.size(), 5);
//				out += "   LENGTH:  "
//						+ Cm2_Toolkit.extendLeft(""
//								+ (this.list.size() - this.listPos - 1), 5);
//
//				this.positionList.addElement(out);
//				this.listPos = this.list.size() + 1;
//			}
//		}
//
//		public void finish() {
//			this.all = this.contents.toArray(new Line[0][0]);
//			this.all2 = this.list.toArray(new Line[0]);
//			this.objects.clear();
//			this.contents.clear();
//
//		}
//
//		public Line[] getAll(final int index, final int group) {
//
//			int a = 0;
//			if (group > 0)
//				a += PDBWriter.this.model.getProteinCount();
//			if (group == 2)
//				a += PDBWriter.this.model.getLipidCountExtra();
//			a += index;
//			return this.all[a];
//		}
//
//		public Line getAt(final int index) {
//
//			return this.all2[index];
//		}
//
//		public int getLength() {
//
//			return this.all2.length;
//		}
//
//
//
//		public void addConect(Molecule_3D mol) {
//
//			for (int i = 0; i < mol.atoms.length; i++) {
//
//				int[] sorted = new int[mol.atoms[i].currentNeighbors.size()];
//
//				for (int j = 0; j < sorted.length; j++) {
//					sorted[j] = mol.atoms[i].currentNeighbors.get(j).indexForPDBWriter.intValue();
//					
//				}
//				Arrays.sort(sorted);
//				Vector<Integer> sortedSerials = new Vector<Integer>();
//				for (int j = 0; j < sorted.length; j++) {
//					sortedSerials.addElement(sorted[j]);
//				}
//				Enumeration<Integer> en = sortedSerials.elements();
//				
//				while (en.hasMoreElements()) {
//					
//					String string = "";
//
//					string = "CONECT";
//					int refInd = mol.atoms[i].indexForPDBWriter.intValue();
//					string += Cm2_Toolkit.extendLeft("" + refInd, 5);
//					
//					while (true) {
//						
//						if (!en.hasMoreElements()) {
//							this.conects.addElement("" + string);
//							
//							break;
//						} else {
//							int serial = en.nextElement().intValue();
//							string += Cm2_Toolkit.extendLeft("" + serial, 5);
//						}
//
//					}
//
//				}
//
//			}
//		}
//
//		public void setStructure(final Molecule_3D mol, final int molnumber,
//				final int group) throws IOException {
//
//			this.IDs = new Hashtable<String, String>();
//
//			final Vector<String> chars2use = new Vector<String>();
//			chars2use.setSize(Cm2_Toolkit.CHARS.length);
//
//			for (int i = 0; i < mol.getChainIDs().length; i++) {
//				final char s = mol.getChainIDs()[i].charAt(0);
//				final int pos = Cm2_Toolkit.getCharacterPosOf(s);
//				chars2use.set(pos, mol.getChainIDs()[i]);
//
//			}
//
//			int c = 0;
//			for (int i = 0; i < chars2use.size(); i++) {
//				if (chars2use.get(i) == null)
//					continue;
//
//				this.IDs.put(chars2use.get(i), "" + this.getCHARAt(c));
//				c++;
//			}
//
//			final Line[] lines = this.getAll(molnumber, group);
//			for (int a = 0; a < lines.length; a++) {
//
//				final Line current = lines[a];
//
//				if (current != null) {
//
//					if (mol.getChainCount() < 1) {
//						current.chainId = "";
//
//						continue;
//					}
//
//					if (current.chainId.length() < 1) {
//						current.chainId = "";
//
//						continue;
//					}
//
//					current.chainId = this.getCharFor(current.chainId);
//				}
//			}
//
//			String[] input = new PdbReader_3D().readSection(mol.getPDBFile(),
//					"SEQRES");
//
//			for (int i = 0; i < input.length; i++) {
//
//				String current = input[i];
//
//				try {
//					final String orig = "" + current.charAt(11);
//
//					current = current.substring(0, 11) + this.getCharFor(orig)
//							+ current.substring(12);
//
//					this.seqres.addElement(current);
//				} catch (final Exception e) {
//
//				}
//
//			}
//
//			input = new PdbReader_3D().readSection(mol.getPDBFile(), "HELIX");
//
//			for (int i = 0; i < input.length; i++) {
//				String current = input[i];
//
//				try {
//
//					current = current.substring(0, 6)
//							+ Cm2_Toolkit.extendLeft(""
//									+ (this.helix.size() + 1), 4)
//							+ current.substring(10);
//
//					final String orig = "" + current.charAt(19);
//
//					current = current.substring(0, 19) + this.getCharFor(orig)
//							+ current.substring(20);
//					current = current.substring(0, 31) + this.getCharFor(orig)
//							+ current.substring(32);
//
//					this.helix.addElement(current);
//
//				} catch (final Exception e) {
//
//				}
//
//			}
//
//			input = new PdbReader_3D().readSection(mol.getPDBFile(), "SHEET");
//
//			for (int i = 0; i < input.length; i++) {
//				String current = input[i];
//
//				try {
//
//					final String s = "" + current.charAt(21);
//					current = current.substring(0, 21) + this.getCharFor(s)
//							+ current.substring(22);
//				} catch (final Exception e) {
//
//				}
//
//				try {
//
//					final String s = "" + current.charAt(32);
//					current = current.substring(0, 32) + this.getCharFor(s)
//							+ current.substring(33);
//
//				} catch (final Exception e) {
//
//				}
//				try {
//
//					final String s = "" + current.charAt(49);
//					current = current.substring(0, 49) + this.getCharFor(s)
//							+ current.substring(50);
//
//				} catch (final Exception e) {
//
//				}
//				try {
//
//					final String s = "" + current.charAt(64);
//					current = current.substring(0, 64) + this.getCharFor(s)
//							+ current.substring(65);
//
//				} catch (final Exception e) {
//
//				}
//
//				this.sheet.addElement(current);
//			}
//
//			input = new PdbReader_3D().readSection(mol.getPDBFile(), "TURN");
//
//			for (int i = 0; i < input.length; i++) {
//				String current = input[i];
//
//				try {
//
//					current = current.substring(0, 6)
//							+ Cm2_Toolkit.extendLeft(""
//									+ (this.turn.size() + 1), 4)
//							+ current.substring(10);
//
//					String orig = "" + current.charAt(19);
//
//					current = current.substring(0, 19) + this.getCharFor(orig)
//							+ current.substring(20);
//
//					orig = "" + current.charAt(30);
//
//					current = current.substring(0, 30) + this.getCharFor(orig)
//							+ current.substring(31);
//
//					this.turn.addElement(current);
//
//				} catch (final Exception e) {
//
//				}
//
//			}
//
//			input = new PdbReader_3D().readSection(mol.getPDBFile(), "SSBOND");
//
//			for (int i = 0; i < input.length; i++) {
//				String current = input[i];
//
//				try {
//
//					final String s = "" + current.charAt(15);
//					current = current.substring(0, 15) + this.getCharFor(s)
//							+ current.substring(16);
//					current = current.substring(0, 6)
//							+ Cm2_Toolkit.extendLeft(""
//									+ (this.ssbond.size() + 1), 4)
//							+ current.substring(10);
//				} catch (final Exception e) {
//					continue;
//				}
//
//				try {
//
//					final String s = "" + current.charAt(29);
//					current = current.substring(0, 29) + this.getCharFor(s)
//							+ current.substring(30);
//
//				} catch (final Exception e) {
//					continue;
//				}
//
//				this.ssbond.addElement(current);
//
//			}
//
//			input = new PdbReader_3D().readSection(mol.getPDBFile(), "SITE");
//
//			for (int i = 0; i < input.length; i++) {
//				String current = input[i];
//
//				try {
//
//					final String s = "" + current.charAt(22);
//					current = current.substring(0, 22) + this.getCharFor(s)
//							+ current.substring(23);
//				} catch (final Exception e) {
//
//				}
//
//				try {
//
//					final String s = "" + current.charAt(33);
//					current = current.substring(0, 33) + this.getCharFor(s)
//							+ current.substring(34);
//
//				} catch (final Exception e) {
//
//				}
//				try {
//
//					final String s = "" + current.charAt(44);
//					current = current.substring(0, 44) + this.getCharFor(s)
//							+ current.substring(45);
//
//				} catch (final Exception e) {
//
//				}
//				try {
//
//					final String s = "" + current.charAt(55);
//					current = current.substring(0, 55) + this.getCharFor(s)
//							+ current.substring(56);
//
//				} catch (final Exception e) {
//
//				}
//
//				this.site.addElement(current);
//			}
//
//			input = new PdbReader_3D().readSection(mol.getPDBFile(), "DBREF");
//
//			for (int i = 0; i < input.length; i++) {
//				String current = input[i];
//
//				try {
//
//					final String orig = "" + current.charAt(12);
//
//					current = current.substring(0, 12) + this.getCharFor(orig)
//							+ current.substring(13);
//
//					this.dbref.addElement(current);
//
//				} catch (final Exception e) {
//
//				}
//
//			}
//
//			input = new PdbReader_3D().readSection(mol.getPDBFile(), "SEQADV");
//
//			for (int i = 0; i < input.length; i++) {
//				String current = input[i];
//
//				try {
//
//					final String orig = "" + current.charAt(16);
//
//					current = current.substring(0, 16) + this.getCharFor(orig)
//							+ current.substring(17);
//
//					this.seqadv.addElement(current);
//
//				} catch (final Exception e) {
//
//				}
//
//			}
//			input = new PdbReader_3D().readSection(mol.getPDBFile(), "MODRES");
//
//			for (int i = 0; i < input.length; i++) {
//				String current = input[i];
//
//				try {
//
//					final String orig = "" + current.charAt(16);
//
//					current = current.substring(0, 16) + this.getCharFor(orig)
//							+ current.substring(17);
//
//					this.modres.addElement(current);
//
//				} catch (final Exception e) {
//
//				}
//
//			}
//			input = new PdbReader_3D().readSection(mol.getPDBFile(), "HET");
//
//			for (int i = 0; i < input.length; i++) {
//				String current = input[i];
//
//				try {
//
//					final String orig = "" + current.charAt(12);
//
//					current = current.substring(0, 12) + this.getCharFor(orig)
//							+ current.substring(13);
//
//					this.het.addElement(current);
//
//				} catch (final Exception e) {
//
//				}
//
//			}
//			input = new PdbReader_3D().readSection(mol.getPDBFile(), "LINK");
//
//			for (int i = 0; i < input.length; i++) {
//				String current = input[i];
//
//				try {
//
//					String orig = "" + current.charAt(21);
//
//					current = current.substring(0, 21) + this.getCharFor(orig)
//							+ current.substring(22);
//
//					orig = "" + current.charAt(51);
//
//					current = current.substring(0, 51) + this.getCharFor(orig)
//							+ current.substring(52);
//
//					this.link.addElement(current);
//
//				} catch (final Exception e) {
//
//				}
//
//			}
//			input = new PdbReader_3D().readSection(mol.getPDBFile(), "CISPEP");
//
//			for (int i = 0; i < input.length; i++) {
//				String current = input[i];
//
//				try {
//
//					current = current.substring(0, 6)
//							+ Cm2_Toolkit.extendLeft(""
//									+ (this.cispep.size() + 1), 4)
//							+ current.substring(10);
//
//					String orig = "" + current.charAt(15);
//
//					current = current.substring(0, 15) + this.getCharFor(orig)
//							+ current.substring(16);
//
//					orig = "" + current.charAt(29);
//
//					current = current.substring(0, 29) + this.getCharFor(orig)
//							+ current.substring(30);
//
//					this.cispep.addElement(current);
//
//				} catch (final Exception e) {
//
//				}
//
//			}
//
//			if (mol.getChainCount() > 0) {
//				String n = "    ";
//				if (this.compnd.size() > 1)
//					n = Cm2_Toolkit.extendLeft("" + this.compnd.size(), 4)
//							+ " ";
//				this.compnd.add("COMPND"
//						+ n
//						+ "MOL_ID:"
//						+ Cm2_Toolkit.extendLeft(""
//								+ ((this.compnd.size() / 3) + 1), 2) + ";");
//
//				n = Cm2_Toolkit.extendLeft("" + this.compnd.size(), 4) + " ";
//
//				this.compnd.add("COMPND" + n + "MOLECULE: "
//						+ mol.getHeader().trim() + ";");
//
//				n = Cm2_Toolkit.extendLeft("" + this.compnd.size(), 4) + " ";
//
//				String chains = "";
//
//				final String[] ids = mol.getChainIDs();
//
//				for (int i = 0; i < ids.length; i++) {
//
//					final char ch = Cm2_Toolkit.CHARS[this.currentPos + i];
//
//					chains += ch;
//					if (i < ids.length - 1)
//						chains += ", ";
//				}
//
//				this.compnd.add("COMPND" + n + "CHAIN: " + chains + ";");
//			}
//
//			this.currentPos += mol.getChainCount();
//
//		}
//
//		public void removeChainsIDs(final Molecule_3D mol, final int molnumber,
//				final int group) {
//			final Line[] lines = this.getAll(molnumber, group);
//			for (int a = 0; a < lines.length; a++) {
//
//				lines[a].chainId = "";
//
//			}
//
//		}
//
//		private String getCHARAt(final int i) {
//			try {
//				return "" + Cm2_Toolkit.CHARS[this.currentPos + i];
//			} catch (final Exception e) {
//				return " ";
//			}
//		}
//
//		private String getCharFor(final String s) {
//			try {
//				final String ret = this.IDs.get(s);
//				return ret != null ? ret : " ";
//			} catch (final Exception e) {
//				return " ";
//			}
//		}
//
//	}
//
//	public static boolean coordinatesWouldExceedValidNumberRangeCurrently(
//			int domainID, int layerID, int shiftXZ, int shiftY, boolean swapYZ,
//			boolean saveOPMDummyLayersInPDB) {
//
//		MembraneModel model = getAlteredModel(domainID, layerID);
//
//		float maxx = -1000000, maxy = -1000000, maxz = -1000000, minx = 1000000, miny = 1000000, minz = 1000000;
//
//		final Molecule_3D[] mols = model.getAllMoleculesInMembrane();
//
//		for (int i = 0; i < mols.length; i++) {
//
//			Atom_3D[] atoms = mols[i].getAtoms(true, false);
//
//			for (int j = 0; j < atoms.length; j++) {
//
//				final Point3f p = atoms[j].position;
//				minx = Math.min(minx, p.x);
//				miny = Math.min(miny, p.y);
//				minz = Math.min(minz, p.z);
//
//				maxx = Math.max(maxx, p.x);
//				maxy = Math.max(maxy, p.y);
//				maxz = Math.max(maxz, p.z);
//
//			}
//		}
//
//		float subX = 0;
//		float subZ = 0;
//		float subY = 0;
//		if (shiftXZ == 2) {
//			subX += model.getSize().width / 2;
//			subZ += model.getSize().height / 2;
//
//		} else if (shiftXZ == 1) {
//			subX = minx + ((maxx - minx) / 2);
//			subZ = minz + ((maxz - minz) / 2);
//		}
//
//		if (shiftY == 2) {
//			subY = miny;
//		} else if (shiftY == 1) {
//			subY = miny + ((maxy - miny) / 2);
//		}
//
//		Vector<Atom_3D> finalAtoms = new Vector<Atom_3D>();
//		for (int i = 0; i < mols.length; i++) {
//			Point3f p = new Point3f(mols[i].getPosition());
//			p.sub(new Point3f(subX, subY, subZ));
//			mols[i].setPosition(p);
//
//			Atom_3D[] atoms = mols[i].getAtoms(true, true);
//
//			for (int j = 0; j < atoms.length; j++) {
//				finalAtoms.addElement(atoms[j]);
//			}
//
//			if (!saveOPMDummyLayersInPDB)
//				continue;
//
//			for (int j = 0; j < mols[i].opmAtoms.length; j++) {
//				Atom_3D a = mols[i].opmAtoms[j].copy();
//				a.position = Cm2_Toolkit.transformToNewPoint(mols[i]
//						.getRotationMatrix(), a.position);
//				a.position.sub(new Point3f(subX, subY, subZ));
//				finalAtoms.addElement(a);
//			}
//
//		}
//
//		for (int i = 0; i < finalAtoms.size(); i++) {
//			Atom_3D a = finalAtoms.get(i);
//			if ((a.position.x < -999.999f || a.position.x > 9999.999f)
//					|| (a.position.y < -999.999f || a.position.y > 9999.999f)
//					|| (a.position.z < -999.999f || a.position.z > 9999.999f))
//				return true;
//		}
//
//		return false;
//	}
//
//}
